Type: Package
Package: GenomeAdmixR
Title: Simulate Admixture of Genomes
Version: 2.1.13.2
Authors@R: 
    c(person(given = "Thijs",
             family = "Janzen",
             role = c("aut", "cre"),
             email = "thijsjanzen@gmail.com"),
      person(given = "Fernando",
             family = "Diaz G.",
             role = "ctb",
             email = "ferdiazfer@gmail.com"),
      person(given = "Richèl J.C.",
             family = "Bilderbeek",
             role = "ctb"))
Description: Individual-based simulations forward in time,
    simulating how patterns in ancestry along the genome change after
    admixture. Full description can be found in Janzen (2021)
    <doi:10.1111/2041-210X.13612>.
License: GPL (>= 2)
URL: https://github.com/thijsjanzen/GenomeAdmixR
BugReports: https://github.com/thijsjanzen/GenomeAdmixR/issues
Imports: ggplot2, ggridges, hierfstat, Rcpp, RcppParallel, rlang,
        stringr, tibble, vcfR
Suggests: dplyr, junctions, knitr, magrittr, rmarkdown, testit,
        testthat, pbapply
LinkingTo: Rcpp, RcppArmadillo, RcppParallel
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 2.10)
SystemRequirements: C++17
NeedsCompilation: yes
Packaged: 2026-05-13 15:19:33 UTC; thijsjanzen
Author: Thijs Janzen [aut, cre],
  Fernando Diaz G. [ctb],
  Richèl J.C. Bilderbeek [ctb]
Maintainer: Thijs Janzen <thijsjanzen@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-14 14:10:28 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-14 15:02:07 UTC; unix
Archs: GenomeAdmixR.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,individual)
S3method(print,genomeadmixr_data)
S3method(print,individual)
S3method(print,population)
export(ancestry_module)
export(calculate_allele_frequencies)
export(calculate_average_ld)
export(calculate_dist_junctions)
export(calculate_fst)
export(calculate_heterozygosity)
export(calculate_ld)
export(calculate_marker_frequency)
export(combine_input_data)
export(create_artificial_genomeadmixr_data)
export(create_iso_female)
export(default_params_doc)
export(iso_female_ancestry)
export(iso_female_sequence)
export(load_population)
export(migration_settings)
export(plink_to_genomeadmixr_data)
export(plot_chromosome)
export(plot_difference_frequencies)
export(plot_dist_junctions)
export(plot_frequencies)
export(plot_joyplot_frequencies)
export(plot_over_time)
export(plot_start_end)
export(read_input_data)
export(save_population)
export(sequence_module)
export(simulate_admixture)
export(simulate_ancestry_migration)
export(simulate_sequence_migration)
export(simulation_data_to_genomeadmixr_data)
export(vcfR_to_genomeadmixr_data)
export(write_as_plink)
import(Rcpp)
importFrom(RcppParallel, RcppParallelLibs)
importFrom(rlang,.data)
useDynLib(GenomeAdmixR, .registration = TRUE)

# GenomeAdmixR  <img src="man/figures/GenomeAdmixR.png" align="right" width="180" />

[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/GenomeAdmixR)](https://cran.r-project.org/package=GenomeAdmixR)
[![](http://cranlogs.r-pkg.org/badges/grand-total/GenomeAdmixR)](https://cran.r-project.org/package=GenomeAdmixR)
[![](http://cranlogs.r-pkg.org/badges/GenomeAdmixR)](https://cran.r-project.org/package=GenomeAdmixR)


| Branch     | [![GitHub Actions logo](man/figures/github_actions_logo.png)](https://github.com/features/actions)                                                   | [![Codecov logo](man/figures/Codecov.png)](https://about.codecov.io/)                                                                                |
|------------|------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------|
|   `master` | [![R-CMD-check](https://github.com/thijsjanzen/GenomeAdmixR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/thijsjanzen/GenomeAdmixR/actions/workflows/R-CMD-check.yaml)                | [![codecov.io](https://codecov.io/gh/thijsjanzen/GenomeAdmixR/branch/master/graph/badge.svg)](https://app.codecov.io/github/thijsjanzen/GenomeAdmixR/branch/master)   |
|  `develop` | [![R-CMD-check](https://github.com/thijsjanzen/GenomeAdmixR/actions/workflows/R-CMD-check.yaml/badge.svg?branch=develop)](https://github.com/thijsjanzen/GenomeAdmixR/actions/workflows/R-CMD-check.yaml) | [![codecov.io](https://codecov.io/gh/thijsjanzen/GenomeAdmixR/branch/develop/graph/badge.svg)](https://app.codecov.io/github/thijsjanzen/GenomeAdmixR/branch/develop) |

# What is GenomeAdmixR?
A package to perform individual-based simulations of genome evolution with ancestry.

# More information
More information can be found in the accompanying paper :  https://doi.org/10.1111/2041-210X.13612

# Version history
Version 2.1.13.2 - Fixed weird behaviour with num_threads and TBB, now embedded parallel code in task_arena <br />
Version 2.1.13 - Added support for installs without TBB (e.g. Alpine Linux) <br />
Version 2.1.12 - Update to C++17 <br />
Version 2.1.11 - Update internal seeding <br />
Version 2.1.10 - Fixed memory bug, improved documentation <br />
Version 2.1.9  - updated tbb::task_scheduler_init to tbb::global_control <br />
Version 2.1.7  - Improve documentation <br />
Version 2.1.6  - check classes with inherits <br />
Version 2.1.5  - Removed debugging output <br />
Version 2.1.4  - Only output when verbose = TRUE <br />
Version 2.1.3  - Changed DOI link in description <br />
Version 2.1.2  - Improved testing <br />
Version 2.1.1  - Removed GNU make dependency <br />
Version 2.1    - Removed error in calculate_allele_frequency <br />
Version 2.0.1 - Moved migration outside the modules <br />
Version 2.0  - Added ancestry_module and sequence_module to distinguish between implementations of the model <br />
Version 1.2  - Added example sequencing data <br />
Version 1.2  - Added the option to load sequence data for admixing <br />
Version 1.1  - Fixed a minor bug with plot_joyplot_frequencies <br />
Version 1.1  - Improved tests <br />
Version 1.1  - Improved recombination code (again) <br />
Version 1.00 - Release associated with bioRxiv submission, to be found here: https://doi.org/10.1101/2020.10.19.343491 <br />
Version 0.66 - Improved recombination code, about twice as fast <br />
Version 0.65 - Added testing and added logo <br />
Version 0.64 - Reduced cyclomatic complexity <br />
Version 0.63 - Updated random number generation <br />
Version 0.62 - Updated to Roxygen <br />
Version 0.61 - Added plot_over_time <br />
Version 0.60 - Added admixture with migration <br />
Version 0.59 - Updated underlying code tracking frequencies <br />
Version 0.58 - Removed many old functions, and improved usability for many existing functions <br />
Version 0.58 - Renamed to GenomeAdmixR <br />
Version 0.57 - Added function to generate admixed individuals <br />
Version 0.56 - Added starting frequencies to 'simulate_admixture' <br />
Version 0.55 - extended 'calculate_marker_frequency' to handle a vector of locations <br />
Version 0.55 - increased accuracy of choosing a random position for recombination, this should prevent the rare bug fixed in version 0.54 <br />
Version 0.54 - Fixed a MAJOR bug regarding recombination: in rare cases, a crossover position could be picked on an existing junction, due to the limited number of digits in uniform() <br />
Version 0.54 - Improved plot_difference_frequencies to handle modified input <br />
Version 0.53 - Added multiplicative_selection <br />
Version 0.52 - Added plot_difference_frequencies <br />
Version 0.51 - Added tajima's d calculation <br />
Version 0.50 - added simulated_admixture until <br />
Version 0.49 - Added 'simulate' to cpp <br />
Version 0.48 - Added a general 'simulate' function <br />
Version 0.47 - Changed the effect of migration <br />
Version 0.46 - Added joyplot & increase_ancestor <br />
Version 0.45 - Removed create_two_populations <br />
Version 0.44 - Added tracking regions <br />
Version 0.43 - Fixed bugs in select_population <br />
Version 0.42 - Added initial and final frequency tables <br />
Version 0.41 - Added multiple marker support <br />
Version 0.40 - Collapsed selection functions <br />
Version 0.39 - Added support for non-additive selection <br />
Version 0.38 - Added track frequencies <br />
Version 0.37 - Removed selection on regions <br />
Version 0.36 - Added progress_bar option <br />
Version 0.35 - Added calculate_marker_frequency <br />
Version 0.34 - Added selection_markers <br />
Version 0.33 - Fixed bugs in selection <br />
Version 0.32 - Moved Fish.h code to Fish.cpp <br />
Version 0.31 - Changed random number generator to R based <br />
Version 0.30 - Added Recombination = 1 code <br />
Version 0.29 - Changed internal junction representation: removed .left <br />
Version 0.28 - Reverted to Agner Fog Random number generation <br />
Version 0.27 - Speed up return types <br />
Version 0.26 - Added class verification code <br />
Version 0.25 - Squashed plotting bug <br />
Version 0.24 - Removed Output.cpp <br />
Version 0.23 - Removed number_of_founders from calc_allele_spectrum <br />
Version 0.22 - Added save and load functions <br />
Version 0.21 - Changed random-seed management <br />
Version 0.20 - Removed superfluous code <br />
Version 0.19 - Removed number_of_founders from Fst and LD code <br />
Version 0.18 - Start of tracking changes <br />

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )

## ----load libraries-----------------------------------------------------------
library(GenomeAdmixR)
library(ggplot2)
packageVersion("GenomeAdmixR")

## ----create populations-------------------------------------------------------
pops <-
  simulate_admixture(
        module = ancestry_module(),
        migration = migration_settings(migration_rate = 0,
                                         population_size = c(100, 100),
                                         initial_frequencies =
                                 list(c(1, 1, 1, 1, 0, 0, 0, 0),
                                      c(0, 0, 0, 0, 1, 1, 1, 1))),
     total_runtime = 1000,
     num_threads = 1)

pop_1 <- pops$population_1
pop_2 <- pops$population_2

## ----calculate LD-------------------------------------------------------------
mean(calculate_ld(pop = pop_1,
                  sampled_individuals = 10,
                  markers = 30)$ld_matrix,
     na.rm = TRUE)
mean(calculate_ld(pop = pop_2,
                  sampled_individuals = 10,
                  markers = 30)$ld_matrix,
     na.rm = TRUE)

## ----create isofemale lines---------------------------------------------------
iso_females_pop_1 <- create_iso_female(
                          module = ancestry_module(input_population = pop_1),
                          n = 2,
                          num_threads = 1,
                          inbreeding_pop_size = 100)
iso_females_pop_2 <- create_iso_female(
                          module = ancestry_module(input_population = pop_2),
                          n = 2,
                          num_threads = 1,
                          inbreeding_pop_size = 100)

## ----create from individuals--------------------------------------------------
pop_1_1 <- simulate_admixture(
               module = ancestry_module(input_population = iso_females_pop_1),
               pop_size = 1000,
               num_threads = 1,
               total_runtime = 1000)

pop_1_2 <- simulate_admixture(
                    module = ancestry_module(input_population =
                                               list(iso_females_pop_1[[1]],
                                                    iso_females_pop_2[[1]])),
                    pop_size = 1000,
                    num_threads = 1,
                    total_runtime = 1000)

pop_2_2 <- simulate_admixture(
                module = ancestry_module(input_population = iso_females_pop_2),
                num_threads = 1,
                pop_size = 1000,
                total_runtime = 1000)

## ----FST calculation----------------------------------------------------------
f1 <- calculate_fst(pop_1_1, pop_1_2,
                    sampled_individuals = 10, number_of_markers = 100)
# this one should be highest
f2 <- calculate_fst(pop_1_1, pop_2_2,
                    sampled_individuals = 10, number_of_markers = 100)
f3 <- calculate_fst(pop_1_2, pop_2_2,
                    sampled_individuals = 10, number_of_markers = 100)
f1
f2
f3

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )

## -----------------------------------------------------------------------------
library(GenomeAdmixR)
library(ggplot2)
packageVersion("GenomeAdmixR")


## ----setup select_matrix------------------------------------------------------
select_matrix <- matrix(ncol = 5, nrow = 2)
s <- 0.1
select_matrix[1, ] <- c(0.23, 1.0, 1 + 0.5 * s, 1 + s, 0)
select_matrix[2, ] <- c(0.27, 1.0, 1 + 0.5 * s, 1 + s, 1)

markers <- seq(from = 0.2, to = 0.3, length.out = 100)

## ----simulate-----------------------------------------------------------------
selected_pop <- simulate_admixture(
                    module = ancestry_module(number_of_founders = 2,
                                             morgan = 1,
                                             markers = markers),
                    pop_size = 100,
                    total_runtime = 1001,
                    num_threads = 1,
                    select_matrix = select_matrix)

## ----joyplot all--------------------------------------------------------------
time_points <- seq(from = 0, to = 1000, by = 100)
plot_joyplot_frequencies(selected_pop$frequencies,
                         time_points,
                         picked_ancestor = "ALL")

## ----joyplot separate 0-------------------------------------------------------
p1 <- plot_joyplot_frequencies(selected_pop$frequencies,
                               time_points,
                               picked_ancestor = 0)
p1 + ggplot2::geom_vline(xintercept = 0.23, lty = 2)

## ----joyplot separate 1-------------------------------------------------------
p2 <- plot_joyplot_frequencies(selected_pop$frequencies,
                               time_points,
                               picked_ancestor = 1)
p2 + ggplot2::geom_vline(xintercept = 0.27, lty = 2)

## ----setup, include=FALSE-----------------------------------------------------
library(GenomeAdmixR)
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
print_mat <- function(mat) {
  n <- nrow(mat)
  c("\\begin{bmatrix}",
    paste0(sapply(seq_len(n - 1),
                  function(i) paste0(mat[i, ], collapse = " & ")),
           " \\\\"),
    paste0(mat[n, ], collapse = " & "),
    "\\end{bmatrix}")
}
RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )

## ----creating fake data 2-----------------------------------------------------
  fake_input_data <-
    create_artificial_genomeadmixr_data(number_of_individuals = 10,
                                        marker_locations = 1:10)

  fake_input_data$markers
  fake_input_data$genomes


## ----simulate admix data------------------------------------------------------
simulated_pop <- simulate_admixture(
                    module = sequence_module(molecular_data = fake_input_data),
                    pop_size = 100,
                    num_threads = 1,
                    total_runtime = 100)

## ----creating fake data-------------------------------------------------------
  chosen_markers <- 1:100
   fake_input_data1 <-
    create_artificial_genomeadmixr_data(number_of_individuals = 100,
                                        marker_locations = chosen_markers,
                                        used_nucleotides = 1:2)
  fake_input_data2 <-
    create_artificial_genomeadmixr_data(number_of_individuals = 100,
                                        marker_locations = chosen_markers,
                                        used_nucleotides = 3:4)
  
  combined_data <- combine_input_data(input_data_list = list(fake_input_data1,
                                                             fake_input_data2),
                                      frequencies = c(0.5, 0.5),
                                      pop_size = 1000)

## ----simulate-----------------------------------------------------------------
simulated_pop <- simulate_admixture(
                      module = sequence_module(molecular_data = combined_data,
                                               markers = 1:100,
                                               morgan = 1),
                      pop_size = 1000,
                      num_threads = 1,
                      total_runtime = 100)

## ----visualise----------------------------------------------------------------
plot_over_time(simulated_pop$frequencies, focal_location = 50)
plot_joyplot_frequencies(simulated_pop$frequencies, time_points = c(0, 50, 100))

## ----simulate selection-------------------------------------------------------
selection_matrix <- matrix(NA, nrow = 1, ncol = 5)
selection_matrix[1, ] <- c(50, 1, 1.1, 1.2, 1)

# we expect allele 'a' to do well at 0.5 Morgan:
selected_pop <- simulate_admixture(
                      module = sequence_module(molecular_data = fake_input_data1,
                                               morgan = 1,
                                               markers = chosen_markers),
                      pop_size = 1000,
                      total_runtime = 100,
                      num_threads = 1,
                      select_matrix = selection_matrix)
plot_over_time(selected_pop$frequencies, focal_location = 50)

## ----migration----------------------------------------------------------------

migr_pop <-
  simulate_admixture(
        module = sequence_module(molecular_data = list(fake_input_data1,
                                                       fake_input_data2)),
        migration = migration_settings(migration_rate = 0.01,
                                       population_size = c(100, 100)),
        num_threads = 1,
        total_runtime = 100)

## ----from_simulation----------------------------------------------------------
simulated_pop <- simulate_admixture(
                      module = ancestry_module(), 
                      pop_size = 100,
                      num_threads = 1,
                      total_runtime = 100)
prepared_pop  <-
  simulation_data_to_genomeadmixr_data(simulation_data = simulated_pop,
                                       markers = seq(0, 1, length.out = 100))

simulated_pop2 <- simulate_admixture(
                        module = sequence_module(molecular_data = prepared_pop),
                        pop_size = 100,
                        num_threads = 1,
                        total_runtime = 100)

## ----results = 'asis'---------------------------------------------------------
writeLines(print_mat(matrix(1 / 4, nrow = 4, ncol = 4)))

## ----results = 'asis'---------------------------------------------------------
kappa <- 0.2
diag_entry <- 1 - (1 / 4 + 1 / 4 + 0.2 / 4)

writeLines(print_mat(matrix(c(diag_entry, kappa, 1 / 4, 1 / 4,
                              kappa, diag_entry, 1 / 4, 1 / 4,
                              1 / 4, 1 / 4, diag_entry, kappa,
                              1 / 4, 1 / 4, kappa, diag_entry),
                              nrow = 4, ncol = 4)))

## ----example mutation---------------------------------------------------------
  fake_input_data3 <-
  create_artificial_genomeadmixr_data(number_of_individuals = 100,
                                      marker_locations = 1:1000,
                                      used_nucleotides = 1)
  
  kappa <- 0.5
  rate_matrix <- matrix(c(0, kappa, 1, 1,
                          kappa, 0, 1, 1,
                          1, 1, 0, kappa,
                          1, 1, kappa, 0), nrow = 4, ncol = 4)
  
  mutation_rate <- 1e-5
  
  simulated_pop <- simulate_admixture(
                      module = sequence_module(molecular_data = fake_input_data3,
                                               morgan = 1,
                                               markers = chosen_markers,
                                               mutation_rate = mutation_rate,
                                               substitution_matrix = rate_matrix),
                      pop_size = 1000,
                      num_threads = 1,
                      total_runtime = 100)
  
  freqs <- calculate_allele_frequencies(simulated_pop)
  
  require(magrittr, quietly = TRUE)
  freqs %>%
    dplyr::group_by(ancestor) %>%
    dplyr::summarise(mean(frequency))

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )

## -----------------------------------------------------------------------------
library(GenomeAdmixR)
library(ggplot2)
packageVersion("GenomeAdmixR")

## ----create wildpop-----------------------------------------------------------
wildpop <-  simulate_admixture(
                    module = ancestry_module(number_of_founders = 4,
                                             morgan = 1),
                    pop_size = 100,
                    total_runtime = 1000,
                    num_threads = 1)

## -----------------------------------------------------------------------------
wildpop

## -----------------------------------------------------------------------------
wildpop[[1]]

## ----create isofemale---------------------------------------------------------
isofemale <- create_iso_female(
                  module = ancestry_module(input_population = wildpop,
                                           morgan = 1),
                  n = 1,
                  inbreeding_pop_size = 1000,
                  run_time = 200,
                  num_threads = 1)

## -----------------------------------------------------------------------------
plot(isofemale[[1]])

## ----toyexample---------------------------------------------------------------
wildpop <-  simulate_admixture(
                  module = ancestry_module(number_of_founders = 4,
                                           morgan = 1),
                  pop_size = 100,
                  num_threads = 1,
                  total_runtime = 100)

isofemale <- create_iso_female(
                  module = ancestry_module(input_population = wildpop,
                                           morgan = 1),
                  n = 1,
                  inbreeding_pop_size = 100,
                  num_threads = 1,
                  run_time = 10000)
plot(wildpop$population[[1]])
plot(isofemale[[1]])

## ----plot selection of chromosome---------------------------------------------
plot_chromosome(isofemale[[1]]$chromosome1, xmin = 0.0, xmax = 0.5)

## ----create two populations---------------------------------------------------
both_populations <-
  simulate_admixture(
    module = ancestry_module(morgan = 1),
    migration = migration_settings(population_size = c(100, 100),
                        migration_rate = 0.0,
                        initial_frequencies =
                                 list(c(rep(1, 20), rep(0, 20)),
                                      c(rep(0, 20), rep(1, 20)))
                      ),
    total_runtime = 1000,
    num_threads = 1)

population_1 <- both_populations$population_1

population_2 <- both_populations$population_2

## ----draw two isofemales------------------------------------------------------
isofemales <- create_iso_female(
                  module = ancestry_module(input_population = population_1,
                                           morgan = 1),
                  n = 2,
                  inbreeding_pop_size = 100,
                  run_time = 10000,
                  num_threads = 1)

plot_chromosome(isofemales[[1]]$chromosome1, 0, 1)
plot_chromosome(isofemales[[2]]$chromosome1, 0, 1)

## ----seed mixed population----------------------------------------------------
mixed_population <-
  simulate_admixture(
        module = ancestry_module(input_population = list(isofemales[[1]],
                                                         isofemales[[2]]),
                                 morgan = 1),
        pop_size = 100,
        num_threads = 1,
        total_runtime = 100)

## ----plot mixed_population----------------------------------------------------
plot(mixed_population$population[[1]])

## ----calc FST-----------------------------------------------------------------
fst <- calculate_fst(population_1,
                 population_2,
                 sampled_individuals = 10,
                 number_of_markers = 100,
                 random_markers = TRUE)
fst

## -----------------------------------------------------------------------------
  ld_results <- calculate_ld(wildpop,
                             markers = 10)

  plot(ld_results$ld_matrix~ld_results$dist_matrix,
       xlab = "Genetic Distance (Morgan)",
       ylab = "LD",
       pch = 16)
  plot(ld_results$rsq_matrix~ld_results$dist_matrix,
       xlab = "Genetic Distance (Morgan)",
       ylab = "r_sq",
       pch = 16)
  plot(ld_results$ld_matrix~ld_results$rsq_matrix,
       xlab = "r_sq",
       ylab = "LD",
       pch = 16)

## ----no LD--------------------------------------------------------------------
no_ld_pop <- simulate_admixture(
                  module = ancestry_module(number_of_founders = 4,
                                           morgan = 1),
                  pop_size = 100,
                  num_threads = 1,
                  total_runtime = 1000)

ld_results <- calculate_ld(no_ld_pop,
                           sampled_individuals = 10,
                           markers = 10)

plot(ld_results$ld_matrix~ld_results$dist_matrix,
     pch = 16,
     xlab = "Distance",
     ylab = "LD",
     xlim = c(0, 1),
     ylim = c(0, 1))

## ----strong LD----------------------------------------------------------------
strong_ld_pop <- simulate_admixture(
                  module = ancestry_module(number_of_founders = 4,
                                           morgan = 1),
                  pop_size = 100,
                  num_threads = 1,
                  total_runtime = 10)

ld_results <- calculate_ld(strong_ld_pop,
                           sampled_individuals = 10,
                           markers = 10)

plot(ld_results$ld_matrix~ld_results$dist_matrix,
     pch = 16,
     xlab = "Distance",
     ylab = "LD",
     xlim = c(0, 1),
     ylim = c(0, 1))

## ----heterozygote selection---------------------------------------------------
s <- 0.1
selection_matrix <- matrix(nrow = 1, ncol = 5)
selection_matrix[1, ] <- c(0.5,
                           1.0, 1.0 + s, 1.0,
                           0)

markers <- 0.5

selected_pop <- simulate_admixture(
                    module = ancestry_module(number_of_founders = 2,
                                             morgan = 1,
                                             markers = markers),
                    pop_size = 1000,  
                    total_runtime = 100,
                    num_threads = 1,
                    select_matrix = selection_matrix)

plot_over_time(selected_pop$frequencies, markers)

## ----homozygote selection-----------------------------------------------------
s <- 0.1
selection_matrix[1, ] <- c(0.5,
                           1.0, 1.0, 1.0 + s,
                           0)

selected_pop <- simulate_admixture(
                    module = ancestry_module(number_of_founders = 10,
                                             morgan = 1,
                                             markers = markers),
                    pop_size = 1000,  
                    total_runtime = 300,
                    num_threads = 1,
                    select_matrix = selection_matrix)

plot_over_time(selected_pop$frequencies, markers)

#' Individual based simulation of the breakdown of contiguous ancestry blocks in
#' two populations linked by migration
#' @description Individual based simulation of the breakdown of contiguous
#' ancestry blocks, with or without selection. Simulations can be started from
#' scratch, or from a predefined input population. Two populations are
#' simulated, connected by migration
#' @param input_population_1 Potential earlier simulated population used as
#' starting point for the simulation. If not provided by the user, the
#' simulation starts from scratch.
#' @param input_population_2 Potential earlier simulated population used as
#' starting point for the simulation. If not provided by the user,
#' the simulation starts from scratch.
#' @param pop_size Vector containing the number of individuals in both
#' populations.
#' @param initial_frequencies A list describing the initial frequency of each
#' ancestor in each population. Each entry in the list contains a vector with
#' the frequencies for all ancestor. The length of the vector indicates the
#' number of unique ancestors. If a vector not summing to 1 is provided, the
#' vector is normalized.
#' @inheritParams default_params_doc
#' @param migration_rate  Rate of migration between the two populations.
#' Migration is implemented such that with probability m (migration rate) one
#' of the two parents of a new offspring is from the other population, with
#' probability 1-m both parents are of the focal population.
#' @param stop_at_critical_fst option to stop at a critical FST value
#' , default is FALSE
#' @param critical_fst the critical fst value to stop, if
#' \code{stop_simulation_at_critical_fst} is TRUE
#' @param generations_between_update The number of generations after which the
#' simulation has to check again whether the critical Fst value is exceeded
#' @param sampled_individuals Number of individuals to be sampled at random from
#' the population to estimate Fst
#' @param number_of_markers Number of markers to be used to estimate Fst
#' @param random_markers Are the markers to estimate Fst randomly distributed,
#' or regularly distributed? Default is TRUE.
#' @return A list with: \code{population_1}, \code{population_2} two population
#' objects, and three tibbles with allele frequencies (only contain values of a
#' vector was provided to the argument \code{markers}: \code{frequencies},
#' \code{initial_frequencies} and \code{final_frequencies}. Each tibble contains
#' five columns, \code{time}, \code{location}, \code{ancestor}, \code{frequency}
#' and \code{population}, which indicates the number of generations, the
#' location along the chromosome of the marker, the ancestral allele at that
#' location in that generation, the frequency of that allele and the population
#' in which it was recorded (1 or 2). If a critical fst value was used to
#' terminate the simulation, and object \code{FST} with the final FST estimate
#' is returned as well.
#' @export
simulate_ancestry_migration <- function(input_population_1 = NA,
                                        input_population_2 = NA,
                                         pop_size = c(100, 100),
                                         initial_frequencies = list(c(1.0, 0),
                                                                    c(0, 1.0)),
                                         total_runtime = 100,
                                         morgan = 1,
                                         num_threads = 1,
                                         select_matrix = NA,
                                         markers = NA,
                                         verbose = FALSE,
                                         track_junctions = FALSE,
                                         multiplicative_selection = TRUE,
                                         migration_rate = 0.0,
                                         stop_at_critical_fst = FALSE,
                                         critical_fst = 0.1,
                                         generations_between_update = 100,
                                         sampled_individuals = 10,
                                         number_of_markers = 100,
                                         random_markers = TRUE) {

  if (stop_at_critical_fst) {
    if (verbose) message("stopping when FST is: ", critical_fst)
    return(simulate_ancestry_until(input_population_1 = input_population_1,
                                    input_population_2 = input_population_2,
                                    pop_size = pop_size,
                                    initial_frequencies = initial_frequencies,
                                    total_runtime = total_runtime,
                                    morgan = morgan,
                                    num_threads = num_threads,
                                    select_matrix = select_matrix,
                                    markers = markers,
                                    verbose = verbose,
                                    track_junctions = track_junctions,
                                    multiplicative_selection =
                                      multiplicative_selection,
                                    migration_rate = migration_rate,
                                    critical_fst = critical_fst,
                                    generations_between_update =
                                      generations_between_update,
                                    sampled_individuals = sampled_individuals,
                                    number_of_markers = number_of_markers,
                                    random_markers = random_markers))
  }

  if (!is.list(initial_frequencies)) {
    if (length(initial_frequencies) <= 2) {
      stop("need separate frequencies for each population")
    }
  }

  set_num_threads(num_threads)

  input_population_1 <- check_input_pop(input_population_1)
  input_population_2 <- check_input_pop(input_population_2)

  initial_frequencies <- check_initial_frequencies(initial_frequencies)

  select_matrix <-     check_select_matrix(select_matrix,
                                           markers,
                                           use_data = FALSE)

  if (length(pop_size) == 1) {
    pop_size <- c(pop_size, pop_size)
  }


  if (length(markers) == 1) {
    if (is.na(markers))  {
      markers <- c(-1, -1)
      track_frequency <- FALSE
    } else {
      track_frequency <- TRUE
    }
  } else {
    track_frequency <- TRUE
  }

  init_freq_matrix <- matrix(nrow = length(initial_frequencies),
                             ncol = length(initial_frequencies[[1]]))

  for (i in seq_along(initial_frequencies)) {
    for (j in seq_along(initial_frequencies[[i]])) {
      init_freq_matrix[i, j] <- initial_frequencies[[i]][j]
    }
  }

  input_population_1 <- population_to_vector(input_population_1)
  input_population_2 <- population_to_vector(input_population_2)

  selected_pop <- simulate_migration_cpp(input_population_1,
                                         input_population_2,
                                         select_matrix,
                                         pop_size,
                                         init_freq_matrix,
                                         total_runtime,
                                         morgan,
                                         verbose,
                                         track_frequency,
                                         markers,
                                         track_junctions,
                                         multiplicative_selection,
                                         migration_rate,
                                         num_threads)

  selected_popstruct_1 <- create_pop_class(selected_pop$population_1)
  selected_popstruct_2 <- create_pop_class(selected_pop$population_2)

  colnames(selected_pop$initial_frequencies) <- c("time",
                                                  "location",
                                                  "ancestor",
                                                  "frequency",
                                                  "population")

  colnames(selected_pop$final_frequencies) <- c("time",
                                                "location",
                                                "ancestor",
                                                "frequency",
                                                "population")

  initial_freq_tibble <- tibble::as_tibble(selected_pop$initial_frequencies)
  final_freq_tibble   <- tibble::as_tibble(selected_pop$final_frequencies)

  output <- generate_output_list_two_pop(selected_pop,
                                         selected_popstruct_1,
                                         selected_popstruct_2,
                                         initial_freq_tibble,
                                         final_freq_tibble,
                                         track_frequency,
                                         track_junctions)

  class(output) <- "genomadmixr_simulation"
  reset_num_threads()
  return(output)
}

#' Plot the frequencies of all ancestors over time
#' @description This function plots the frequency of all ancestors over time at
#' a specific location on the chromosome, after performing a simulation.
#' @param frequencies  A tibble containing four columns: \code{time},
#' \code{location}, \code{ancestor}, \code{frequency}. A fifth colum
#' \code{population} can be included if the tibble is the result of
#' \code{simulate_admixture_migration}.
#' @param focal_location Location (in Morgan) where to plot the allele
#' frequencies.
#' @return a ggplot2 object
#' @examples
#' pop <- simulate_admixture(
#'            module = ancestry_module(number_of_founders = 10,
#'                                     markers = 0.5),
#'            pop_size = 1000,
#'            total_runtime = 11)
#' require(ggplot2)
#' plot_over_time(frequencies = pop$frequencies,
#'                focal_location = 0.5)
#' @export
plot_over_time <- function(frequencies,
                           focal_location) {

  p1 <- c()
  to_plot <- dplyr::filter(frequencies, .data[["location"]] == focal_location)
  if (length(to_plot$time) == 0) {
    stop("No data to plot, are you sure that the focal location\nis within the
        vector of molecular markers?\n")
  }

  if ("population" %in% colnames(frequencies)) {
    p1 <- ggplot2::ggplot(to_plot)  +
      ggplot2::geom_step(ggplot2::aes(x = .data[["time"]],
                                      y = .data[["frequency"]],
                                      col = interaction(.data[["ancestor"]],
                                                        .data[["population"]]
                                                        )
                                      )
      )
  } else {
    p1 <- ggplot2::ggplot(to_plot,
                          ggplot2::aes(x = .data[["time"]],
                                       y = .data[["frequency"]],
                                       col = as.factor(.data[["ancestor"]]))) +
      ggplot2::geom_step()
  }

  p1 <- p1 +
    ggplot2::xlab("Time (Generations)") +
    ggplot2::ylab("Frequency") +
    ggplot2::labs(col = "Ancestor")

  return(p1)
}

#' Individual based simulation of the breakdown of contiguous ancestry blocks.
#' @description Individual based simulation of the breakdown of contiguous
#' ancestry blocks, with or without selection. Simulations can be started from
#' scratch, or from a predefined input population.
#' @param input_population Potential earlier simulated population used as
#' starting point for the simulation. If not provided by the user, the
#' simulation starts from scratch.
#' @param pop_size The number of individuals in the population. If the number is
#' larger than the number of individuals in the input population (if provided),
#' additional individuals are sampled randomly from the input population to
#' reach the intended size.
#' @param number_of_founders Number of unique ancestors
#' @param initial_frequencies A vector describing the initial frequency of each
#' ancestor. By default, equal frequencies are assumed. If a vector not summing
#' to 1 is provided, the vector is normalized.
#' @inheritParams default_params_doc
#' @return A list with: \code{population} a population object, and three tibbles
#' with allele frequencies (only contain values of a vector was provided to the
#' argument \code{markers}: \code{frequencies} , \code{initial_frequencies} and
#' \code{final_frequencies}. Each tibble contains four columns, \code{time},
#' \code{location}, \code{ancestor} and \code{frequency}, which indicates the
#' number of generations, the location along the chromosome of the marker, the
#' ancestral allele at that location in that generation, and finally, the
#' frequency of that allele.
simulate_ancestry <- function(input_population = NA,
                               pop_size = NA,
                               number_of_founders = 2,
                               initial_frequencies = NA,
                               total_runtime = 100,
                               morgan = 1,
                               num_threads = 1,
                               select_matrix = NA,
                               markers = NA,
                               verbose = FALSE,
                               track_junctions = FALSE,
                               multiplicative_selection = TRUE) {

  set_num_threads(num_threads)

  input_population <- check_input_pop(input_population)

  if (is.na(pop_size)) {
    if (inherits(input_population, "population")) {
      pop_size <- length(input_population)
    } else {
      reset_num_threads()
      stop("pop_size is undefined, need an input population")
    }
  }

  select_matrix <- check_select_matrix(select_matrix,
                                       markers,
                                       use_data = FALSE)

  if (length(markers) == 1) {
    if (is.na(markers))  {
      markers <- c(-1, -1)
      track_frequency <- FALSE
    } else {
      track_frequency <- TRUE
    }
  } else {
    track_frequency <- TRUE
  }

  if (inherits(input_population, "population")) {
    input_population <- population_to_vector(input_population)
  }

  selected_pop <- simulate_cpp(input_population,
                               select_matrix,
                               pop_size,
                               number_of_founders,
                               initial_frequencies,
                               total_runtime,
                               morgan,
                               verbose,
                               track_frequency,
                               markers,
                               track_junctions,
                               multiplicative_selection,
                               num_threads)

  selected_popstruct <- create_pop_class(selected_pop$population)

  colnames(selected_pop$initial_frequencies) <- c("time",
                                                  "location",
                                                  "ancestor",
                                                  "frequency")
  colnames(selected_pop$final_frequencies) <-
     colnames(selected_pop$initial_frequencies)


  initial_freq_tibble <- tibble::as_tibble(selected_pop$initial_frequencies)
  final_freq_tibble   <- tibble::as_tibble(selected_pop$final_frequencies)


  output <- generate_output_list_one_pop(selected_popstruct,
                                         selected_pop,
                                         initial_freq_tibble,
                                         final_freq_tibble,
                                         track_frequency,
                                         track_junctions)

  class(output) <- "genomadmixr_simulation"
  reset_num_threads()
  return(output)
}

#' Plot both the starting frequencies and the final frequencies in one plot
#' @description This function plots the distribution of both the starting and
#' the final frequencies in one plot
#' @param results An object which is the result of \code{simulate_admixture},
#' being a list with four properties:
#' \code{population}, \code{frequencies}, \code{initial_frequencies} and
#' \code{final frequencies}
#' @param picked_ancestor Default is "ALL", where different colors indicate
#' different ancestors. Alternatively, for clarity, the user can specify a
#' specific ancestral allele, and only that allele is plotted
#' @param picked_population If multiple populations were simulated (in the case
#' of \code{simulate_admixture_migration}), which population should be plotted?
#' Default is population_1.
#' @return a ggplot object
#' @examples
#' markers <- seq(from = 0.2, to = 0.3, length.out = 100)
#'
#' pop <- simulate_admixture(
#'             module = ancestry_module(number_of_founders = 3,
#'                                      morgan = 1,
#'                                      markers = markers),
#'            pop_size = 1000,
#'            total_runtime = 11)
#' require(ggplot2)
#' plot_start_end(pop,
#'                picked_ancestor = "ALL")
#' plot_start_end(pop,
#'                picked_ancestor = 1)
#' @export
plot_start_end <- function(results,
                           picked_ancestor = "ALL",
                           picked_population = 1) {


  a1 <- results$initial_frequency
  a2 <- results$final_frequency

  if ("population" %in% colnames(a1)) {
    a1 <- subset(a1,
                 a1$population == picked_population)
    a2 <- subset(a2,
                 a2$population == picked_population)
  }

  a1_m <- dplyr::mutate(a1, timepoint = "start")
  a2_m <- dplyr::mutate(a2, timepoint = "end")

  to_plot_m <- rbind(a1_m, a2_m)

  if (picked_ancestor[[1]] == "ALL") {
    to_plot <- to_plot_m

    p1 <- ggplot2::ggplot(to_plot,
                      ggplot2::aes(x = .data[["location"]],
                                   y = .data[["frequency"]],
                                   colour = as.factor(.data[["ancestor"]]),
                                   group = interaction(.data[["ancestor"]],
                                                       .data[["timepoint"]]))) +
      ggplot2::geom_step(ggplot2::aes(lty = .data[["timepoint"]]))
  } else {

    to_plot <- subset(to_plot_m,
                      to_plot_m$ancestor %in% picked_ancestor)

    p1 <- ggplot2::ggplot(to_plot,
                        ggplot2::aes(x = .data[["location"]],
                                   y =  .data[["frequency"]],
                                   colour = as.factor(.data[["ancestor"]]),
                                   group = interaction(.data[["ancestor"]],
                                                       .data[["timepoint"]]))) +
      ggplot2::geom_step(ggplot2::aes(lty = .data[["timepoint"]]))
  }

  if (max(to_plot$location) < 10) {
    p1 <- p1 +
      ggplot2::xlab("Location (Morgan)")
  } else {
    p1 <- p1 +
      ggplot2::xlab("Location (bp)")
  }

  p1 <- p1 +
    ggplot2::ylab("Frequency") +
    ggplot2::labs(col = "Ancestor",
                  lty = "Time Point")

  return(p1)
}

context("selection two alleles")

test_that("select population two_alleles", {
  select_matrix <- matrix(ncol = 5, nrow = 1)
  s <- 0.1
  select_matrix[1, ] <- c(0.05, 1.0, 1 + 0.5 * s, 1 + s, 0)

  selected_pop <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                                10,
                                                              morgan = 1),
                                     pop_size = 100,
                                     total_runtime = 100,
                                     select_matrix = select_matrix)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))

  vv <- calculate_marker_frequency(selected_pop$population, 0.05)


  number_of_founders <- 10
  run_time <- 100

  selected_pop <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                                number_of_founders,
                                                              morgan = 1),
                                     pop_size = 100,
                                     total_runtime = run_time,
                                     select_matrix = select_matrix)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))
})

test_that("select on population", {
  sourcepop <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                             10,
                                                           morgan = 1),
                                  pop_size = 100,
                                 total_runtime = 1000)

  testthat::expect_true(verify_population(sourcepop))

  select_matrix <- matrix(ncol = 5, nrow = 1)
  s <- 0.1
  select_matrix[1, ] <- c(0.05, 1.0, 1 + 0.5 * s, 1 + s, 0)

  selected_pop <- simulate_admixture(module =
                                       ancestry_module(input_population = sourcepop,
                                                       morgan = 1),
                                     select_matrix = select_matrix,
                                    pop_size = 100,
                                    total_runtime = 100)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))

  selected_pop <- simulate_admixture(module =
                                      ancestry_module(input_population = sourcepop,
                                                       morgan = 1),
                                     select_matrix = select_matrix,
                                               pop_size = 100,
                                               total_runtime = 100)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))
})


test_that("select population two_alleles multiple markers", {
  select_matrix <- matrix(ncol = 5, nrow = 2)
  s <- 0.1
  select_matrix[1, ] <- c(0.25, 1.0, 1 + 0.5 * s, 1 + s, 0)
  select_matrix[2, ] <- c(0.75, 1.0, 1, 1 + s,  1)

  selected_pop <- simulate_admixture(module = ancestry_module(number_of_founders = 10),
                                     pop_size = 100,
                                     total_runtime = 100,
                                     select_matrix = select_matrix)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))

  sourcepop <- simulate_admixture(module = ancestry_module(number_of_founders = 10),
                                  pop_size = 100,
                                  total_runtime = 1000)

  testthat::expect_true(verify_population(sourcepop))

  selected_pop <- simulate_admixture(module =
                                       ancestry_module(input_population = sourcepop),
                                     select_matrix = select_matrix,
                                     pop_size = 100,
                                     total_runtime = 100)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))

  selected_pop <- simulate_admixture(module =
                                       ancestry_module(input_population = sourcepop,
                                                       morgan = 1),
                                     select_matrix = select_matrix,
                                     pop_size = 100,
                                     total_runtime = 100)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))
})

test_that("select population two_alleles multiply vs sum", {
  select_matrix <- matrix(ncol = 5, nrow = 2)
  s <- 0.2
  select_matrix[1, ] <- c(0.25, 1.0, 1 + 0.5 * s, 1 + s, 0)
  select_matrix[2, ] <- c(0.75, 1.0, 1, 1 + s,  1)

  selected_pop <- simulate_admixture(module =
                                       ancestry_module(number_of_founders = 10,
                                                       markers = c(0.25, 0.75)),
                                     pop_size = 100,
                                     total_runtime = 100,
                                     select_matrix = select_matrix,
                                     multiplicative_selection = TRUE)

  selected_pop2 <- simulate_admixture(module =
                                       ancestry_module(number_of_founders = 10,
                                                       markers = c(0.25, 0.75)),
                                     pop_size = 100,
                                     total_runtime = 100,
                                     select_matrix = select_matrix,
                                     multiplicative_selection = FALSE)

  vv <- GenomeAdmixR::calculate_marker_frequency(selected_pop,
                                                 location = c(0.25, 0.75))
  vv2 <- GenomeAdmixR::calculate_marker_frequency(selected_pop2,
                                                  location = c(0.25, 0.75))

  testthat::expect_gt(length(vv2$ancestor), 1)

  testthat::expect_gt(length(vv$ancestor), 1)
})

test_that("select population two_alleles regions", {
  select_matrix <- matrix(ncol = 5, nrow = 2)
  s <- 0.1
  select_matrix[1, ] <- c(0.25, 1.0, 1 + 0.5 * s, 1 + s, 0)
  select_matrix[2, ] <- c(0.75, 1.0, 1, 1 + s,  1)

  markers <- seq(from = 0.2, to = 0.3, length.out = 21)

  selected_pop <- simulate_admixture(module = ancestry_module(number_of_founders = 10,
                                                              markers = markers),
    pop_size = 100,
                                     total_runtime = 100,
                                     select_matrix = select_matrix)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))

  sourcepop <- simulate_admixture(module = ancestry_module(number_of_founders = 10),
                                  pop_size = 100,
                                  total_runtime = 1000)

  testthat::expect_true(verify_population(sourcepop))

  selected_pop <- simulate_admixture(module = ancestry_module(input_population = sourcepop,
                                                              markers = markers),
                                     select_matrix = select_matrix,
                                     pop_size = 100,
                                     total_runtime = 100)

  testthat::expect_equal(length(selected_pop$population), 100)
  testthat::expect_true(verify_population(selected_pop$population))

  plot_start_end(selected_pop)
  plot_start_end(selected_pop,
                 picked_ancestor = 0)

  plot_difference_frequencies(selected_pop)
  plot_difference_frequencies(selected_pop,
                              picked_ancestor = 0)
})

test_that("selection abuse", {
  sourcepop <- simulate_admixture(pop_size = 100,
                                  total_runtime = 100)

  testthat::expect_true(verify_population(sourcepop))

  select_matrix <- matrix(ncol = 5, nrow = 3)
  s <- 0.1
  select_matrix[1, ] <- c(0.05, 1.0, 1 + 0.5 * s, 1 + s, 0)
  select_matrix[2, ] <- c(0.15, 1.0, 1 + 0.5 * s, 1 + s, 0)

  testthat::expect_error(
      simulate_admixture(module = ancestry_module(input_population = sourcepop),
                         select_matrix = select_matrix,
                         pop_size = 1000,
                         total_runtime = 1000),
    "Can't start, there are NA values in the selection matrix!"
  )

  testthat::expect_error(
    simulate_admixture(select_matrix = select_matrix,
                       pop_size = 1000,
                       total_runtime = 1000),
    "Can't start, there are NA values in the selection matrix!"
  )

  select_matrix <- matrix(ncol = 3, nrow = 3)
  select_matrix[1, ] <- c(0.0, NA, 0)
  select_matrix[2, ] <- c(NA, 1.0, 1)


  testthat::expect_error(
    simulate_admixture(module = ancestry_module(input_population = sourcepop),
                       select_matrix = select_matrix,
                       pop_size = 1000,
                       total_runtime = 1000),
    "Can't start, there are NA values in the selection matrix!"
  )

  testthat::expect_error(
    simulate_admixture(select_matrix = select_matrix,
                       pop_size = 100,
                       total_runtime = 10),
    "Can't start, there are NA values in the selection matrix!"
  )


  select_matrix <- matrix(ncol = 5, nrow = 2)
  s <- 0.1
  select_matrix[1, ] <- c(0.05, 1.0, 1 + 0.5 * s, 1 + s, 0)
  select_matrix[2, ] <- c(0.15, 1.0, 1 + 0.5 * s, 1 + s, 0)

  select_matrix <- matrix(ncol = 3, nrow = 1)
  s <- 0.1
  select_matrix[1, ] <- c(0.5, 0.1, 0.2)

  testthat::expect_error(
    simulate_admixture(module = ancestry_module(input_population = sourcepop),
                       select_matrix = select_matrix,
                       pop_size = 1000,
                       total_runtime = 10)
  )

  testthat::expect_error(
    simulate_admixture(module = ancestry_module(input_population = sourcepop,
                                                number_of_founders = 10),
                       select_matrix = select_matrix,
                       pop_size = 100,
                       total_runtime = 10)
  )
})

context("simulate_migration")

test_that("simulate_migration base", {
  testthat::expect_silent(
    vx <- simulate_admixture(migration = migration_settings(migration_rate = 0.1),
                           total_runtime = 5)
  )
})

test_that("simulate_migration", {
  vx <- simulate_admixture(migration = migration_settings(migration_rate = 0.1),
                           total_runtime = 10)

  testthat::expect_true(verify_population(vx$population_1))
  testthat::expect_true(verify_population(vx$population_2))

  testthat::expect_silent(
    vz <- simulate_admixture(module = ancestry_module(input_population =
                                                        list(vx$population_1[[1]],
                                                             vx$population_2[[2]])),
                            migration = migration_settings(migration_rate = 0.01),
                             total_runtime = 10)
  )

  markers <- seq(from = 0.4, to = 0.6, length.out = 100)
  vy <- simulate_admixture(module = ancestry_module(markers = markers),
                           migration =
                             migration_settings(migration_rate = 0.01,
                                                initial_frequencies =
                                                  list(c(0.5, 0.5, 0, 0),
                                                       c(0, 0, 0.5, 0.5))),
                           total_runtime = 100)

  testthat::expect_true(verify_population(vy$population_1))
  testthat::expect_true(verify_population(vy$population_2))

  testthat::expect_true(length(markers) ==
                          length(unique(vy$frequencies$location)))


  select_matrix <- matrix(NA, nrow = 1, ncol = 5)

  s <- 5
  select_matrix[1, ] <- c(0.5, 1.0, 1.0 + 0.25 * s, 1.0 + s, 0)

  markers <- seq(from = 0.4, to = 0.60, by = 0.01)

  testthat::expect_message(
    vy <- simulate_admixture(module = ancestry_module(markers = markers),
                             migration = migration_settings(migration_rate = 0.01,
                                                            initial_frequencies = list(c(1, 0),
                                                                                       c(0, 1))),
                             select_matrix = select_matrix,
                             total_runtime = 100)
  )

  plot_difference_frequencies(vy)
  plot_start_end(vy)

  plot_frequencies(vy$population_1, locations = c(0.3, 0.5, 0.8))
  plot_frequencies(vy$population_2, locations = c(0.3, 0.5, 0.8))
  vv <- plot_joyplot_frequencies(vy$frequencies, time_points = c(0, 10, 50))

  testthat::expect_silent(
  vy <- simulate_admixture(module = ancestry_module(markers = 0.5,
                                                    track_junctions = TRUE),
    migration = migration_settings(migration_rate = 0.0,
                                   initial_frequencies =
                                     list(c(0.5, 0.5, 0, 0),
                                          c(0, 0, 0.5, 0.5))),
    total_runtime = 100)
  )
})

#' Plot the frequencies of all ancestors along the genome.
#' @description This function plots the frequency of all ancestors after
#' performing a simulation.
#' @param result An object which is the result of \code{select_population} or
#' \code{create_population_selection}, being a list with four properties:
#' \code{population}, \code{frequencies}, \code{initial_frequencies} and
#' \code{final frequencies}
#' @param locations A vector indicating the locations (in Morgan) where to
#' calculate the allele frequencies.
#' @param progress_bar Displays a progress_bar if TRUE. Default value is FALSE
#' @return a ggplot2 object
#' @examples
#' pop <- simulate_admixture(
#'              module = ancestry_module(number_of_founders = 4),
#'              pop_size = 1000,
#'              total_runtime = 11)
#' require(ggplot2)
#' plot_frequencies(result = pop)
#' @export
plot_frequencies <- function(result,
                             locations = seq(0, 1, length.out = 100),
                             progress_bar = FALSE) {

  to_plot <- calculate_allele_frequencies(result,
                                          locations,
                                          progress_bar)

  p1 <- ggplot2::ggplot(to_plot,
                        ggplot2::aes(x = .data[["location"]],
                                     y = .data[["frequency"]],
                                     colour = as.factor(.data[["ancestor"]]))) +
    ggplot2::geom_step()

  if (max(to_plot$location) < 10) {
    p1 <- p1 +
      ggplot2::xlab("Location (Morgan)")
  } else {
    p1 <- p1 +
      ggplot2::xlab("Location (bp)")
  }


  p1 <- p1 +
    ggplot2::ylab("Frequency") +
    ggplot2::labs(col = "Ancestor")

  return(p1)
}

#' Create isofemale
#' @description Creates isofemale individuals, given a population
#' @param input_data Source population from which isofemales are generated
#' @param n Number of isofemales to be generated
#' @param inbreeding_pop_size Population size of the population used to generate
#' homozygous individuals
#' @param run_time Maximum runtime used for inbreeding
#' @param morgan Size of the chromosome in Morgan (e.g. the number of crossovers
#' during meiosis)
#' @param recombination_rate rate in cM / Mbp, used to map recombination to the
#' markers. If the recombination_rate is not set, the value for Morgan is used,
#' assuming that the markers included span an entire chromosome.
#' @param num_threads number of threads. Default is 1. Set to -1 to use all
#' available threads
#' @param verbose Displays verbose output if TRUE. Default value is FALSE
#' @details To create an isofemale, two individuals are randomly picked from
#' the source population. Using these two individuals, a new population is
#' seeded, of size \code{inbreeding_pop_size}. Then, this population is allowed
#' to inbreed until either \code{run_time} is reached, or until all individuals
#' are homozygous and genetically identical, whatever happens first.
#' @return A list of length \code{n}, where each entry is a fully homozygous
#' isofemale.
#' @export
iso_female_sequence <- function(input_data = NA,
                                n = 1,
                                inbreeding_pop_size = 100,
                                run_time = 2000,
                                morgan = 1,
                                recombination_rate = NA,
                                num_threads = 1,
                                verbose = FALSE) {

  input_data <- verify_genomeadmixr_data(input_data)

  indivs <- 1:(length(input_data$genomes[, 1]) /  2)

  # first we select the individuals that will be the parents of the isofemales
  indices <- sample(indivs,
                    n * 2,
                    replace = FALSE)

  # for each isofemale
  # pick the female, then simulate until run_time
  # then pick a random individual (assuming the population is fixed now)

  output_females <- list()
  for (i in 1:n) {
    parents <- list()

    index_indiv_1 <- 1 + (indices[i] - 1) * 2
    index_indiv_2 <- 1 + (indices[i + n] - 1) * 2


    parents$markers <- input_data$markers
    parents$genomes <- rbind(input_data$genomes[index_indiv_1, ],
                             input_data$genomes[index_indiv_1 + 1, ],
                             input_data$genomes[index_indiv_2, ],
                             input_data$genomes[index_indiv_2 + 1, ])
    class(parents) <- "genomeadmixr_data"

    inbred_population <- simulate_admixture(module = sequence_module(molecular_data = parents,
                                                                     morgan = morgan,
                                                                     recombination_rate = recombination_rate),
                                            pop_size = inbreeding_pop_size,
                                            total_runtime = run_time,
                                            verbose = verbose)
    output_females[[i]] <-
      inbred_population$population[[
        sample(seq_along(inbred_population$population), 1)
      ]]

    class(output_females[[i]]) <- "individual"
  }
  return(output_females)
}

#' read sequence data from file to be used in simulation
#' @description Create data in a format that can be used by GenomeAdmixR
#' @param file_names names of input files
#' @param type type of data, options are 'ped' and 'vcf'
#' @param chosen_chromosome GenomeAdmixR simulates only a single chromosome.
#' @param number_of_snps number of snps to be loaded from file, default
#' is to load all snps
#' @param random_snps if a subset of all snps has to be taken, should these
#' be sampled sequentially (e.g. the first 100 snps) or randomly (100 randomly
#' sampled snps) (examples are for 'number_of_snps' = 100).
#' @param verbose give verbose output
#' @return list with two properties: \code{genomes} a matrix with the
#' sequence translated to numerics, such that [actg] corresponds to [1234], and
#' missing data is represented with "-". Rows in the matrix correspond to
#' chromosomes, and columns represent bases. Two consecutive rows represent an
#' individual, such that rows 1-2 are individual, rows 3-4 are one individual
#' etc. \code{markers} corresponds to the locations of the markers (in bp) on
#' the chosen chromosome.
#' @export
read_input_data <- function(file_names,
                            type,
                            chosen_chromosome,
                            number_of_snps = NA,
                            random_snps = TRUE,
                            verbose = FALSE) {
  input_data <- c()
  if (type == "ped") {
    message("reading plink style data: ped/map pair")
    input_data <- read_ped(file_names[1], file_names[2], chosen_chromosome)
  }
  if (type == "vcf") {
    message("reading vcf data")
    input_data <- read_vcf(file_names[1], chosen_chromosome,
                           number_of_snps,
                           random_snps,
                           verbose)
  }
  return(input_data)
}

sample_output_matrix <- function(input_data_list,
                                 frequencies,
                                 pop_size) {

  num_markers <- length(input_data_list[[1]]$markers)

  output_matrix <- matrix(NA,
                          nrow = pop_size * 2,
                          ncol = num_markers)

  for (indiv in 1:pop_size) {
    chosen_pop <- sample(seq_along(input_data_list), size = 1,
                         prob = frequencies)

    focal_pop <- input_data_list[[chosen_pop]]$genomes
    indivs <- seq(from = 1, to = length(focal_pop[, 1]), by = 2)
    sampled_indiv <- sample(indivs, size = 1)
    indiv_index <- 1 + (indiv - 1) * 2
    output_matrix[indiv_index, ] <- focal_pop[sampled_indiv, ]
    output_matrix[indiv_index + 1, ] <- focal_pop[sampled_indiv + 1, ]
  }
  return(output_matrix)
}

#' combine sequence data that was previously read from file into a population
#' @description Create data in a format that can be used by GenomeAdmixR,
#' entries are sampled randomly from each input data set, with replacement.
#' Probability of sampling from each input data set is driven by the input
#' frequencies, and total number of individuals sampled is driven by pop_size.
#' @param input_data_list list where each entry is the result of
#' \code{create_input_data}
#' @param frequencies frequency of each entry in the list in the starting
#' population
#' @param pop_size intended population size
#' @return the input data entries are combined to one single population that can
#' be used to seed \code{simulate_admixture_data}. Output is identical to
#' \code{create_input_data}
#' @export
combine_input_data <- function(input_data_list,
                               frequencies = NA,
                               pop_size) {

  if (length(frequencies) == 1) {
    if (is.na(frequencies)) {
      frequencies <- rep(1 / length(input_data_list),
                         length(input_data_list))
    }
  }

  testit::assert(is.list(input_data_list))
  for (i in seq_along(input_data_list)) {
    testit::assert(inherits(input_data_list[[i]], "genomeadmixr_data"))
  }


  testit::assert(length(frequencies) == length(input_data_list))

  for (i in seq_along(input_data_list))  {
    for (j in seq_along(input_data_list)) {
      if (!all.equal(input_data_list[[i]]$markers,
                     input_data_list[[j]]$markers)) {
        stop("all input data sets need to use the same marker positions")
      }
    }
  }

  if (sum(frequencies) != 1) {
    message("frequencies normalized")
    frequencies <- frequencies / sum(frequencies)
  }

  output <- list()
  output$genomes <- sample_output_matrix(input_data_list,
                                         frequencies,
                                         pop_size)
  output$markers <- input_data_list[[1]]$markers
  class(output) <- "genomeadmixr_data"
  return(output)
}

#' @keywords internal
convert_dna_to_numeric <- function(dna_matrix) {

  dna_matrix <- as.matrix(dna_matrix)
  dna_matrix[dna_matrix == "a"] <- 1
  dna_matrix[dna_matrix == "A"] <- 1
  dna_matrix[dna_matrix == "c"] <- 2
  dna_matrix[dna_matrix == "C"] <- 2
  dna_matrix[dna_matrix == "t"] <- 3
  dna_matrix[dna_matrix == "T"] <- 3
  dna_matrix[dna_matrix == "TRUE"] <- 3
  dna_matrix[dna_matrix == "g"] <- 4
  dna_matrix[dna_matrix == "G"] <- 4
  dna_matrix[is.na(dna_matrix)] <- 0
  dna_matrix[dna_matrix == "0"] <- 0
  dna_matrix[dna_matrix == "?"] <- 0
  dna_matrix[dna_matrix == "N"] <- 0

  odd_entries <- which(!(dna_matrix %in% c(0, 1, 2, 3, 4)))

  if (length(odd_entries) > 0) {
    message(paste0("found ", length(odd_entries), " non-biallelic entries"))
    message(paste0("these were set to missing data"))
    dna_matrix[odd_entries] <- 0
  }

  return(dna_matrix)
}

#' function to convert ped/map data to genome_admixr_data
#' @param simulation_data result of simulate_admixture
#' @param markers vector of locations of markers (in Morgan). If no vector is
#' provided, the function searches for marker locations in the simulation_data.
#' @param verbose provide verbose output (default is FALSE)
#' @return genomeadmixr_data object ready for simulate_admixture_data
#' @export
simulation_data_to_genomeadmixr_data <- function(simulation_data, # nolint
                                                 markers = NA,
                                                 verbose = FALSE) {

  output <- list()
  if (sum(is.na(markers)) > 0) {
    output$markers <- sort(unique(simulation_data$frequencies$location))
    if (is.null(output$markers)) {
      stop("no markers found, either provide them as argument, or make sure
           the simulation used to generate the data included markers")
    }
  } else {
    output$markers <- sort(unique(markers))
  }

  if (length(markers) > 1) {
    if (max(output$markers) <= 1.0) {
      # we have to rescale the markers to bp
      mark <- output$markers[output$markers > 0]
      min_mark <- min(mark)
      rescale_val <- 1 / min_mark
      output$markers <- output$markers * rescale_val
    }
  }

  if (!inherits(simulation_data, "population")) {
    if (is.list(simulation_data)) {
      if (inherits(simulation_data, "individual")) {
         indiv <- simulation_data
         simulation_data <- list()
         simulation_data[[1]] <- indiv
         class(simulation_data) <- "population"
      } else {
        if (inherits(simulation_data$population, "population")) {
          simulation_data <- simulation_data$population
        }
      }
    }
  }


  pop_for_cpp <- population_to_vector(simulation_data)
  gen_mat <- simulation_data_to_genomeadmixr_data_cpp(pop_for_cpp,
                                                      output$markers)

  output$genomes <- gen_mat
  if (sum(is.na(output$genomes)) > 0) {
    stop("NA values remain in genome matrix")
  }

  class(output) <- "genomeadmixr_data"
  return(output)
}

#' function to convert plink style (ped/map) data to genome_admixr_data
#' @param ped_data result of read.table(ped_file, header = F)
#' @param map_data result of read.table(map_file, header = F)
#' @param chosen_chromosome chromosome of choice
#' @param verbose verbose output
#' @return genomeadmixr_data object ready for simulate_admixture_data
#' @export
plink_to_genomeadmixr_data <- function(ped_data,  # nolint
                                       map_data,
                                       chosen_chromosome,
                                       verbose = FALSE) {
  # Base R
  map_data_duplicated <- map_data[rep(seq_len(nrow(map_data)), each = 2), ]

  map_indices <- which(map_data[, 1] == chosen_chromosome)
  map_data <- map_data[map_indices, ]

  ped_indices <- which(map_data_duplicated[, 1] == chosen_chromosome)
  ped_data <- ped_data[, 6 + ped_indices]

  # we have to first convert all letters to numbers,
  # then undouble
  if (verbose) message("converting atcg/ATCG entries to 1/2/3/4")
  ped_data <- convert_dna_to_numeric(ped_data)

  if (verbose) message("converting to numeric")
  ped_data <- as.matrix(ped_data,
                        nrow = length(ped_data[, 1]),
                        ncol = length(ped_data[1, ]))

  to_numeric <- function(v) {
    return(as.numeric(v))
  }
  ped_data <- apply(ped_data, 2, to_numeric)


  if (verbose) message("splitting markers into one row per chromosome")
  num_indiv <- length(ped_data[, 1])
  num_markers <- length(ped_data[1, ]) / 2

  output_matrix <- matrix(NA, ncol = num_markers,
                          nrow = num_indiv * 2)

  odd_indices <- seq(1, length(ped_data[1, ]), by = 2)
  even_indices <- seq(2, length(ped_data[1, ]), by = 2)

  for (j in seq_along(ped_data[, 1])) {
    chrom1 <- ped_data[j, odd_indices]
    chrom2 <- ped_data[j, even_indices]
    index1 <- 1 + (j - 1) * 2
    index2 <- index1 + 1
    output_matrix[index1, ] <- chrom1
    output_matrix[index2, ] <- chrom2
  }

  if (verbose) message("done")
  output <- list()
  output$genomes <- output_matrix
  output$markers <- map_data[, 4]

  class(output) <- "genomeadmixr_data"
  return(output)
}

#' @keywords internal
read_ped <- function(ped_name, map_name, chosen_chromosome) {
  message(paste("reading ped file:", ped_name))
  ped_data <- utils::read.table(ped_name, header = F)
  message(paste("reading map file:", map_name))
  map_data <- utils::read.table(map_name, header = F)

  return(plink_to_genomeadmixr_data(ped_data,
                                    map_data,
                                    chosen_chromosome))
}

convert_vcf_to_alleles <- function(v) {
  if (is.na(v[[1]])) {
    return(c(0, 0))
  }

  available_alleles <- c(v[[2]], v[[3]])  # using ref / alt
  alleles <- stringr::str_split(v[[1]], pattern = "/")
  output <- c(NA, NA)
  for (i in 1:2) {
    output[i] <- available_alleles[1 + as.numeric(alleles[[1]][i])]
  }
  if (output[1] != output[2]) {
    # we don't know the phasing, so we shuffle
    if (stats::runif(1, 0, 1) < 0.5) {
      output <- rev(output)
    }
  }
  return(output)
}

convert_map_data_to_numeric <- function(map_data) {
  map_data <- map_data[, c(4, 5)]
  map_data[map_data == "a"] <- 1
  map_data[map_data == "A"] <- 1
  map_data[map_data == "c"] <- 2
  map_data[map_data == "C"] <- 2
  map_data[map_data == "t"] <- 3
  map_data[map_data == "T"] <- 3
  map_data[map_data == "TRUE"] <- 3
  map_data[map_data == "g"] <- 4
  map_data[map_data == "G"] <- 4
  map_data[is.na(map_data)] <- 0
  map_data[map_data == "0"] <- 0
  map_data[map_data == "?"] <- 0
  map_data[map_data == "N"] <- 0

  odd_entries <- which(!(map_data[, 1] %in% c("0", "1", "2", "3", "4")))
  odd_entries2 <- which(!(map_data[, 2] %in% c("0", "1", "2", "3", "4")))
  odd_entries <- unique(c(odd_entries, odd_entries2))

  map_data[odd_entries, 1] <- -1
  if (sum(map_data[, 1] == "-1") > 0) {
    message(paste0("found ", sum(map_data[, 1] == "-1"), " INDELs"))
    message(paste0("these were removed from the dataset"))
  }

  return(map_data)
}


#' function to convert a vcfR object to genome_admixr_data
#' @param vcfr_object result of vcfR::read.vcfR
#' @param chosen_chromosome chromosome of choice
#' @param number_of_snps number of snps to be loaded from the vcf file, default
#' is to load all snps
#' @param random_snps if a subset of all snps has to be taken, should these
#' be sampled sequentially (e.g. the first 100 snps) or randomly (100 randomly
#' sampled snps) (examples are for 'number_of_snps' = 100).
#' @param verbose if true, print progress bar
#' @return genomeadmixr_data object ready for simulate_admixture_data
#' @export
vcfR_to_genomeadmixr_data <- function(vcfr_object, chosen_chromosome,  # nolint
                                      number_of_snps = NA,
                                      random_snps = TRUE,
                                      verbose = FALSE) {

  if (!inherits(vcfr_object, "vcfR")) {
    stop("input has to be of class vcfR")
  }

  # now need to extract relevant data
  indices <- which(vcfr_object@fix[, 1] == chosen_chromosome)

  map_data <- convert_map_data_to_numeric(vcfr_object@fix[indices, ])
  to_remove <- which(map_data[, 1] == "-1")

  genome_data <- vcfr_object@gt[indices, ]
  genome_data <- genome_data[, -1]
  genome_data <- genome_data[-to_remove, ]

  marker_data <- vcfr_object@fix[indices, 2]
  marker_data <- marker_data[-to_remove]

  map_data <- map_data[-to_remove, ]
  v1 <- as.numeric(map_data[, 1])
  v2 <- as.numeric(map_data[, 2])
  map_data <- cbind(v1, v2)  # this is an ugly solution... but it works?

  if (!is.na(number_of_snps)) {
    # subsample snps
    snp_indices <- seq_along(map_data[, 1])
    if (random_snps) {
      snp_indices <- sort(sample(snp_indices, size = number_of_snps))
    } else {
      snp_indices <- 1:number_of_snps
    }
    map_data <- map_data[snp_indices, ]
    genome_data <- genome_data[snp_indices, ]
    marker_data <- marker_data[snp_indices]
  }

  if (verbose) message("extracting genotypes")

  numeric_matrix_for_cpp <- convert_to_numeric_matrix(genome_data)
  genome_matrix <- vcf_to_matrix_cpp(numeric_matrix_for_cpp,
                                     map_data[, 1],
                                     map_data[, 2])

  if (verbose) message("done")
  output <- list()
  output$genomes <- genome_matrix
  output$markers <- as.numeric(marker_data)
  class(output) <- "genomeadmixr_data"
  return(output)
}


#' @keywords internal
read_vcf <- function(vcf_name, chosen_chromosome,
                     number_of_snps,
                     random_snps, verbose = FALSE) {
  if (verbose) message("reading vcf file")
  vcf_data <- vcfR::read.vcfR(vcf_name)

  return(vcfR_to_genomeadmixr_data(vcf_data, chosen_chromosome,
                                   number_of_snps,
                                   random_snps, verbose))

}

#' function to generate artificial genomeadmixr_data
#' @param number_of_individuals number of individuals
#' @param marker_locations location of markers, either in bp or Morgan
#' @param used_nucleotides subset or full set of [1/2/3/4] (reflecting a/c/t/g)
#' @param nucleotide_frequencies frequencies of the used nucleotides, if not
#' provided, equal frequencies are assumed.
#' @return genomeadmixr_data object ready for simulate_admixture_data
#' @export
create_artificial_genomeadmixr_data <- function(number_of_individuals, # nolint
                                                marker_locations,
                                                used_nucleotides = 1:4,
                                                nucleotide_frequencies = NA) {

  if (is.na(nucleotide_frequencies)) {
    nucleotide_frequencies <- rep(1, length(used_nucleotides))
    nucleotide_frequencies <-
      nucleotide_frequencies / sum(nucleotide_frequencies)
  }
  num_markers <- length(marker_locations)
  fake_data <- list()
  fake_data$markers <- marker_locations

  if (length(used_nucleotides) == 1) {
    fake_data$genomes <- matrix(data = used_nucleotides,
                                nrow = number_of_individuals * 2,
                                ncol = num_markers)
  } else {
    fake_data$genomes <- matrix(data = sample(x = used_nucleotides,
                                              size = number_of_individuals * 2 *
                                                num_markers,
                                              replace = T),
                                nrow = number_of_individuals * 2,
                                ncol = num_markers)
  }

  class(fake_data) <- "genomeadmixr_data"
  return(fake_data)
}

#' function to write simulation output as PLINK style data
#' @param input_pop input population, either of class "population" or of class
#' "genomeadmixr_data"
#' @param marker_locations location of markers, in bp
#' @param file_name_prefix prefix of the ped/map files.
#' @param chromosome chromosome indication for map file
#' @param recombination_rate recombination rate in cM / kb
#' @return No return value
#' @export
write_as_plink <- function(input_pop,
                           marker_locations,
                           file_name_prefix,
                           chromosome = 1,
                           recombination_rate = 1) {


  if (is.list(input_pop)) {
    if (inherits(input_pop$population, "population")) {
      input_pop <- input_pop$population
    }
  }

  if (inherits(input_pop, "population")) {
    pop_for_cpp <- population_to_vector(input_pop)
    input_pop$genomes <- simulation_data_to_plink_cpp(pop_for_cpp,
                                                      marker_locations)
  } else {
    stop("input pop has to be of class population")
  }


  family_id <- "SIM"
  indiv_id <- paste0("indiv_", seq_along(input_pop$genomes[, 1]))
  paternal_id <- 0
  maternal_id <- 0
  sex  <- 0
  phenotype <- -9

  ped_table <- cbind(family_id,
                     indiv_id,
                     paternal_id,
                     maternal_id,
                     sex,
                     phenotype,
                     input_pop$genomes)

  utils::write.table(ped_table, file = paste0(file_name_prefix, ".ped"),
              quote = FALSE, row.names = FALSE, col.names = FALSE)
  message("ped info written to: ", paste0(file_name_prefix, ".ped"))

  identifier <- paste0("rs", seq_along(marker_locations))
  recom_pos <- create_recombination_map(marker_locations, recombination_rate)
  recom_pos <- cumsum(recom_pos)

  output_matrix <- cbind(chromosome, identifier, recom_pos, marker_locations)
  utils::write.table(output_matrix, file = paste0(file_name_prefix, ".map"),
              quote = FALSE, row.names = FALSE, col.names = FALSE)
  message("map info written to: ", paste0(file_name_prefix, ".map"))
}

context("test plink data")

test_that("plink data", {

  chosen_markers <- 1:100

  fake_input_data1 <- create_artificial_genomeadmixr_data(
    number_of_individuals = 100,
    marker_locations = chosen_markers,
    used_nucleotides = 1
  )


  recombination_rate <- 2

  simulation_result <- simulate_admixture(module =
                                            sequence_module(molecular_data =
                                                              fake_input_data1,
                                                            recombination_rate =
                                                              recombination_rate),
                                               pop_size = 1000,
                                               total_runtime = 10)

  write_as_plink(input_pop = simulation_result$population,
                 marker_locations = chosen_markers,
                 file_name_prefix = "plink_test",
                 recombination_rate = recombination_rate)

  read_result <- read_input_data(file_names = c("plink_test.ped",
                                                "plink_test.map"),
                                   chosen_chromosome = 1,
                                   type = "ped",
                                   verbose = TRUE)

  genomeadmixr_data <-
    simulation_data_to_genomeadmixr_data(simulation_result$population,
                                         markers = chosen_markers)

  testthat::expect_true(all.equal(genomeadmixr_data$markers,
                                  read_result$markers))

  testthat::expect_true(all.equal(genomeadmixr_data$genomes,
                                  read_result$genomes))

  testthat::expect_true(file.remove("plink_test.ped"))
  testthat::expect_true(file.remove("plink_test.map"))
  testthat::expect_false(file.exists("plink_test.ped"))
  testthat::expect_false(file.exists("plink_test.map"))
})

context("test input data")

test_that("input data", {

  chosen_markers <- 1:100

  fake_input_data1 <- GenomeAdmixR::create_artificial_genomeadmixr_data(
    number_of_individuals = 100,
    marker_locations = chosen_markers,
    used_nucleotides = 1
  )

  fake_input_data2 <- GenomeAdmixR::create_artificial_genomeadmixr_data(
    number_of_individuals = 100,
    marker_locations = chosen_markers,
    used_nucleotides = 2
  )

  combined_data <- combine_input_data(input_data_list = list(fake_input_data1,
                                                             fake_input_data2),
                                      frequencies = c(0.3, 0.7),
                                      pop_size = 1000)

  vv <- table(combined_data$genomes)
  a <- vv[[1]] / (sum(vv))
  testthat::expect_equal(a, 0.3, tolerance = 0.1)

  testthat::expect_error(
    combined_data <- combine_input_data(input_data_list =
                                          list(fake_input_data1,
                                               fake_input_data2),
                                        frequencies = c(0.1, 0.7, 0.2),
                                        pop_size = 1000)
  )

  chosen_markers <- 1:1000

  fake_input_data1 <- GenomeAdmixR::create_artificial_genomeadmixr_data(
    number_of_individuals = 100,
    marker_locations = chosen_markers,
    used_nucleotides = 1
  )

  fake_input_data2 <- GenomeAdmixR::create_artificial_genomeadmixr_data(
    number_of_individuals = 100,
    marker_locations = chosen_markers,
    used_nucleotides = 2
  )

  combined_data <- combine_input_data(input_data_list = list(fake_input_data1,
                                                             fake_input_data2),
                                      frequencies = c(0.5, 0.5),
                                      pop_size = 100)

  simulation_result <- simulate_admixture(module = sequence_module(molecular_data =
                                                                     combined_data,
                                                                   morgan = 1),
                                               pop_size = 1000,
                                               total_runtime = 10)

  if (requireNamespace("junctions")) {

    num_j <- c()
    for (i in seq_along(simulation_result$population)) {
      focal_indiv <- simulation_result$population[[i]]
      chrom1 <- focal_indiv$chromosome1
      chrom2 <- focal_indiv$chromosome2
      num_j_1 <- sum(abs(diff(chrom1[, 2])))
      num_j_2 <- sum(abs(diff(chrom2[, 2])))
      num_j <- c(num_j, num_j_1, num_j_2)
    }
    expected_j <- junctions::number_of_junctions(N = 1000, R = 1000, t = 10)
    testthat::expect_equal(mean(num_j), expected_j, tolerance = 1)
  }
})

test_that("input data simulation", {
  vx <- simulate_admixture(pop_size = 100,
                           total_runtime = 100)

  vy <- simulation_data_to_genomeadmixr_data(vx,
                                              markers = seq(0, 1,
                                                            length.out = 1000))

  for (i in seq_along(vx$population)) {
    num_j_1 <- sum(abs(diff(vx$population[[i]]$chromosome1)))
    num_j_2 <- sum(abs(diff(vx$population[[i]]$chromosome2)))

    index_c1 <- 1 + (i - 1) * 2
    index_c2 <- index_c1 + 1
    nj1 <- sum(abs(diff(vy$genomes[index_c1, ])))
    nj2 <- sum(abs(diff(vy$genomes[index_c2, ])))
    testthat::expect_lte(nj1, num_j_1)
    testthat::expect_lte(nj2, num_j_2)
  }
})

context("test simulate admixture data")

test_that("simulate_admixture_data", {

  num_markers <- 100
  num_indiv <- 100
  chosen_markers <- 1:num_markers

  fake_input_data1 <- create_artificial_genomeadmixr_data(
    number_of_individuals = num_indiv,
    marker_locations = chosen_markers,
    used_nucleotides = 1:4
  )

  fake_input_data2 <- create_artificial_genomeadmixr_data(
    number_of_individuals = num_indiv,
    marker_locations = chosen_markers,
    used_nucleotides = 1:4
  )

  simul_pop <- simulate_admixture(module = sequence_module(
                                    molecular_data = list(fake_input_data1,
                                                          fake_input_data2),
                                    initial_frequencies = c(0.5, 0.5),
                                    markers = chosen_markers,
                                    morgan = 1),
                                  pop_size = 100,
                                  total_runtime = 100)

  testthat::expect_silent(
    plot_chromosome(simul_pop$population[[1]]$chromosome1)
  )

  testthat::expect_silent(
    plot_difference_frequencies(results = simul_pop)
  )
  testthat::expect_silent(
    calculate_allele_frequencies(source_pop = simul_pop,
                                 progress_bar = FALSE)

  )
  testthat::expect_silent(
    plot_frequencies(simul_pop,
                     locations = unique(simul_pop$frequencies$location))
  )
  testthat::expect_silent(
    plot_over_time(simul_pop$frequencies, focal_location = 50)
  )
  testthat::expect_silent(
    plot_start_end(simul_pop)
  )

  testthat::expect_silent(
    calculate_heterozygosity(simul_pop$population,
                             locations = unique(simul_pop$frequencies$location))
  )

  testthat::expect_silent(
    plot_joyplot_frequencies(simul_pop$frequencies,
                             time_points = c(0, 10, 50))
  )

  testthat::expect_silent(
    calculate_ld(simul_pop$population)
  )
  testthat::expect_silent(
    calculate_marker_frequency(simul_pop, location = 50)
  )
})

test_that("simulate_admixture_data_mutation", {

  num_markers <- 100
  num_indiv <- 100
  chosen_markers <- 1:num_markers

  fake_input_data1 <- create_artificial_genomeadmixr_data(
    number_of_individuals = num_indiv,
    marker_locations = chosen_markers,
    used_nucleotides = 1:2
  )

  fake_input_data2 <- create_artificial_genomeadmixr_data(
    number_of_individuals = num_indiv,
    marker_locations = chosen_markers,
    used_nucleotides = 3:4
  )

  combined_data <- combine_input_data(input_data_list = list(fake_input_data1,
                                                             fake_input_data2),
                                      frequencies = c(0.5, 0.5),
                                      pop_size = 1000)

  sub_matrix <- matrix(0.25, nrow = 4, ncol = 4)

  testthat::expect_output(
    simul_pop <- simulate_admixture(module = sequence_module(
                                                molecular_data = combined_data,
                                                mutation_rate = 0.1,
                                                substitution_matrix = sub_matrix,
                                                markers = chosen_markers),
                                    pop_size = 100,
                                    total_runtime = 100,
                                    verbose = TRUE)
  )

  a1 <- simul_pop$initial_frequency
  a2 <- simul_pop$final_frequency

  aa <- a1 %>%
          dplyr::group_by(ancestor) %>%
    dplyr::summarise("mean_freq" = mean(frequency))

  bb <- a2 %>%
    dplyr::group_by(ancestor) %>%
    dplyr::summarise("mean_freq" = mean(frequency))

  testthat::expect_equal(mean(bb$mean_freq[2:5]), 0.25)

  bases <- c("a", "c", "t", "g")
  for (i in 1:4) {
    sub_matrix <- matrix(0, nrow = 4, ncol = 4)
    sub_matrix[, i] <- 1

    testthat::expect_output(
      testthat::expect_warning(
        simul_pop <- simulate_admixture(module = sequence_module(
                                                    molecular_data = combined_data,
                                                    mutation_rate = 0.1,
                                                    substitution_matrix = sub_matrix,
                                                    markers = chosen_markers),
                                        pop_size = 100,
                                        total_runtime = 100,
                                        verbose = TRUE)
      )
    )

    bb <- simul_pop$final_frequency %>%
      dplyr::group_by(ancestor) %>%
      dplyr::summarise("mean_freq" = mean(frequency))

    highest_base <- bb$ancestor[which.max(bb$mean_freq)]
    testthat::expect_equal(bases[i], highest_base)
  }
})

test_that("simulate_admixture_data_recombination_map", {
  num_markers <- 2
  num_indiv <- 10
  chosen_markers <- c(1000000, 2000000)
  recom_rate <- 1

  fake_input_data1 <- create_artificial_genomeadmixr_data(
    number_of_individuals = num_indiv,
    marker_locations = chosen_markers,
    used_nucleotides = 1
  )

  fake_input_data2 <- create_artificial_genomeadmixr_data(
    number_of_individuals = num_indiv,
    marker_locations = chosen_markers,
    used_nucleotides = 2
  )

  pop_size <- 10000
  simul_pop <- simulate_admixture(module = sequence_module(
                                              molecular_data = list(fake_input_data1,
                                                                    fake_input_data2),
                                              initial_frequencies = c(0.5, 0.5),
                                              markers = chosen_markers,
                                              recombination_rate = recom_rate),
                                  pop_size = pop_size,
                                  total_runtime = 2,
                                  verbose = FALSE)


  found_junctions <- c()
  for (i in seq_along(simul_pop$population)) {
    a <- simul_pop$population[[i]]$chromosome1[, 2]
    b <- simul_pop$population[[i]]$chromosome2[, 2]
    a <- sum(abs(diff(a)))
    b <- sum(abs(diff(b)))
    found_junctions <- c(found_junctions, a, b)
  }

  all_j <- sum(found_junctions)


  # expected number recombinations is:
  # 0.5: only half the population is admixing
  # 2  : 2 chromosomes
  # recom_rate / 100 : recom_rate is in cM, and we want in Morgan.
  # recom_rate per bp -> div by 1e6
  # then, num indivs (pop size) * 0.5 * recom_prob = expected junctions

  recom_prob <-  (recom_rate / 100) * diff(chosen_markers) / 1e6

  expected_num_j <- 2 * pop_size * recom_prob * 0.5

  testthat::expect_equal(all_j, expected_num_j, tolerance = 0.4)
})

context("simulate_admixture_until")

test_that("simulate_admixture_until", {
   vx <- simulate_admixture(total_runtime = 100,
                           migration = migration_settings(
                             population_size = c(50, 50),
                             initial_frequencies = list(c(0.5, 0.5),
                                                        c(0.5, 0.5)),
                             stop_at_critical_fst = TRUE,
                             generations_between_update = 5,
                             critical_fst = 0.2,
                             migration_rate = 0.001),
                           verbose = FALSE)

  fst_2 <- calculate_fst(vx$population_1,
                         vx$population_2,
                         sampled_individuals = 100,
                         number_of_markers = 100,
                         random_markers = TRUE)

  testthat::expect_true(vx$FST >= 0.01)
  testthat::expect_true(fst_2 >= 0.01)


  testthat::expect_equal(length(vx$population_1), 50)
  testthat::expect_equal(length(vx$population_2), 50)
  testthat::expect_true(verify_population(vx$population_1))
  testthat::expect_true(verify_population(vx$population_2))
  testthat::expect_true(length(all.equal(vx$population_1,
                                         vx$population_2)) > 10)
})

test_that("simulate_admixture_until_data", {

  num_indiv <- 100
  chosen_markers <- 1:100

  fake_input_data1 <-
    create_artificial_genomeadmixr_data(number_of_individuals = num_indiv,
                                        marker_locations = chosen_markers,
                                        used_nucleotides = 1)

  fake_input_data2 <-
    create_artificial_genomeadmixr_data(number_of_individuals = num_indiv,
                                        marker_locations = chosen_markers,
                                        used_nucleotides = 2)


  vx <- simulate_admixture(
    module = sequence_module(molecular_data =
                               list(fake_input_data1,
                                    fake_input_data2),
                             markers = chosen_markers,
                             morgan = 1),
    migration = migration_settings(
      population_size = c(100, 100),
      migration_rate = 0.001,
      critical_fst = 0.2,
      generations_between_update = 10),
    total_runtime = 100)



  fst_2 <- calculate_fst(vx$population_1,
                         vx$population_2,
                         sampled_individuals = 10,
                         number_of_markers = 100,
                         random_markers = TRUE)

  testthat::expect_true(vx$FST >= 0.05)
  testthat::expect_true(fst_2 >= 0.05)


  testthat::expect_equal(length(vx$population_1), 100)
  testthat::expect_equal(length(vx$population_2), 100)
  testthat::expect_true(length(all.equal(vx$population_1,
                                         vx$population_2)) > 10)
})

#' @keywords internal
simulate_sequence_until <- function(input_data_population_1 = NA,
                                    input_data_population_2 = NA,
                                    pop_size = c(100, 100),
                                    total_runtime = 100,
                                    morgan = 1,
                                    recombination_rate = NA,
                                    num_threads = 1,
                                    select_matrix = NA,
                                    markers = NA,
                                    verbose = FALSE,
                                    multiplicative_selection = TRUE,
                                    migration_rate = 0.0,
                                    generations_between_update = 100,
                                    critical_fst = 0.1,
                                    sampled_individuals = 10,
                                    number_of_markers = 100,
                                    random_markers = TRUE,
                                    mutation_rate = 0,
                                    substitution_matrix =
                                      matrix(1 / 4, nrow = 4, ncol = 4)) {
  set_num_threads(num_threads)
  pops <- simulate_sequence_migration(
    input_data_population_1 = input_data_population_1,
    input_data_population_2 = input_data_population_2,
    pop_size = pop_size,
    total_runtime = generations_between_update,
    num_threads = num_threads,
    morgan = morgan,
    recombination_rate = recombination_rate,
    select_matrix = select_matrix,
    markers = markers,
    verbose = verbose,
    multiplicative_selection = multiplicative_selection,
    migration_rate = migration_rate,
    mutation_rate = mutation_rate,
    substitution_matrix = substitution_matrix)

  fst <- calculate_fst(pops$population_1, pops$population_2,
                       sampled_individuals = sampled_individuals,
                       number_of_markers = number_of_markers,
                       random_markers = random_markers)

  if (verbose) message("Number of Generations\tFST\n")
  if (verbose) message(generations_between_update, "\t", fst, "\n")

  total_generations <- generations_between_update
  while (fst < critical_fst && total_generations < total_runtime) {

    pop1_for_data_cpp <-
      simulation_data_to_genomeadmixr_data(pops$population_1,
                                           input_data_population_1$markers)
    pop2_for_data_cpp <-
      simulation_data_to_genomeadmixr_data(pops$population_2,
                                           input_data_population_1$markers)

    pops <- simulate_sequence_migration(
      input_data_population_1 = pop1_for_data_cpp,
      input_data_population_2 = pop2_for_data_cpp,
      pop_size = pop_size,
      total_runtime = generations_between_update,
      morgan = morgan,
      recombination_rate = recombination_rate,
      num_threads = num_threads,
      select_matrix = select_matrix,
      markers = markers,
      verbose = verbose,
      multiplicative_selection = multiplicative_selection,
      migration_rate = migration_rate,
      mutation_rate = mutation_rate,
      substitution_matrix = substitution_matrix)

    fst <- calculate_fst(pops$population_1, pops$population_2,
                         sampled_individuals = sampled_individuals,
                         number_of_markers = number_of_markers,
                         random_markers = random_markers)

    total_generations <- total_generations + generations_between_update
    message(total_generations, "\t", fst, "\n")
  }
  reset_num_threads()
  return(list("population_1" = pops$population_1,
              "population_2" = pops$population_2,
              "Number_of_generations" = total_generations,
              "FST" = fst))
}

#' Calculate allele frequencies
#' @description Calculate for a number of regularly spaced markers the relative
#' frequency of each ancestor in the population.
#' @param source_pop Population for which to estimate allele frequencies
#' @param locations A vector indicating the locations (in Morgan) where
#' to calculate the allele frequencies.
#' @param progress_bar Displays a progress_bar if TRUE. Default value is TRUE
#' @details Markers are equidistantly spaced, with a distance of
#' \code{step_size} in between them.
#' @return A tibble containing the allele frequencies
#' @examples
#' number_founders = 20
#' wildpop =  simulate_admixture(
#'    module = ancestry_module(number_of_founders = 20, morgan = 1),
#'    pop_size = 1000,
#'    total_runtime = 10,
#'    num_threads = 1)
#'
#' freq_output <- calculate_allele_frequencies(wildpop,
#'                                             progress_bar = TRUE)
#'
#' require(ggplot2)
#' ggplot(freq_output, aes(x=location, y = frequency,
#'                         col = as.factor(ancestor))) +
#'   geom_line()
#' @export
calculate_allele_frequencies <- function(source_pop,
                                         locations = seq(0, 1,
                                                         length.out = 100),
                                         progress_bar = FALSE) {

  source_pop <- check_input_pop(source_pop)

  pop_for_cpp <- population_to_vector(source_pop)

  frequency_table <- calculate_allele_spectrum_cpp(pop_for_cpp,
                                                   locations,
                                                   progress_bar)

  output <- frequency_table
  colnames(output) <- c("time", "location", "ancestor", "frequency")
  output <- tibble::as_tibble(output)

  using_sequencing_data <- check_for_bases(source_pop)
  if (using_sequencing_data) {
    output$ancestor[output$ancestor == 0] <- 0
    output$ancestor[output$ancestor == 1] <- "a"
    output$ancestor[output$ancestor == 2] <- "c"
    output$ancestor[output$ancestor == 3] <- "t"
    output$ancestor[output$ancestor == 4] <- "g"
  }

  return(output)
}

#' Default parameter documentation
#'
#' This function's purpose is to list all parameter documentation to be
#' inherited by the relevant functions.
#'
#' @param module Chosen module to simulate, either created with
#' \code{module_ancestry} or
#' \code{module_sequence}.
#' @param pop_size The number of individuals in the population. If the number is
#' larger than the number of individuals in the input population (if provided),
#' additional individuals are sampled randomly from the input population to
#' reach the intended size.
#' @param total_runtime  Number of generations
#' @param migration settings associated with migration, should be created with
#' \code{\link{migration_settings}}
#' @param select_matrix Selection matrix indicating the markers which are under
#' selection. If not provided by the user, the simulation proceeds neutrally. If
#' provided, each row in the matrix should contain five entries:
#' \itemize{
#' \item{location of the marker under selection (in Morgan) }
#' \item{fitness of wildtype (aa)}
#' \item{fitness of heterozygote (aA)}
#' \item{fitness of homozygote mutant (AA)}
#' \item{Ancestral type that represents the mutant allele A}
#' }
#' @param multiplicative_selection Default: TRUE. If TRUE, fitness is calculated
#' for multiple markers by multiplying fitness values for each marker. If FALSE,
#' fitness is calculated by adding fitness values for each marker.
#' @param verbose Verbose output if TRUE. Default value is FALSE
#' @param num_threads number of threads. Default is 1. Set to -1 to use all
#' available threads
#' @param morgan Length of the chromosome in Morgan (e.g. the number of
#' crossovers during meiosis)
#' @param markers A vector of locations of markers (relative locations in
#' [0, 1]). If a vector is provided, ancestry at these marker positions is
#' tracked for every generation.
#' @param track_junctions Track the average number of junctions over time if
#' TRUE
#' @return Nothing
#' @keywords internal
#' @export
default_params_doc <- function(module,
                               total_runtime,
                               migration,
                               select_matrix,
                               pop_size,
                               multiplicative_selection,
                               verbose,
                               num_threads,
                               morgan,
                               markers,
                               track_junctions) {
  # Nothing
}

#' create sequence module
#' @description creates a sequence module, which contains all relevant
#' information in order to perform a simulation based on sequence data.
#' @param molecular_data Genomic data used as input, should be of type
#' genomeadmixr_data. Either a single dataset is provided, or a list of
#' multiple genomeadmixr_data objects.
#' @param initial_frequencies A vector describing the initial contribution of
#' each provided input data set to the starting hybrid swarm. By default, equal
#' frequencies are assumed. If a vector not summing to 1 is provided, the vector
#' is normalized.
#' @param morgan Length of the molecular sequence in Morgan (e.g. the number of
#' crossovers during meiosis), alternatively, the recombination rate can be
#' used, see below.
#' @param recombination_rate rate in cM / Mbp, used to map recombination to the
#' markers. If the recombination_rate is not set, the value for Morgan is used,
#' assuming that the markers included span an entire chromosome.
#' @param markers A vector of locations of markers, these markers are
#' tracked for every generation.
#' @param mutation_rate the per base probability of mutation. Default is 0.
#' @param substitution_matrix a 4x4 matrix representing the probability of
#' mutating to another base (where [1/2/3/4] = [a/c/t/g]), conditional on the
#' event of a mutation happening. Default is the JC69 matrix, with equal
#' probabilities for all transitions / transversions.
#' @return sequence module object, used as starting point for
#' simulate_admixture.
#' @export
sequence_module <- function(molecular_data = NA,
                            initial_frequencies = NA,
                            morgan = 1,
                            recombination_rate = NA,
                            markers = NA,
                            mutation_rate = 0,
                            substitution_matrix =
                              matrix(1 / 4, 4, 4)) {

  local_module <- list(input_data = molecular_data,
                       initial_frequencies = initial_frequencies,
                       morgan = morgan,
                       recombination_rate = recombination_rate,
                       markers = markers,
                       mutation_rate = mutation_rate,
                       substitution_matrix = substitution_matrix,
                       type  = "sequence")
  return(local_module)
}

context("general usage")

test_that("general usage", {
  data("dgrp2.3R.5k.data")
  mks <- sample(dgrp2.3R.5k.data$markers, size = 100,
                replace = FALSE, prob = NULL)


  testthat::expect_silent(
    simulated_pop <- simulate_admixture(
      module = sequence_module(molecular_data = dgrp2.3R.5k.data,
                               morgan = 1,
                               markers = mks),
      pop_size = 1000,
      total_runtime = 11)
  )

  ### The list option is working now with genomeadmixr_data type:
  testthat::expect_output(
    testthat::expect_message(
      simulated_pop_2 <- simulate_admixture(
        module = sequence_module(molecular_data =
                                   list(dgrp2.3R.5k.data,
                                        dgrp2.3R.5k.data),
                                 markers = mks,
                                 morgan = 1),
        pop_size = 100,
        total_runtime = 5,
        verbose = TRUE),
      "found multiple input populations"
    )
  )

  testthat::expect_message(
    testthat::expect_output(
      simulated_pop_2 <- simulate_admixture(
        module = sequence_module(molecular_data =
                                   list(simulated_pop,
                                        simulated_pop),
                                 morgan = 1,
                                 markers = mks),
        pop_size = 100,
        total_runtime = 5,
        verbose = TRUE)
    )
  )

  a1 <- calculate_marker_frequency(simulated_pop, location = mks[50])
  a2 <- subset(simulated_pop$frequencies, time == 10 & location == mks[50])

  for (x in unique(a1$ancestor)) {
    a3 <- subset(a1, a1$ancestor == x)
    if (x == 0) x = "-" # 0 is - in a2.
    a4 <- subset(a2, a2$ancestor == x)
    testthat::expect_equal(a3$frequency[1],
                           a4$frequency[1], tolerance = 0.05)
  }

  selection_matrix <- matrix(nrow = 1, ncol = 5)
  selection_matrix[1, ] <- c(mks[50], 0.4, 0.7, 1.0, "t")


  testthat::expect_silent(
    iso_100 <- create_iso_female(
      module = sequence_module(molecular_data = dgrp2.3R.5k.data,
                               morgan = 1),
      inbreeding_pop_size = 100,
      n = 20,
      run_time = 2)
  )

  testthat::expect_message(
    selected_pop <- simulate_admixture(
      module = sequence_module(molecular_data = simulated_pop,
                               morgan = 1,
                               markers = mks),
      pop_size = 100,
      total_runtime = 5,
      select_matrix = selection_matrix)
  )

  testthat::expect_output(
    testthat::expect_message(
      two_pops <- simulate_admixture(
        module = sequence_module(molecular_data = list(
          simulated_pop, simulated_pop),
          markers = mks,
          morgan = 1),
        verbose = TRUE,
        migration = migration_settings(migration_rate = 0,
                                       population_size = c(30, 30),
                                       stop_at_critical_fst = TRUE,
                                       critical_fst = 0.05,
                                       generations_between_update = 100),
        total_runtime = 5)
    )
  )
})

test_that("isofemale usage", {
  data("dgrp2.3R.5k.data")

  mks = sample(dgrp2.3R.5k.data$markers, size = 300, replace = FALSE, prob = NULL)

  testthat::expect_silent(
    simulated_pop <- simulate_admixture(
      module = sequence_module(molecular_data = dgrp2.3R.5k.data,
                               morgan = 1,
                               markers = mks),
      pop_size = 100,
      total_runtime = 10)
  )

  testthat::expect_output(
    isos <- create_iso_female(
      module = sequence_module(molecular_data = simulated_pop),
      n = 20,
      inbreeding_pop_size = 100,
      run_time = 50,
      verbose = TRUE)
  )

  testthat::expect_output(
    simulated_pop <- simulate_admixture(
      module = sequence_module(molecular_data = isos[1],
                               morgan = 1,
                               markers = mks),

      pop_size = 100,
      total_runtime = 10,
      verbose = TRUE)
  )
})

context("test simulate admixture data migration")

test_that("simulate_admixture_data", {
  num_markers <- 100
  num_indiv <- 100
  chosen_markers <- 1:num_markers

  fake_input_data1 <- list()
  fake_input_data1$genomes <- matrix(data = 1,
                                     nrow = num_indiv,
                                     ncol = num_markers)


  fake_input_data1$markers <- chosen_markers

  fake_input_data2 <- list()
  fake_input_data2$genomes <- matrix(data = 2,
                                     nrow = num_indiv,
                                     ncol = num_markers)
  fake_input_data2$markers <- chosen_markers

  class(fake_input_data1) <- "genomeadmixr_data"
  class(fake_input_data2) <- "genomeadmixr_data"

  simul_two_pop <- simulate_admixture(
    module = sequence_module(molecular_data = list(fake_input_data1,
                                                   fake_input_data2),
                             markers = chosen_markers),
    migration = migration_settings(
      population_size = c(100, 100),
      migration_rate = 0.0),

    total_runtime = 100)

  a <- simul_two_pop$initial_frequency
  a1 <- a %>%
    dplyr::group_by(population, ancestor) %>%
    dplyr::summarise("mean_freq" = mean(frequency))


  b <- simul_two_pop$final_frequency
  b1 <- b %>%
    dplyr::group_by(population, ancestor) %>%
    dplyr::summarise("mean_freq" = mean(frequency))

  testthat::expect_true(all.equal(a1, b1))

  a2 <- subset(a1, a1$population == 1 &
                 a1$ancestor == 1)
  testthat::expect_equal(a2$mean_freq, 1)
  b2 <- subset(b1, b1$population == 1 &
                 b1$ancestor == 1)
  testthat::expect_equal(b2$mean_freq, 1)


  simul_two_pop <- simulate_admixture(
    module = sequence_module(molecular_data = list(fake_input_data1,
                                                   fake_input_data2),
                             markers = chosen_markers),
    migration = migration_settings(population_size = c(100, 100),
                                   migration_rate = 0.5),
    total_runtime = 100)

  a <- simul_two_pop$initial_frequency
  a1 <- a %>%
    dplyr::group_by(population, ancestor) %>%
    dplyr::summarise("mean_freq" = mean(frequency))

  b <- simul_two_pop$final_frequency
  b1 <- b %>%
    dplyr::group_by(population, ancestor) %>%
    dplyr::summarise("mean_freq" = mean(frequency))

  v <- all.equal(a1, b1)
  testthat::expect_false(v == TRUE)
})

test_that("simulate_admixture_data with selection", {
  num_markers <- 100
  num_indiv <- 100
  chosen_markers <- 1:num_markers

  fake_input_data1 <- list()
  fake_input_data1$genomes <- matrix(data = 1,
                                     nrow = num_indiv,
                                     ncol = num_markers)


  fake_input_data1$markers <- chosen_markers

  fake_input_data2 <- list()
  fake_input_data2$genomes <- matrix(data = 2,
                                     nrow = num_indiv,
                                     ncol = num_markers)
  fake_input_data2$markers <- chosen_markers

  class(fake_input_data1) <- "genomeadmixr_data"
  class(fake_input_data2) <- "genomeadmixr_data"

  select_matrix <- matrix(ncol = 5, nrow = 1)
  s <- 0.1
  select_matrix[1, ] <- c(50, 1.0, 1 + 0.5 * s, 1 + s, 1)

  simul_two_pop <- simulate_admixture(
    module = sequence_module(molecular_data = list(fake_input_data1,
                                                   fake_input_data2),
                             markers = chosen_markers),
    migration = migration_settings(
      population_size = c(100, 100),
      migration_rate = 0.01),
    select_matrix = select_matrix,
    total_runtime = 100)

  a <- simul_two_pop$initial_frequency
  a1 <- a %>%
    dplyr::group_by(population, ancestor) %>%
    dplyr::summarise("mean_freq" = mean(frequency))

  b <- simul_two_pop$final_frequency
  b1 <- b %>%
    dplyr::group_by(population, ancestor) %>%
    dplyr::summarise("mean_freq" = mean(frequency))

  v <- all.equal(a1, b1)
  testthat::expect_false(v == TRUE)
})

#' Function to manage settings associated with migration
#' @description creates a list with settings associated with migration.
#' @param migration_rate  Rate of migration between the two populations.
#' Migration is implemented such that with probability m (migration rate) one
#' of the two parents of a new offspring is from the other population, with
#' probability 1-m both parents are of the focal population.
#' @param stop_at_critical_fst option to stop at a critical FST value
#' , default is FALSE
#' @param critical_fst the critical fst value to stop, if
#' \code{stop_simulation_at_critical_fst} is TRUE
#' @param population_size vector of population sizes, one size for each
#' population
#' @param initial_frequencies A list describing the initial frequency of each
#' ancestor in each population. Each entry in the list contains a vector with
#' the frequencies for all ancestor. The length of the vector indicates the
#' number of unique ancestors. If a vector not summing to 1 is provided, the
#' vector is normalized.
#' @param generations_between_update The number of generations after which the
#' simulation has to check again whether the critical Fst value is exceeded
#' @param sampled_individuals Number of individuals to be sampled at random from
#' the population to estimate Fst
#' @param number_of_markers Number of markers to be used to estimate Fst
#' @param random_markers Are the markers to estimate Fst randomly distributed,
#' or regularly distributed? Default is TRUE.
#' @return list with migration associated settings. To be used to pass on
#' migration settings to simulate_admixture.
#' @export
migration_settings <- function(migration_rate = NA,
                               stop_at_critical_fst = FALSE,
                               critical_fst = NA,
                               population_size = c(100, 100),
                               initial_frequencies = list(c(1.0, 0),
                                                          c(0, 1.0)),
                               generations_between_update = 10,
                               sampled_individuals = 10,
                               number_of_markers = 100,
                               random_markers = TRUE) {

  local_list = list(migration_rate = migration_rate,
                    stop_at_critical_fst = stop_at_critical_fst,
                    population_size = population_size,
                    initial_frequencies = initial_frequencies,
                    critical_fst = critical_fst,
                    generations_between_update = generations_between_update,
                    sampled_individuals = sampled_individuals,
                    number_of_markers = number_of_markers,
                    random_markers = random_markers,
                    type = "migration_settings")
  return(local_list)
}

#' Creates a module to start simulations tracking local ancestry
#' @description Module to perform simulations based on local ancestry
#' @param input_population Potential earlier simulated population used as
#' starting point for the simulation. If not provided by the user, the
#' simulation starts from scratch.
#' @param number_of_founders Number of unique ancestors / ancestries to be
#' tracked in the simulation
#' @param initial_frequencies A vector describing the initial frequency of each
#' ancestor / ancestry. By default, equal frequencies are assumed. If a vector
#' not summing to 1 is provided, the vector is normalized.
#' @param morgan Length of the genomic stretch simulated, expressed in Morgan
#' (e.g. the number of crossovers during meiosis)
#' @param markers A vector of locations of markers, with the location in Morgan.
#' Ancestry at these marker positions is tracked for every generation.
#' @param track_junctions Tracks the average number of junctions over time if
#' TRUE
#' @return list with type = "Ancestry". Can be used in simulate_admixture.
#' @export
ancestry_module <- function(input_population = NA,
                            number_of_founders = 2,
                            initial_frequencies = NA,
                            morgan = 1,
                            markers = NA,
                            track_junctions = FALSE) {

  if (sum(is.na(initial_frequencies))) {
    initial_frequencies <- rep(1.0 / number_of_founders,
                               times = number_of_founders)
  }

  if (sum(initial_frequencies) != 1) {
    initial_frequencies <- initial_frequencies / sum(initial_frequencies)
  }

  local_module <- list(input_population = input_population,
                       number_of_founders = number_of_founders,
                       initial_frequencies = initial_frequencies,
                       morgan = morgan,
                       markers = markers,
                       track_junctions = track_junctions,
                       type = "ancestry")

  return(local_module)
}

context("simulate_admixture")

test_that("simulate_admixture", {
  select_matrix <- matrix(NA, nrow = 2, ncol = 5)

  s <- 0.1
  select_matrix[1, ] <- c(0.5, 0.5, 0.5 + 0.5 * s, 0.5 + s, 0)
  select_matrix[2, ] <- c(0.6, 0.5, 0.5 + 0.5 * s, 0.5 + s, 0)

  testthat::expect_message(
    vx <- simulate_admixture(pop_size = 100,
                             module = ancestry_module(number_of_founders = 2,
                                                      morgan = 1),
                             total_runtime = 100,
                             select_matrix = select_matrix,
                             multiplicative_selection = FALSE)
  )
})


test_that("simulate admixture use", {
  testthat::expect_output(
    vx <- simulate_admixture(pop_size = 100,
                             module = ancestry_module(number_of_founders = 2,
                                                      morgan = 1,
                                                      track_junctions = FALSE),
                             total_runtime = 100,
                             select_matrix = NA,
                             verbose = TRUE,
                             multiplicative_selection = TRUE)
  )

  select_matrix <- matrix(NA, nrow = 1, ncol = 5)
  testthat::expect_message(
    testthat::expect_error(simulate_admixture(pop_size = 100,
                                              total_runtime = 100,
                                              select_matrix = select_matrix,
                                              multiplicative_selection = TRUE))
  )
  select_matrix <- matrix(NA, nrow = 1, ncol = 3)
  testthat::expect_message(
    testthat::expect_error(
      simulate_admixture(pop_size = 100,
                         total_runtime = 100,
                         select_matrix = select_matrix,
                         multiplicative_selection = TRUE)
    )
  )

  markers <- seq(from = 0.4, to = 0.6, length.out = 100)
  testthat::expect_silent(
    vx <- simulate_admixture(module = ancestry_module(markers = markers),
                             pop_size = 100,
                             total_runtime = 100,
                             select_matrix = NA,
                             multiplicative_selection = TRUE)
  )

  testthat::expect_silent(
    vx <- simulate_admixture(module = ancestry_module(track_junctions = TRUE),
                             pop_size = 100,
                             total_runtime = 100,
                             select_matrix = NA,
                             multiplicative_selection = TRUE)
  )

  testthat::expect_silent(
    vx <- simulate_admixture(module = ancestry_module(markers = markers,
                                                      track_junctions = TRUE),
                             pop_size = 100,
                             total_runtime = 100,
                             multiplicative_selection = TRUE)
  )

  testthat::expect_silent(
  vx <- simulate_admixture(module = ancestry_module(
                                        initial_frequencies = c(0.5, 0.5)),
                           pop_size = 100,
                           total_runtime = 100)
  )

  testthat::expect_silent(
    vx <- simulate_admixture(pop_size = 100,
                             total_runtime = 100)
  )
  markers <- 0.5
  testthat::expect_silent(
  vx <- simulate_admixture(module = ancestry_module(markers = markers),
                           pop_size = 100,
                           total_runtime = 100)
  )

  # code coverage for displaying functions:
  testthat::expect_silent(plot(vx$population[[1]]))

  testthat::expect_output(
    testthat::expect_equal(print(vx$population),
                           "Population with 100 individuals")
  )
  testthat::expect_output(print(vx$population[[1]]))
})


test_that("simulate admixture use, junctions", {
  vx <- simulate_admixture(module = ancestry_module(track_junctions = TRUE),
                           pop_size = 1000,
                           total_runtime = 100)

  num_j <- length(vx$junctions)
  testthat::expect_gt(num_j, 0)
  testthat::expect_equal(num_j, 100)
})

test_that("simulate admixture use, markers", {
  pop <- simulate_admixture(module = ancestry_module(markers =
                                                       seq(0,
                                                           1,
                                                           length.out = 1000),
                                                     morgan = 1),
                            pop_size = 1000,
                            total_runtime = 3)

  a <- subset(pop$final_frequency, pop$final_frequency$location < 0.5)
  b <-  subset(pop$final_frequency, pop$final_frequency$location > 0.5)
  testthat::expect_equal(dim(a), dim(b))

  testthat::expect_silent(
    GenomeAdmixR::plot_difference_frequencies(pop)
  )


  pop <- simulate_admixture(module = ancestry_module(markers =
                                                       seq(0, 3,
                                                           length.out = 1000),
                                                     morgan = 3),
    pop_size = 1000,
                            total_runtime = 3)

  a <- subset(pop$final_frequency, pop$final_frequency$location < 1.5)
  b <-  subset(pop$final_frequency, pop$final_frequency$location > 1.5)
  testthat::expect_equal(dim(a), dim(b))

  GenomeAdmixR::plot_difference_frequencies(pop)

  pop <- simulate_admixture(module = ancestry_module(markers =
                                                       seq(0, 5,
                                                           length.out = 1000),
                                                     morgan = 5),
              pop_size = 1000,
                            total_runtime = 3)

  a <- subset(pop$final_frequency, pop$final_frequency$location < 2.5)
  b <-  subset(pop$final_frequency, pop$final_frequency$location > 2.5)
  testthat::expect_equal(dim(a), dim(b))
  GenomeAdmixR::plot_difference_frequencies(pop)
})

test_that("simulate admixture use, pop size", {
  pop <- simulate_admixture(pop_size = 100,
                            total_runtime = 3)

  testthat::expect_equal(length(pop$population), 100)

  in_pop <- list(pop$population[[1]],
                 pop$population[[2]])

  pop2 <- simulate_admixture(module =
                               ancestry_module(input_population = in_pop),
                             pop_size = 2,
                             total_runtime = 3)

  testthat::expect_equal(length(pop2$population),
                         length(in_pop))

  pop3 <- simulate_admixture(module =
                               ancestry_module(input_population = in_pop),
                             pop_size = 100,
                             total_runtime = 3)

  testthat::expect_equal(length(pop3$population), 100)
})

context("LD stats")

test_that("calculate_average_LD", {
  pop_size <- 100
  number_of_founders <- 2
  run_time <- 1000
  morgan <- 1

  pop1 <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                        number_of_founders,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)$population

  markers <- c(0.01, 0.99)

  outcome <- calculate_ld(pop1,
                          markers = markers)

  testthat::expect_output(
    testthat::expect_message(
      calculate_ld(pop1,
                   markers = markers,
                   verbose = TRUE)
    )
  )
  testthat::expect_equal(mean(outcome$ld_matrix, na.rm = T), 0.0)


  pop_size <- 1000
  number_of_founders <- 2
  run_time <- 1
  morgan <- 1

  pop1 <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                        number_of_founders,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)$population

  markers <- c(0.5, 0.5 + 1e-4)

  outcome <- calculate_ld(pop1,
                          markers = markers)

  testthat::expect_equal(mean(outcome$ld_matrix, na.rm = T), 1.0)
})

test_that("calculate_LD_matrix", {
  pop_size <- 100
  number_of_founders <- 2
  sampled_individuals <- pop_size
  run_time <- 1
  morgan <- 1

  pop1 <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                        number_of_founders,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)

  testthat::expect_true(verify_population(pop1))

  vv <- calculate_ld(pop1, sampled_individuals,
                     markers = 10)

  vv1 <- as.vector(vv$ld_matrix[!is.na(vv$ld_matrix)])
  vv2 <- as.vector(vv$dist_matrix[!is.na(vv$dist_matrix)])

  linear_model <- lm(vv1 ~ vv2)
  # it should at least be negative
  testthat::expect_lt(linear_model$coefficients[[2]], 0.0)
})

test_that("joyplot", {
  markers <- seq(from = 0.2, to = 0.3, length.out = 100)

  selected_pop <- simulate_admixture(module = ancestry_module(number_of_founders = 3,
                                                              morgan = 1,
                                                              markers = markers
                                                            ),
                                      pop_size = 1000,
                                      total_runtime = 101)

  testthat::expect_lte(max(selected_pop$frequencies$location), max(markers))
  testthat::expect_gte(min(selected_pop$frequencies$location), min(markers))


  px <- plot_joyplot_frequencies(selected_pop$frequencies,
                                 time_points = c(0, 50, 100), picked_ancestor = "ALL")
  testthat::expect_identical(px$labels$x, "Location (Morgan)")
  testthat::expect_identical(px$labels$y, "Time")
  testthat::expect_identical(px$labels$fill, "Ancestor")

  px <- plot_joyplot_frequencies(selected_pop$frequencies,
                                 time_points = c(0, 50, 100), picked_ancestor = 1)
  testthat::expect_identical(px$labels$x, "Location (Morgan)")
  testthat::expect_identical(px$labels$y, "Time")
  testthat::expect_identical(px$labels$fill, "Ancestor")
  ax <- as.numeric(unique(px$data$ancestor)[[1]])
  testthat::expect_identical(ax, 2)
  # 2 because 1 is the second in levels [0, 1, 2]
})

#' Calculate FST
#' @description The FST value between two populations is calculated, given a
#' number of markers. Markers are superimposed upon the (known) ancestry along
#' the chromosome for all sampled individuals. Markers can be chosen to be
#' regularly spaced, or randomly distributed.
#' @param pop1 Population object
#' @param pop2 Population object
#' @param sampled_individuals Number of individuals to base the FST upon.
#' Individuals are randomly drawn from each population, a lower number speeds
#' up calculations.
#' @param number_of_markers Number of markers along the chromosome used to
#' calculate FST metrics.
#' @param random_markers  If TRUE, markers are randomly spaced along the
#' chromosome, if FALSE, markers are equidistantly spaced along the chromosome.
#' @details Uses the function \code{wc} from the package \code{hierfstat} to
#' calculate the FST. The function \code{wc} computes the Weir and Cockerham
#' F statistic.
#' @return FST value
#' @examples
#' two_populations <- simulate_admixture(
#'                         module = ancestry_module(),
#'                        migration = migration_settings(migration_rate = 0.01,
#'                                            population_size = c(100, 100)))
#'
#' FST <- calculate_fst(pop1 = two_populations$population_1,
#'                      pop2 = two_populations$population_2,
#'                      sampled_individuals = 10,
#'                      number_of_markers = 100,
#'                      random_markers = TRUE)
#' @export
calculate_fst <- function(pop1,
                          pop2,
                          sampled_individuals = 10,
                          number_of_markers = 100,
                          random_markers = FALSE) {

  pop1 <- check_input_pop(pop1)

  pop2 <- check_input_pop(pop2)

  number_of_markers <- round(number_of_markers)

  if (sampled_individuals > max(length(pop1), length(pop2))) {
    sampled_individuals <- max(length(pop1), length(pop2))
  }

  all_loci <- create_loci_matrix(
                pop1[sample(seq_along(pop1), sampled_individuals)],
                pop2[sample(seq_along(pop2), sampled_individuals)],
                number_of_markers,
                random_markers)

  hierf_wc <- hierfstat::wc(as.data.frame(all_loci))

  return(hierf_wc$FST)
}


#' @keywords internal
create_loci_matrix <- function(pop1,
                               pop2,
                               number_of_markers,
                               random_markers) {

  all_loci <- matrix(nrow = length(pop1) + length(pop2),
                     ncol = 1 + number_of_markers, 0)
  all_loci[, 1] <- c(rep(1, length(pop1)), rep(2, length(pop1)))
  colnames(all_loci) <- c("population", 1:number_of_markers)

  marker_range <- get_marker_range(pop1, pop2)


  markers <- seq(marker_range[1], marker_range[2],
                 length.out = number_of_markers)
  if (random_markers) {
    markers <- create_random_markers(number_of_markers,
                                     marker_range[1],
                                     marker_range[2])
  }

  for (x in seq_along(markers)) {
    focal_marker <- markers[x]
    for (i in seq_along(pop1)) {
      allele_1 <- 10 + findtype(pop1[[i]]$chromosome1, focal_marker)
      allele_2 <- 10 + findtype(pop1[[i]]$chromosome2, focal_marker)
      final_allele <- paste0(allele_1, allele_2)
      if (is.na(allele_1) && is.na(allele_2)) {
        final_allele <- NA
      }
      all_loci[i, x + 1] <- as.numeric(final_allele)
    }

    for (i in seq_along(pop2)) {
      allele_1 <- 10 + findtype(pop2[[i]]$chromosome1, focal_marker)
      allele_2 <- 10 + findtype(pop2[[i]]$chromosome2, focal_marker)
      final_allele <- paste0(allele_1, allele_2)
      if (is.na(allele_1) && is.na(allele_2)) {
        final_allele <- NA
      }
      all_loci[length(pop1) + i, x + 1] <- as.numeric(final_allele)
    }
  }

  return(all_loci)
}

# #' check input population
# #' @param pop population, object to be checked
# #' @return corrected input population, or error if not supplied
#' @rawNamespace useDynLib(GenomeAdmixR, .registration = TRUE)
#' @rawNamespace import(Rcpp)
#' @rawNamespace importFrom(RcppParallel, RcppParallelLibs)
#' @keywords internal
check_input_pop <- function(pop) {

  if (inherits(pop, "individual")) {
    pop <- list(pop)
    class(pop) <- "population"
  }

  if (!inherits(pop, "population")) {
    if (is.list(pop)) {
      if (inherits(pop$population, "population")) {
        pop <- pop$population
      } else{
        if (inherits(pop$population_1, "population")) {
          warning("Warning, only population 1 is processed\n
              explicitly pass a population to remedy this\n")
          pop <- pop$population_1
        } else {
          types <- c()
          for (i in seq_along(pop)) {
            types[i] <- class(pop[[i]])
          }
          if (sum(types == "individual") == length(types)) {
            class(pop) <- "population"
          } else {
            stop("Input object is not of class 'population'")
          }
        }
      }
    } else {
      pop <- c(-1e6, -1e6)
    }
  }

  return(pop)
}

#' @keywords internal
check_initial_frequencies <- function(initial_frequencies) {
  if (!is.list(initial_frequencies)) {
    if (length(initial_frequencies) %% 2 == 1) {
      stop("wrong length of initial frequencies vector\n")
    }

    if (length(initial_frequencies) == 2) {
      warning("Assuming a single population with two ancestors\n")
      if (sum(initial_frequencies) != 1) {
        initial_frequencies <- initial_frequencies / sum(initial_frequencies)
        warning("initial frequencies were normalized to 1")
      }
    }

    message("found a vector instead of a list, converting by assuming")
    message("the second half is the second population\n")
    num_founders <- length(initial_frequencies) / 2
    output_freq <- list()
    a <- initial_frequencies[1:num_founders]
    output_freq[[1]] <- a
    b <- initial_frequencies[(num_founders + 1):length(initial_frequencies)]
    output_freq[[2]] <- b

    initial_frequencies <- output_freq
  }

  if (sum(initial_frequencies[[1]]) != 1) {
    initial_frequencies[[1]] <-
      initial_frequencies[[1]] / sum(initial_frequencies[[1]])
    warning("starting frequencies were normalized to 1\n")
  }
  if (sum(initial_frequencies[[2]]) != 1) {
    initial_frequencies[[2]] <-
      initial_frequencies[[2]] / sum(initial_frequencies[[2]])
    warning("starting frequencies were normalized to 1\n")
  }
  return(initial_frequencies)
}

#' @keywords internal
check_select_matrix <- function(select_matrix,
                                markers,
                                use_data = FALSE) {
  if (is.matrix(select_matrix)) {
    message(
      "Found a selection matrix, performing simulation including selection")
    if (sum(is.na(select_matrix))) {
      stop("Can't start, there are NA values in the selection matrix!\n")
    }

    if (dim(select_matrix)[[2]] != 5) {
      stop("Incorrect dimensions of select_matrix,
           are you sure you provided all fitnesses?\n")
    }
  } else {
    if (is.na(select_matrix)) {
      select_matrix <- matrix(-1, nrow = 2, ncol = 2)
    }
  }

  if (dim(select_matrix)[2] == 5 && use_data == TRUE) {
    select_matrix[, 5] <- convert_dna_to_numeric(select_matrix[, 5])

    # this is super ugly code, but at least it works.
    other_matrix <- matrix(NA, nrow = length(select_matrix[, 1]),
                           ncol = 5)
    for (i in seq_along(select_matrix[, 1])) {
      for (j in 1:5) {
        other_matrix[i, j] <- as.numeric(select_matrix[i, j])
      }
    }
    select_matrix <- other_matrix

    sites_under_selection <- select_matrix[, 1]

    if (!(sites_under_selection %in% markers)) {
      stop("location of sites under selection have to exist in original data")
    }
  }
  return(select_matrix)
}

#' @keywords internal
generate_output_list_two_pop <- function(selected_pop,
                                         selected_popstruct_1,
                                         selected_popstruct_2,
                                         initial_freq_tibble,
                                         final_freq_tibble,
                                         track_frequency = FALSE,
                                         track_junctions = FALSE
) {
  output <- list()
  if (track_frequency == FALSE && track_junctions == FALSE) {
    output <- list("population_1" = selected_popstruct_1,
                   "population_2" = selected_popstruct_2)
  }

  if (track_frequency == FALSE && track_junctions == TRUE) {
    output <- list("population_1" = selected_popstruct_1,
                   "population_2" = selected_popstruct_2,
                   "junctions" = selected_pop$junctions)
  }

  if (track_frequency == TRUE && track_junctions == FALSE) {
    colnames(selected_pop$frequencies) <- c("time",
                                            "location",
                                            "ancestor",
                                            "frequency",
                                            "population")
    frequencies_tibble <- tibble::as_tibble(selected_pop$frequencies,
                                            .name_repair = "unique")


    output <- list("population_1" = selected_popstruct_1,
                   "population_2" = selected_popstruct_2,
                   "frequencies" = frequencies_tibble,
                   "initial_frequency" = initial_freq_tibble,
                   "final_frequency" = final_freq_tibble)
  }

  if (track_frequency == TRUE && track_junctions == TRUE) {

    colnames(selected_pop$frequencies) <- c("time",
                                            "location",
                                            "ancestor",
                                            "frequency",
                                            "population")
    frequencies_tibble <- tibble::as_tibble(selected_pop$frequencies)

    output <- list("population_1" = selected_popstruct_1,
                   "population_2" = selected_popstruct_2,
                   "frequencies" = frequencies_tibble,
                   "initial_frequency" = initial_freq_tibble,
                   "final_frequency" = final_freq_tibble,
                   "junctions" = selected_pop$junctions)
  }

  return(output)
}


#' @keywords internal
generate_output_list_one_pop <- function(selected_popstruct,
                                         selected_pop,
                                         initial_freq_tibble,
                                         final_freq_tibble,
                                         track_frequency = FALSE,
                                         track_junctions = FALSE) {
  output <- list()
  if (track_frequency == FALSE && track_junctions == FALSE) {
    output <- list("population" = selected_popstruct)
  }

  if (track_frequency == FALSE && track_junctions == TRUE) {
    output <- list("population" = selected_popstruct,
                   "junctions" = selected_pop$junctions)
  }

  if (track_frequency == TRUE && track_junctions == FALSE) {
    colnames(selected_pop$frequencies) <- c("time", "location",
                                            "ancestor", "frequency")
    frequencies_tibble <- tibble::as_tibble(selected_pop$frequencies)


    output <- list("population" = selected_popstruct,
                   "frequencies" = frequencies_tibble,
                   "initial_frequency" = initial_freq_tibble,
                   "final_frequency" = final_freq_tibble)
  }

  if (track_frequency == TRUE && track_junctions == TRUE) {
    colnames(selected_pop$frequencies) <- c("time", "location",
                                            "ancestor", "frequency")

    frequencies_tibble <- tibble::as_tibble(selected_pop$frequencies)

    output <- list("population" = selected_popstruct,
                   "frequencies" = frequencies_tibble,
                   "initial_frequency" = initial_freq_tibble,
                   "final_frequency" = final_freq_tibble,
                   "junctions" = selected_pop$junctions)
  }
  return(output)
}



#' @keywords internal
population_to_vector <- function(source_pop) {

  get_chroms <- function(indiv) {
    a <- as.vector(t(indiv$chromosome1))
    b <- as.vector(t(indiv$chromosome2))
    return(c(a, b))
  }

  if (is.vector(source_pop)) return(source_pop)
  chroms <- lapply(source_pop, get_chroms)
  pop_for_cpp <- unlist(chroms)

  return(pop_for_cpp)
}

#' @keywords internal
increase_ancestor <- function(population, increment = 20) {
  increase_indiv <- function(indiv) {

    # -1 indicates the end, no increment there!
    positive <- which(indiv$chromosome1[, 2] > -1)
    indiv$chromosome1[positive, 2] <- indiv$chromosome1[positive, 2] + increment

    # -1 indicates the end, no increment there!
    positive <- which(indiv$chromosome2[, 2] > -1)
    indiv$chromosome2[positive, 2] <- indiv$chromosome2[positive, 2] + increment
    return(indiv)
  }

  if (!inherits(population, "population")) {
    if (inherits(population$population, "population")) {
      population <- population$population
    }
  }

  pop_2 <- lapply(population, increase_indiv)
  class(pop_2) <- "population"
  return(pop_2)
}

#' @keywords internal
create_random_markers <- function(number_of_markers,
                                  min_pos = 0,
                                  max_pos = 1) {
  markers <- c()
  while (length(markers) < number_of_markers) {
    temp_markers <- stats::runif(number_of_markers - length(markers),
                                 min_pos, max_pos)
    which_dupl <- which(duplicated(temp_markers))
    if (length(which_dupl)) {
      temp_markers <- temp_markers[-which_dupl]
    }
    markers <- c(markers, temp_markers)
  }
  markers <- sort(markers)
  return(markers)
}



#' @keywords internal
calc_allele_frequencies <- function(indiv, alleles) {

  for (i in seq_along(indiv$chromosome1[, 1])) {
    left <- indiv$chromosome1[i, 1]
    right <- 1
    if (i + 1 <= length(indiv$chromosome1[, 1])) {
      right <- indiv$chromosome1[i + 1, 1]
    }

    allele <- 1 + indiv$chromosome1[i, 2]
    alleles[allele] <- alleles[allele] + (right - left)
  }

  for (i in seq_along(indiv$chromosome2[, 1])) {
    left <- indiv$chromosome2[i, 1]
    right <- 1
    if (i + 1 <= length(indiv$chromosome2[, 1])) {
      right <- indiv$chromosome2[i + 1, 1]
    }

    allele <- 1 + indiv$chromosome2[i, 2]
    alleles[allele] <- alleles[allele] + (right - left)
  }

  alleles <- alleles / sum(alleles)
  return(alleles)
}

#' print an individual to the console
#' @description prints an object of class individual to the console
#' @param x individual
#' @param ... other arguments
#' @return No return value
#' @export
print.individual <- function(x, ...) {
  print("Individual with two Chromosomes")
  v1 <- paste("Chromosome 1:",
              length(x$chromosome1[, 1]) - 2,
              "junctions")
  v2 <- paste("Chromosome 2:",
              length(x$chromosome2[, 1]) - 2,
              "junctions")
  print(v1)
  print(v2)
}

#' print a population object
#' @description prints the contents of a population nicely
#' @param x input population
#' @param ... other arguments
#' @return No return value
#' @export
print.population <- function(x, ...) {
  v1 <- paste("Population with",
              length(x),
              "individuals")
  print(v1)
}

#' plot the genome of an individual
#' @description visualise ancestry blocks on both chromosomes
#' @param x object of type individual
#' @param cols colors for the different ancestors
#' @param ... other arguments
#' @return No return value
#' @export
plot.individual <- function(x, cols = NA,  ...) {
  alleles_chrom1 <- unique(x$chromosome1[, 2])
  alleles_chrom2 <- unique(x$chromosome2[, 2])
  num_colors <- 1 + max(alleles_chrom1, alleles_chrom2)
  if (num_colors > 20) num_colors <- 20
  color_palette <- grDevices::rainbow(num_colors, alpha = 1)

  if (!is.na(cols[[1]])) {
    color_palette <- cols
  }


  old_par <- graphics::par("mar", "mfrow", no.readonly = TRUE)
  on.exit(graphics::par(old_par))

  graphics::par(mfrow = c(2, 1))
  graphics::par(mar = c(2, 2, 2, 2))
  graphics::plot(NA,
                 xlim = c(0, 1),
                 ylim = c(0, 1),
                 xlab = "",
                 ylab = "",
                 xaxt = "n",
                 yaxt = "n",
                 bty  = "n")

  for (i in seq_along(x$chromosome1[, 1])) {
    xleft <- x$chromosome1[i, 1]
    xrght <- 1
    if (i < length(x$chromosome1[, 1])) {
      xrght <- x$chromosome1[i + 1, 1]
    }
    colour_index <- 1 + x$chromosome1[i, 2]
    colour_to_plot <- color_palette[colour_index]

    graphics::rect(xleft = xleft,
                   xright = xrght,
                   ybottom = 0,
                   ytop = 1,
                   col = colour_to_plot,
                   border = NA)
  }

  graphics::plot(NA,
                 xlim = c(0, 1),
                 ylim = c(0, 1),
                 xlab = "",
                 ylab = "",
                 xaxt = "n",
                 yaxt = "n",
                 bty  = "n")

  for (i in seq_along(x$chromosome2[, 1])) {
    xleft <- x$chromosome2[i, 1]
    xrght <- 1
    if (i < length(x$chromosome2[, 1])) {
      xrght <- x$chromosome2[i + 1, 1]
    }
    colour_index <- 1 + x$chromosome2[i, 2]
    colour_to_plot <- color_palette[colour_index]

    graphics::rect(xleft = xleft,
                   xright = xrght,
                   ybottom = 0,
                   ytop = 1,
                   col = colour_to_plot,
                   border = NA)
  }
}

#' @keywords internal
create_pop_class <- function(pop) {
  set_indiv_class <- function(indiv) {
    class(indiv) <- "individual"
    indiv
  }
  pop <- lapply(pop, set_indiv_class)
  class(pop) <- "population"
  return(pop)
}


# #' verify that an individual is correct
# #' @description function to verify correctness of an object of class
# #' 'individual'
# #' @param indiv object of class 'individual'
#' @keywords internal
verify_individual <- function(indiv) {

  if (!inherits(indiv, "individual")) return(FALSE)

  if (indiv$chromosome1[1, 1] != 0) {
    warning("Chromosome doesn't start at 0\n")
    return(FALSE)
  }

  if (max(abs(indiv$chromosome1[, 2])) > 10000) {
    warning("Memory error recorded in chromosome\n")
    return(FALSE)
  }

  if (indiv$chromosome2[1, 1] != 0) {
    warning("Chromosome doesn't start at 0\n")
    return(FALSE)
  }

  if (max(abs(indiv$chromosome2[, 2])) > 10000) {
    warning("Memory error recorded in chromosome\n")
    return(FALSE)
  }

  return(TRUE)
}


# #' verify that a population is correct
# #' @description function to verify correctness of an object of class
# #' 'population'
# #' @param pop object of class 'population'
#' @keywords internal
verify_population <- function(pop) {

  if (!inherits(pop, "population"))  {
    if (!inherits(pop$population, "population")) {
      warning("pop is nog of class population")
      return(FALSE)
    } else {
      return(verify_population(pop$population))
    }
  }
  v <- unlist(lapply(pop, verify_individual))
  if (sum(v) != length(v)) {
    warning("not all individuals passed test")
    return(FALSE)
  }

  return(TRUE)
}

# find local ancestry
# #' @description returns local ancestry at a site
# #' @param chrom chromosome
# #' @param pos position where to check for local ancestry
# #' @return local ancestry
#' @keywords internal
findtype <- function(chrom, pos) {

  if (pos < chrom[1, 1]) {
    return(NA)
  }

  if (pos == chrom[1, 1]) {
    return(chrom[1, 2])
  }

  if (pos >= utils::tail(chrom[, 1], 1)) {
    return(utils::tail(chrom[, 2], 1)) # return the last ancestry
  }

  b <- which(chrom[, 1] > pos)
  chromtype <- chrom[b[1] - 1, 2]

  return(chromtype[[1]])
}

#' print an individual to the console
#' @description prints an object of class genomeadmixr_data to the console
#' @param x individual
#' @param ... other arguments
#' @return No return value
#' @export
print.genomeadmixr_data <- function(x, ...) {
  print("Data to use as input for GenomeAdmixR")
  v1 <- paste("Number of individuals:",
              length(x$genomes[, 1]) / 2)
  v2 <- paste("Number of markers:",
              length(x$genomes[1, ]))
  print(v1)
  print(v2)
}


#' @keywords internal
print_substitution_matrix <- function(substitution_matrix) {
  for (i in 1:4) {
    print_str <- ""
    for (j in 1:4) {
      print_str <- paste(print_str, substitution_matrix[i, j])
    }
    message(print_str)
  }
}

#' @keywords internal
verify_substitution_matrix <- function(substitution_matrix) {
  if (length(substitution_matrix == 1)) {
    if (is.na(substitution_matrix[[1]])) {
      stop("No substitution matrix provided")
    }
  }

  if (dim(substitution_matrix)[[1]] != 4 ||
      dim(substitution_matrix)[[2]] != 4) {
    stop("\nCan not include mutations without proper substitution matrix\n",
         "substitution matrix should be a 4x4 matrix")
  }

  if (sum(substitution_matrix == 1) > 0) {
    warning("found rate matrix, rescaled all entries")
    # we have to rewrite as relative matrix
    for (i in 1:4) {
      row_entry <- substitution_matrix[i, ]
      row_entry <- row_entry * 0.25
      row_entry[i] <- 0
      row_entry[i] <- 1 - sum(row_entry)
      substitution_matrix[i, ] <- row_entry
    }
  }

  rs <- rowSums(substitution_matrix)
  if (sum(rs != 1) > 0) {
    warning("normalized rows to ensure they sum to 1")
    substitution_matrix[1:4, ] <- substitution_matrix[1:4, ] / rs[1:4]
  }

#  message("using mutation with the following substitution matrix: ")
#  print_substitution_matrix(substitution_matrix)

  return(substitution_matrix)
}

#' @keywords internal
check_markers <- function(markers, data_markers) {
  markers_in_data <- markers %in% data_markers
  which_markers_not_in_data <- which(markers_in_data == FALSE)

  if (length(which_markers_not_in_data) > 0) {
    warning(paste0("removing: ",
                   length(which_markers_not_in_data),
                   " markers as these do not exist in the original dataset"))
    markers <- markers[-which_markers_not_in_data]
  }

  if (length(markers) == 0) {
    stop("you have to provide markers that exist in the original dataset")
  }
  return(sort(markers))
}

#' @keywords internal
get_marker_range <- function(pop1, pop2) {
  get_min_pos <- function(indiv) {
    return(min(indiv$chromosome1[, 1],
               indiv$chromosome2[, 1]))
  }
  get_max_pos <- function(indiv) {
    return(max(indiv$chromosome1[, 1],
               indiv$chromosome2[, 1]))
  }

  min_positions1 <- unlist(lapply(pop1, get_min_pos))
  min_positions2 <- unlist(lapply(pop2, get_min_pos))

  max_positions1 <- unlist(lapply(pop1, get_max_pos))
  max_positions2 <- unlist(lapply(pop2, get_max_pos))

  min_pos <- min(min_positions1, min_positions2)
  max_pos <- max(max_positions1, max_positions2)
  return(c(min_pos, max_pos))
}

#' @keywords internal
check_for_bases <- function(pop) {

  unique_bases <- c()
  for (i in 1:10) {
    if (i < length(pop)) {
      b_c1 <- unique(pop[[i]]$chromosome1[, 2])
      b_c2 <- unique(pop[[i]]$chromosome2[, 2])
      unique_bases <- c(unique_bases, b_c1, b_c2)
    }
  }
  unique_bases <- sort(unique(unique_bases))
  using_sequencing_data <- FALSE
  # sum has to be at least 4, 0 represents missing data, but
  # might not be in the data

  if (sum(unique_bases %in% c(0, 1, 2, 3, 4)) >= 4 &&
      sum(unique_bases %in% c(0, 1, 2, 3, 4)) <= 5) {
    using_sequencing_data <- TRUE
  }
  if (sum(!unique_bases %in% c(0, 1, 2, 3, 4)) > 0) {
    using_sequencing_data <- FALSE
  }

  return(using_sequencing_data)
}

#' @keywords internal
create_recombination_map <- function(markers,
                                     recombination_rate) {
  distances <- diff(markers)
  recomb_map <- c(0, distances)

  # recombination rate is in cM per Mbp
  rate_in_morgan <- recombination_rate / 100
  rate_per_bp <- rate_in_morgan / 1000000

  recomb_map <- recomb_map * rate_per_bp

  return(recomb_map)
}

#' @keywords internal
verify_genomeadmixr_data <- function(input_data, markers = NA,
                                     verbose = FALSE) {
  if (!inherits(input_data, "genomeadmixr_data")) {
    if (inherits(input_data, "genomadmixr_simulation") ||
        inherits(input_data, "individual")) {
      if (verbose) {
        message("found simulation output, converting to genomeadmixr_data")
        message("this may take a while")
      }
      input_data <-
        simulation_data_to_genomeadmixr_data(simulation_data =
                                               input_data,
                                             markers = markers)
      if (verbose) message("done converting, continuing as normal")
      return(input_data)
    } else {
      if (is.list(input_data)) {
        for (i in seq_along(input_data)) {
          input_data[[i]] <- verify_genomeadmixr_data(input_data[[i]], markers)
        }

        if (length(input_data) == 1) {
          input_data <- input_data[[1]]
        }

        return(input_data)
      }
    }
  }

  if (!inherits(input_data, "genomeadmixr_data")) {
    input_data2 <- check_input_pop(input_data)
    if (inherits(input_data2, "population")) {
      if (verbose) {
        message("found simulation output, converting to genomeadmixr_data")
        message("this may take a while")
      }
      input_data <-
        simulation_data_to_genomeadmixr_data(simulation_data = input_data,
                                             markers = markers)
      if (verbose) message("done converting, continuing as normal")
    } else {
      stop("input_data should be of class genomeadmixr_data
              you can create such data with the functions
              create_input_data or vcfR_to_genomeadmixr_data")
    }
  }
  return(input_data)
}

#' @keywords internal
convert_to_numeric_matrix <- function(genome_data) {

  genome_numeric <- genome_data
  genome_numeric[genome_numeric == "0/0"] <- 1
  genome_numeric[genome_numeric == "0/1"] <- 2
  genome_numeric[genome_numeric == "1/0"] <- 2
  genome_numeric[genome_numeric == "1/1"] <- 3
  genome_numeric[is.na(genome_numeric)]   <- 0

  genome_numeric <- t(genome_numeric)
  genome_numeric <- apply(genome_numeric, 2, as.numeric)

  return(genome_numeric)
}

#' @keywords internal
set_num_threads <- function(num_threads) {
  if (num_threads == 1) {
    RcppParallel::setThreadOptions( 1L )
  } else {
    RcppParallel::setThreadOptions(num_threads)
  }
}

#' @keywords internal
reset_num_threads <- function() {
  RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )
}

context("allele_frequencies")


test_that("calculate_allele_frequencies", {
  pop_size <- 100
  number_of_founders <- 2
  run_time <- 5

  morgan <- 1

  found <- c()
  for (r in 1:30) {
    vx <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                        number_of_founders,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)

    testthat::expect_true(verify_population(vx))

    for (i in 1:pop_size) {
      found <- rbind(found,
                     calc_allele_frequencies(vx$population[[i]],
                                             alleles =
                                               rep(0, number_of_founders * 2)
                                             )
      )
    }
  }

  v <- colMeans(found)
  testthat::expect_equal(v[[1]], 0.5, tolerance = 0.2)

  testthat::expect_equal(v[[2]], 0.5, tolerance = 0.2)

  found <- c()
  for (r in 1:30) {
    vx <- simulate_admixture(module = ancestry_module(number_of_founders = 4,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)

    testthat::expect_true(verify_population(vx))

    for (i in 1:pop_size) {
      found <- rbind(found,
                     calc_allele_frequencies(vx$population[[i]],
                                             alleles =
                                               rep(0, number_of_founders * 2)
                                             )
      )
    }
  }

  v <- mean(colMeans(found))
  testthat::expect_equal(v, 0.25, tolerance = 0.05)


  number_of_founders <- 20
  sourcepop <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                             number_of_founders),
                                  pop_size = 1000,
                                  total_runtime = 100)

  testthat::expect_true(verify_population(sourcepop))

  freq_output <- calculate_allele_frequencies(sourcepop$population)

  b <- subset(freq_output, freq_output$location == 0)

  testthat::expect_equal(mean(b$frequency),
                         1 / number_of_founders,
                         tolerance = 1 / number_of_founders)
  testthat::expect_equal(sum(b$frequency), 1,
                         tolerance = 1 / number_of_founders)

  number_of_founders <- 5
  sourcepop <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                             number_of_founders),
                                  pop_size = 1000,
                                  total_runtime = 100)

  testthat::expect_true(verify_population(sourcepop))

  freq_output <- calculate_allele_frequencies(sourcepop$population)
  plot_frequencies(sourcepop)

  testthat::expect_equal(length(unique(freq_output$ancestor)),
                         number_of_founders)

  b <- subset(freq_output, freq_output$location == 0)

  testthat::expect_equal(sum(b$frequency), 1, tolerance = 0.1)
  testthat::expect_equal(mean(b$frequency),
                         1 / number_of_founders,
                         tolerance = 0.1)

  number_founders <- 20
  sourcepop <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                             number_of_founders),
                                  pop_size = 1000,
                                  total_runtime = 1)

  testthat::expect_true(verify_population(sourcepop))
})

context("isoFemale creation")

test_that("create_isofemale", {
  pop_size <- 100
  number_of_founders <- 2
  run_time <- 100
  morgan <- 1

  pop <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                     number_of_founders,
                                                     morgan = morgan),
                            pop_size = pop_size,
                            total_runtime = run_time)


  testthat::expect_true(verify_population(pop))

 testthat::expect_silent(
   females <- create_iso_female(module = ancestry_module(input_population = pop),
                                n = 1,
                                run_time = 1000)
 )

  testthat::expect_equal(length(females), 1)
})

test_that("create_population_from_isofemales", {
  pop_size <- 100
  number_of_founders <- 10
  run_time <- 100
  morgan <- 1

  pop1 <- simulate_admixture( module = ancestry_module(number_of_founders =
                                                         number_of_founders,
                                                       morgan = morgan),
                              pop_size = pop_size,
                             total_runtime = run_time)

  pop2 <- simulate_admixture( module = ancestry_module(number_of_founders =
                                                         number_of_founders,
                                                       morgan = morgan),
                              pop_size = pop_size,
                              total_runtime = run_time)

  pop2 <- increase_ancestor(pop2, number_of_founders)

  testthat::expect_true(verify_population(pop1))
  testthat::expect_true(verify_population(pop2))


  testthat::expect_silent(
      female_1 <- create_iso_female(module = ancestry_module(input_population = pop1),
                                    n = 1,
                                    run_time = 20000)
  )
  testthat::expect_silent(
   female_2 <- create_iso_female(module = ancestry_module(input_population = pop2),
                                 n = 1,
                                 run_time = 20000)
)
  testthat::expect_true(verify_individual(female_1[[1]]))
  testthat::expect_true(verify_individual(female_2[[1]]))

testthat::expect_silent(
  females <- create_iso_female(module = ancestry_module(input_population = pop1),
                               n = 2,
                               run_time = 2000)
)
  pop_size <- 100
  vy <- simulate_admixture(module = ancestry_module(input_population = females,
                                                    morgan = morgan),
                           pop_size = pop_size,
                           total_runtime = 200)

  testthat::expect_equal(length(vy$population), pop_size)
  testthat::expect_true(verify_population(vy))

  vy <- simulate_admixture(module = ancestry_module(input_population =
                                                      list(female_1[[1]],
                                                           female_2[[1]]),
                                                    morgan = morgan),
                           pop_size = pop_size,
                           total_runtime = 2000)

  testthat::expect_equal(length(vy$population), pop_size)
  testthat::expect_true(verify_population(vy))

  testthat::expect_silent(
    plot_chromosome(female_1[[1]]$chromosome1, 0, 1)
  )
})

test_that("cpp classes", {
  a <- matrix(c(0.1, 1, 2, 2), nrow = 2)
  b <- matrix(c(0, 1, 1, -1), nrow = 2)
  indiv <- list(chromosome1 = a, chromosome2 = a)
  class(indiv) <- "individual"

  # chromosome 1
  testthat::expect_warning(v <- verify_individual(indiv),
                           "Chromosome doesn't start at 0")

  indiv <- list(chromosome1 = b, chromosome2 = a)
  class(indiv) <- "individual"

  # chromosome 2
  testthat::expect_warning(v <- verify_individual(indiv),
                          "Chromosome doesn't start at 0")

  a <- matrix(c(0.0, 1, 2, 2), nrow = 2)
  b <- matrix(c(0, 1, 1, -1), nrow = 2)
  indiv <- list(chromosome1 = b, chromosome2 = a)
  class(indiv) <- "individual"
  indiv$chromosome2 <-  indiv$chromosome1
  indiv$chromosome1 <- a


  a <- matrix(c(0.0, 1, 0.5, 29192875037,  1, -1), ncol = 2)

  indiv$chromosome1 <- a
  testthat::expect_warning(v <- verify_individual(indiv),
                          "Memory error recorded in chromosome")

  a <- matrix(c(0.0, 1, 0.5, -92875037,  1, -1), ncol = 2)
  indiv$chromosome2 <- a
  indiv$chromosome1 <- b
  testthat::expect_warning(v <- verify_individual(indiv),
                          "Memory error recorded in chromosome")
})

test_that("create_isofemale_data", {
 data("dgrp2.3R.5k.data")

 females <- create_iso_female(module = sequence_module(
                                           molecular_data = dgrp2.3R.5k.data,
                                           morgan = 1),
                              n = 2,
                              inbreeding_pop_size = 100,
                              run_time = 100)

 testthat::expect_equal(length(females), 2)
})

context("utilities")

test_that("utilities", {
  vx <- simulate_admixture(module = ancestry_module(number_of_founders = 50),
                           pop_size = 100,
                           total_runtime = 5)
  testthat::expect_silent(
    plot_chromosome(vx$population[[1]]$chromosome1)
  )

  vx <- simulate_admixture(module = ancestry_module(markers = seq(0, 1, by = 0.01)),
                           total_runtime = 100,
                           pop_size = 100)
  testthat::expect_silent(
    plot_over_time(vx$frequencies, focal_location = 0.5)
  )

  vy <- simulate_admixture(migration =
                             migration_settings(migration_rate = 0.01))

  testthat::expect_error(plot_over_time(vy$frequencies, focal_location = 0.5))

  vy <- simulate_admixture(
    module = ancestry_module(markers = 0.5),
      migration = migration_settings(migration_rate = 0.01))

  testthat::expect_silent(
    plot_over_time(vy$frequencies, focal_location = 0.5)
  )
})

test_that("initial_frequencies", {
  testthat::expect_error(
    simulate_admixture(
          migration = migration_settings(migration_rate = 0.1,
                                         initial_frequencies = c(0.5, 0.5),
                                         population_size = c(1000, 1000)),
          total_runtime = 100
      )
  )
  testthat::expect_message(
  testthat::expect_warning(
      vx <- simulate_admixture(total_runtime = 5,
                migration = migration_settings(
                  migration_rate = 0.0,
                  initial_frequencies = c(1, 1, 0, 0, 0, 0, 1, 1)))
  ))

  # warning that frequencies don't add up to 1.
testthat::expect_warning(
  vx <- simulate_admixture(total_runtime = 5,
                             migration = migration_settings(
                               migration_rate = 0.0,
                               initial_frequencies = list(c(1, 1, 0, 0),
                                                          c(0, 0, 1, 1)))
                           )
  )

  testthat::expect_error(
    simulate_admixture(total_runtime = 5,
                         migration = migration_settings(
                           migration_rate = 0.0,
                           initial_frequencies = c(1, 1, 0, 0,
                                                   0, 0, 1, 1, 1))
                       )
  )
})

testthat::test_that("random markers", {
  set.seed(42)
  vx <- create_random_markers(1e3)
  vy <- create_random_markers(1e6)
  testthat::expect_true(length(vy) > length(vx))
})

testthat::test_that("verify datatypes", {
  vx <- simulate_admixture(total_runtime = 2,
                           pop_size = 100,
                           migration = migration_settings(migration_rate = 0.001))

testthat::expect_warning(
  check_input_pop(vx)
)
})

context("fst")

testthat::test_that("fst", {

  pop_size <- 100
  number_of_founders <- 20
  run_time <- 1
  morgan <- 1

  pop1 <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                        number_of_founders,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)

  pop2 <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                        number_of_founders,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)

  pop2 <- increase_ancestor(pop2, number_of_founders)

  testthat::expect_true(verify_population(pop1))
  testthat::expect_true(verify_population(pop2))

  number_of_markers <- 10
  sampled_individuals <- 10
  v1 <- calculate_fst(pop1, pop2,
                      sampled_individuals,
                      number_of_markers, random_markers = TRUE)

  v2 <- calculate_fst(pop1, pop2,
                      sampled_individuals,
                      number_of_markers, random_markers = FALSE)

  testthat::expect_equal(v1, v2, tolerance = 0.2)

  pop_size <- 100
  number_of_founders <- 10
  run_time <- 1000
  morgan <- 1

  pop1 <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                        number_of_founders,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)

  pop2 <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                        number_of_founders,
                                                      morgan = morgan),
                             pop_size = pop_size,
                             total_runtime = run_time)
  pop2 <- increase_ancestor(pop2, number_of_founders)

  testthat::expect_true(verify_population(pop1))
  testthat::expect_true(verify_population(pop2))

  number_of_markers <- 10
  v1 <- calculate_fst(pop1, pop2,
                      number_of_markers, random_markers = TRUE)

  testthat::expect_equal(1.0, v1, tolerance = 0.2)
})

#' Individual based simulation of the breakdown of contiguous ancestry blocks.
#' @description Individual based simulation of the breakdown of contiguous
#' ancestry blocks, with or without selection. Simulations can be started from
#' scratch, or from a predefined input population.
#' @param input_data Genomic data used as input, should be of type
#' genomeadmixr_data. Either a single dataset is provided, or a list of
#' multiple genomeadmixr_data objects.
#' @inheritParams default_params_doc
#' @param pop_size Vector containing the number of individuals in both
#' populations.
#' @param initial_frequencies A vector describing the initial contribution of
#' each provided input data set to the starting hybrid swarm. By default, equal
#' frequencies are assumed. If a vector not summing to 1 is provided, the vector
#' is normalized.
#' @param recombination_rate rate in cM / Mbp, used to map recombination to the
#' markers. If the recombination_rate is not set, the value for Morgan is used,
#' assuming that the markers included span an entire chromosome.
#' @param mutation_rate the per base probability of mutation. Default is 0.
#' @param substitution_matrix a 4x4 matrix representing the probability of
#' mutating to another base (where [1/2/3/4] = [a/c/t/g]), conditional on the
#' event of a mutation happening. Default is the JC69 matrix, with equal
#' probabilities for all transitions / transversions.
#' @return A list with: \code{population} a population object, and three tibbles
#' with allele frequencies (only contain values of a vector was provided to the
#' argument \code{markers}: \code{frequencies} , \code{initial_frequencies} and
#' \code{final_frequencies}. Each tibble contains four columns, \code{time},
#' \code{location}, \code{ancestor} and \code{frequency}, which indicates the
#' number of generations, the location along the chromosome of the marker, the
#' ancestral allele at that location in that generation, and finally, the
#' frequency of that allele.
simulate_sequence <- function(input_data = NA,
                              pop_size = NA,
                              initial_frequencies = NA,
                              total_runtime = 100,
                              morgan = 1,
                              recombination_rate = NA,
                              num_threads = 1,
                              select_matrix = NA,
                              markers = NA,
                              verbose = FALSE,
                              multiplicative_selection = TRUE,
                              mutation_rate = 0,
                              substitution_matrix = matrix(1 / 4, 4, 4)) {

  input_data <- verify_genomeadmixr_data(input_data,
                                         markers, verbose = verbose)

  if (!inherits(input_data, "genomeadmixr_data")) {
    if (is.list(input_data)) {
      if (length(input_data) > 1) {
        if (verbose) message("found multiple input populations")
        input_data <- combine_input_data(input_data,
                                         frequencies = initial_frequencies,
                                         pop_size = pop_size)
      }
    }
  }

  if (is.na(pop_size)) {
    if (length(input_data) == 2) {
      pop_size <- length(input_data$genomes[, 1]) / 2
    } else {
      stop("pop_size is undefined, need an input population")
    }
  }
  set_num_threads(num_threads)
  select_matrix <- check_select_matrix(select_matrix,
                                       markers,
                                       use_data = TRUE)

  if (length(markers) == 1) {
    if (is.na(markers))  {
      markers <- c(-1, -1)
      track_frequency <- FALSE
    } else {
      track_frequency <- TRUE
    }
  } else {
    track_frequency <- TRUE
  }

  if (track_frequency) {
    markers <- check_markers(markers, input_data$markers)
  }

  if (mutation_rate > 0) {
    substitution_matrix <- verify_substitution_matrix(substitution_matrix)
  }

  recombination_map <- c(-1, 0)
  if (!is.na(recombination_rate)) {
    recombination_map <- create_recombination_map(input_data$markers,
                                                  recombination_rate)
  }

  selected_pop <- simulate_emp_cpp(input_data$genomes,
                                   input_data$markers,
                                   select_matrix,
                                   pop_size,
                                   total_runtime,
                                   morgan,
                                   verbose,
                                   track_frequency,
                                   markers,
                                   multiplicative_selection,
                                   mutation_rate,
                                   substitution_matrix,
                                   num_threads,
                                   recombination_map)

  selected_popstruct <- create_pop_class(selected_pop$population)

  colnames(selected_pop$initial_frequencies) <- c("time",
                                                  "location",
                                                  "ancestor",
                                                  "frequency")
  colnames(selected_pop$final_frequencies) <-
    colnames(selected_pop$initial_frequencies)


  initial_freq_tibble <- tibble::as_tibble(selected_pop$initial_frequencies)
  final_freq_tibble   <- tibble::as_tibble(selected_pop$final_frequencies)


  output <- generate_output_list_one_pop(selected_popstruct,
                                         selected_pop,
                                         initial_freq_tibble,
                                         final_freq_tibble,
                                         track_frequency)

  output$frequencies <- convert_to_dna(output$frequencies)
  output$initial_frequency <- convert_to_dna(output$initial_frequency)
  output$final_frequency <- convert_to_dna(output$final_frequency)

  class(output) <- "genomadmixr_simulation"
  reset_num_threads()
  return(output)
}

#' @keywords internal
convert_to_dna <- function(frequency_tibble) {
  alleles <- frequency_tibble$ancestor
  alleles[alleles == 1] <- "a"
  alleles[alleles == 2] <- "c"
  alleles[alleles == 3] <- "t"
  alleles[alleles == 4] <- "g"
  alleles[alleles == 0] <- "-"

  frequency_tibble$ancestor <- alleles
  return(frequency_tibble)
}

#' make a joy plot of the distribution of allele frequencies within a region
#' @description This function plots the distribution of allele frequencies
#' within a region over time, making use of a 'joyplot'
#' @param frequencies  A tibble containing four columns: \code{time},
#' \code{location}, \code{ancestor}, \code{frequency}. Typically one of the
#' items returned by \code{create_population_selection} or
#' \code{select_population} when the user specifies \code{track_frequency}.
#' @param time_points  A sequence of time points for which the user wants to
#' create the joyplot
#' @param picked_ancestor Default is "ALL", where different colors indicate
#' different ancestors. Alternatively, for clarity, the user can specify a
#' specific ancestral allele, and only that allele is plotted
#' @param picked_population If multiple populations were simulated (in the case
#' of \code{simulate_admixture_migration}), which population should be plotted?
#' Default is population_1.
#' @return a ggplot object
#' @export
#' @examples
#' \donttest{
#' s <- 0.01
#' select_matrix <- matrix(nrow = 1, ncol = 5)
#' select_matrix[1, ] <- c(0.25, 1.0, 1 + 0.5 * s, 1 + s, 0)
#'
#' markers <- seq(from = 0.2, to = 0.3, length.out = 100)
#'
#' selected_pop <- simulate_admixture(
#'                     module = ancestry_module(number_of_founders = 10,
#'                                              morgan = 1,
#'                                              markers = markers),
#'                     pop_size = 1000,
#'                     total_runtime = 11,
#'                     select_matrix = select_matrix)
#' require(ggplot2)
#' plot_joyplot_frequencies(frequencies = selected_pop$frequencies,
#'                          time_points = 0:11,
#'                          picked_ancestor = "ALL")
#'
#' # joyplot frequencies returns a ggplot object, so we can
#' # add extra elements:
#' plot_joyplot_frequencies(frequencies = selected_pop$frequencies,
#'                          time_points = 0:11,
#'                          picked_ancestor = "ALL") +
#'   ggplot2::xlab("Location") +
#'   ggplot2::ylab("Generations")
#' }
#' @importFrom rlang .data
plot_joyplot_frequencies <- function(frequencies,
                                     time_points,
                                     picked_ancestor = "ALL",
                                     picked_population = 1
) {
  if ("population" %in% colnames(frequencies)) {
    frequencies <- subset(frequencies,
                          frequencies$population == picked_population)
  }

  time_points <- floor(time_points)
  vz <- subset(frequencies,
               frequencies$time %in% time_points)
  vz$ancestor <- as.factor(vz$ancestor)

  if (picked_ancestor == "ALL") {
    p1 <- ggplot2::ggplot(vz, ggplot2::aes(x = .data[["location"]],
                                           y = as.factor(.data[["time"]]),
                                           height = .data[["frequency"]],
                                           fill = .data[["ancestor"]])
                          ) +
      ggridges::geom_ridgeline(scale = 1.3)
  } else {
    vy <- subset(vz, vz$ancestor == picked_ancestor)
    p1 <- ggplot2::ggplot(vy, ggplot2::aes(x = .data[["location"]],
                                           y = as.factor(.data[["time"]]),
                                           height = .data[["frequency"]])) +
      ggridges::geom_ridgeline(scale = 1.3)
  }

  if (max(vz$location) < 10) {
    p1 <- p1 +
      ggplot2::xlab("Location (Morgan)")
  } else {
    p1 <- p1 +
      ggplot2::xlab("Location (bp)")
  }

  p1 <- p1 +
    ggplot2::labs(fill = "Ancestor")  +
    ggplot2::ylab("Time")
  return(p1)
}

#' Individual based simulation of the breakdown of contiguous ancestry blocks.
#' @description Individual based simulation of the breakdown of contiguous
#' ancestry blocks, with or without selection. Simulations can be started from
#' scratch, or from a predefined input population.
#' @inheritParams default_params_doc
#' @return A list with: \code{population} a population object, and three tibbles
#' with allele frequencies (only contain values of a vector was provided to the
#' argument \code{markers}: \code{frequencies} , \code{initial_frequencies} and
#' \code{final_frequencies}. Each tibble contains four columns, \code{time},
#' \code{location}, \code{ancestor} and \code{frequency}, which indicates the
#' number of generations, the location along the chromosome of the marker, the
#' ancestral allele at that location in that generation, and finally, the
#' frequency of that allele.
#' @examples
#' # local ancestry simulation
#' two_populations <- simulate_admixture(
#'                          module = ancestry_module(number_of_founders = 3,
#'                                                   morgan = 0.8),
#'                          migration = migration_settings(
#'                                          migration_rate = 0.01,
#'                                          population_size = c(100, 100)),
#'                          total_runtime = 10)
#'  # sequence simulation
#'  data(dgrp2.3R.5k.data)
#'
#' sequence_population <-
#'       simulate_admixture(
#'                   module = sequence_module(molecular_data = dgrp2.3R.5k.data,
#'                            recombination_rate = 0.2,
#'                            mutation_rate = 1e-5),
#'                   pop_size = 1000,
#'                   total_runtime = 10)
#' @export
simulate_admixture <- function(module = ancestry_module(),
                               pop_size = 100,
                               total_runtime = 100,
                               migration = migration_settings(),
                               select_matrix = NA,
                               multiplicative_selection = TRUE,
                               verbose = FALSE,
                               num_threads = 1) {
  if (is.na(migration$migration_rate)) {
    if (module$type == "ancestry") {
      result <- simulate_ancestry(input_population = module$input_data,
                                  pop_size = pop_size,
                                  number_of_founders = module$number_of_founders,
                                  initial_frequencies =
                                    module$initial_frequencies,
                                  total_runtime = total_runtime,
                                  morgan = module$morgan,
                                  num_threads = num_threads,
                                  select_matrix = select_matrix,
                                  markers = module$markers,
                                  verbose = verbose,
                                  track_junctions = module$track_junctions,
                                  multiplicative_selection =
                                    multiplicative_selection)
      return(result)
    }

    if (module$type == "sequence") {
      result <- simulate_sequence(input_data = module$input_data,
                                  pop_size = pop_size,
                                  initial_frequencies =
                                    module$initial_frequencies,
                                  total_runtime = total_runtime,
                                  morgan = module$morgan,
                                  recombination_rate = module$recombination_rate,
                                  num_threads = num_threads,
                                  select_matrix = select_matrix,
                                  markers = module$markers,
                                  verbose = verbose,
                                  multiplicative_selection =
                                    multiplicative_selection,
                                  mutation_rate = module$mutation_rate,
                                  substitution_matrix =
                                    module$substitution_matrix)
      return(result)
    }
  } else {
    if (verbose)
      message("found positive migration rate, assuming two connected populations")

    if (module$type == "ancestry") {
      if (!inherits(module$input_population, "genomadmixr_simulation")) {
        if (is.list(module$input_population)) {
          input_population2 <- list()
          input_population2$population_1 <- module$input_population[[1]]
          input_population2$population_2 <- module$input_population[[2]]
          module$input_population <- input_population2
        } else {
          if (length(module$input_population) == 1) {
            if (is.na(module$input_population)) {
              module$input_population <- list(population_1 = c(-1e6, -1e6),
                                              population_2 = c(-1e6, -1e6))
            }
          }
        }
      }

      if (verbose) {
        message("starting simulate_ancestry_migration")
        Sys.sleep(1)
      }

      result <- simulate_ancestry_migration(input_population_1 =
                                              module$input_population$population_1,
                                            input_population_2 =
                                              module$input_population$population_2,
                                            pop_size =
                                              migration$population_size,
                                            initial_frequencies =
                                              migration$initial_frequencies,
                                            total_runtime = total_runtime,
                                            morgan = module$morgan,
                                            num_threads = num_threads,
                                            select_matrix = select_matrix,
                                            markers = module$markers,
                                            verbose = verbose,
                                            track_junctions =
                                              module$track_junctions,
                                            multiplicative_selection =
                                              multiplicative_selection,
                                            migration_rate =
                                              migration$migration_rate,
                                            stop_at_critical_fst =
                                              migration$stop_at_critical_fst,
                                            critical_fst =
                                              migration$critical_fst,
                                            generations_between_update =
                                              migration$generations_between_update,
                                            sampled_individuals =
                                              migration$sampled_individuals,
                                            number_of_markers =
                                              migration$number_of_markers,
                                            random_markers =
                                              migration$random_markers)

      return(result)
    }
    if (module$type == "sequence") {

      if (!inherits(module$input_data, "genomadmixr_simulation")) {
        if (is.list(module$input_data)) {
          input_population2 <- list()
          input_population2$population_1 <- module$input_data[[1]]
          input_population2$population_2 <- module$input_data[[2]]
          module$input_data <- input_population2
        } else {
          if (length(module$input_data) == 1) {
            if (is.na(module$input_data)) {
              module$input_data <- list(population_1 = NA,
                                        population_2 = NA)
            }
          }
        }
      }

      result <- simulate_sequence_migration(input_data_population_1 =
                                              module$input_data$population_1,
                                            input_data_population_2 =
                                              module$input_data$population_2,
                                            pop_size =
                                              migration$population_size,
                                            total_runtime = total_runtime,
                                            morgan = module$morgan,
                                            recombination_rate =
                                              module$recombination_rate,
                                            num_threads = num_threads,
                                            select_matrix = select_matrix,
                                            markers = module$markers,
                                            verbose = verbose,
                                            multiplicative_selection =
                                              multiplicative_selection,
                                            migration_rate =
                                              migration$migration_rate,
                                            stop_at_critical_fst =
                                              migration$stop_at_critical_fst,
                                            critical_fst =
                                              migration$critical_fst,
                                            generations_between_update =
                                              migration$generations_between_update,
                                            sampled_individuals =
                                              migration$sampled_individuals,
                                            number_of_markers =
                                              migration$number_of_markers,
                                            random_markers =
                                              migration$random_markers,
                                            mutation_rate =
                                              module$mutation_rate,
                                            substitution_matrix =
                                              module$substitution_matrix)

      return(result)
    }
  }
  stop("wrong module used, please pick a module with
       ancestry_module() or sequence_module()")
}

#' Calculate allele frequencies at a specific marker location
#' @description  Calculate the relative frequency of each ancestor in the
#' population at a specific marker location
#' @param pop Population for which to estimate allele frequencies at the
#' given marker
#' @param location A vector or scalar of location(s) along the chromosome for
#' which allele frequencies are to be calculated. Locations are in Morgan.
#' @return A tibble containing the frequency of each present ancestor at the
#' provided location. Ancestors with frequency = 0 are dropped out of the table.
#' The tibble contains three columns: location, ancestor and frequency.
#' @examples
#' wildpop =  simulate_admixture(
#'    module = ancestry_module(number_of_founders = 20, morgan = 1),
#'    pop_size = 1000,
#'    total_runtime = 10)
#'
#' avg_frequencies <- calculate_marker_frequency(pop = wildpop,
#'                                               location = 0.5)
#'
#' frequencies <-
#'    calculate_marker_frequency(pop = wildpop,
#'                               location = seq(0.4, 0.5, by = 0.01))
#' require(ggplot2)
#' ggplot(frequencies, aes(x = location, y = frequency, col = ancestor)) +
#'   geom_step()
#' @export
calculate_marker_frequency <- function(pop,
                                       location) {

  pop <- check_input_pop(pop)

  per_loc <- function(loc) {
    fun_chrom <- function(indiv) {
      return(c(findtype(indiv$chromosome1, loc),
               findtype(indiv$chromosome2, loc)))
    }
    types <- unlist(lapply(pop, fun_chrom))

    vv <- tibble::as_tibble(table(types))
    colnames(vv) <- c("ancestor", "frequency")

    vv$frequency <- vv$frequency / sum(vv$frequency)
    vv$location <- loc
    return(vv)
  }

  all_types <- lapply(location, per_loc)
  output <- c()
  for (i in seq_along(all_types)) {
    output <- rbind(output, all_types[[i]])
  }
  output <- output[, c("location", "ancestor", "frequency")]

  using_sequencing_data <- check_for_bases(pop)
  if (using_sequencing_data) {
    output$ancestor[output$ancestor == 0] <- 0
    output$ancestor[output$ancestor == 1] <- "a"
    output$ancestor[output$ancestor == 2] <- "c"
    output$ancestor[output$ancestor == 3] <- "t"
    output$ancestor[output$ancestor == 4] <- "g"
  }

  return(output)
}

#' Individual based simulation of the breakdown of contiguous ancestry blocks in
#' two populations linked by migration
#' @description Individual based simulation of the breakdown of contiguous
#' ancestry blocks, with or without selection. Simulations can be started from
#' scratch, or from a predefined input population. Two populations are
#' simulated, connected by migration
#' @param input_data_population_1 Genomic data used as input, should be created
#' by the function \code{create_input_data} or by the function
#' \code{combine_input_data}
#' @param input_data_population_2 Genomic data used as input, should be created
#' by thefunction \code{create_input_data} or by the function
#' \code{combine_input_data}
#' @param pop_size Vector containing the number of individuals in both
#' populations.
#' @inheritParams default_params_doc
#' @param recombination_rate rate in cM / Mbp, used to map recombination to the
#' markers. If the recombination_rate is not set, the value for morgan is used,
#' assuming that the markers included span an entire chromosome.
#' @param num_threads number of threads. Default is 1. Set to -1 to use all
#' available threads
#' @param migration_rate  Rate of migration between the two populations.
#' Migration is implemented such that with probability m (migration rate) one
#' of the two parents of a new offspring is from the other population, with
#' probability 1-m both parents are of the focal population.
#' @param stop_at_critical_fst option to stop at a critical FST value
#' , default is FALSE
#' @param critical_fst the critical fst value to stop, if
#' \code{stop_simulation_at_critical_fst} is TRUE
#' @param generations_between_update The number of generations after which the
#' simulation has to check again whether the critical Fst value is exceeded
#' @param sampled_individuals Number of individuals to be sampled at random from
#' the population to estimate Fst
#' @param number_of_markers Number of markers to be used to estimate Fst
#' @param random_markers Are the markers to estimate Fst randomly distributed,
#' or regularly distributed? Default is TRUE.
#' @param mutation_rate the per base probability of mutation. Default is 0.
#' @param substitution_matrix a 4x4 matrix representing the probability of
#' mutating to another base (where [1/2/3/4] = [a/c/t/g]), conditional on the
#' event of a mutation happening. Default is the JC69 matrix, with equal
#' probabilities for all transitions / transversions.
#' @return A list with: \code{population_1}, \code{population_2} two population
#' objects, and three tibbles with allele frequencies (only contain values of a
#' vector was provided to the argument \code{markers}: \code{frequencies},
#' \code{initial_frequencies} and \code{final_frequencies}. Each tibble contains
#' five columns, \code{time}, \code{location}, \code{ancestor}, \code{frequency}
#' and \code{population}, which indicates the number of generations, the
#' location along the chromosome of the marker, the ancestral allele at that
#' location in that generation, the frequency of that allele and the population
#' in which it was recorded (1 or 2). If a critical fst value was used to
#' terminate the simulation, and object \code{FST} with the final FST estimate
#' is returned as well.
#' @export
simulate_sequence_migration <- function(input_data_population_1 = NA, # nolint
                                        input_data_population_2 = NA,
                                        pop_size = c(100, 100),
                                        total_runtime = 100,
                                        morgan = 1,
                                        recombination_rate = NA,
                                        num_threads = 1,
                                        select_matrix = NA,
                                        markers = NA,
                                        verbose = FALSE,
                                        multiplicative_selection = TRUE,
                                        migration_rate = 0.0,
                                        stop_at_critical_fst = FALSE,
                                        critical_fst = NA,
                                        generations_between_update = 100,
                                        sampled_individuals = 10,
                                        number_of_markers = 100,
                                        random_markers = TRUE,
                                        mutation_rate = 0.0,
                                        substitution_matrix =
                                          matrix(1 / 4, 4, 4)) {

  input_data_population_1 <- verify_genomeadmixr_data(input_data_population_1,
                                                      verbose = verbose)
  input_data_population_2 <- verify_genomeadmixr_data(input_data_population_2,
                                                      verbose = verbose)
  set_num_threads(num_threads)
  if (!is.na(critical_fst)) {
    stop_at_critical_fst <- TRUE
  }

  if (stop_at_critical_fst) {
    if (verbose) message("stopping at FST of: ", critical_fst)
    return(simulate_sequence_until(input_data_population_1 =
                                     input_data_population_1,
                                   input_data_population_2 =
                                     input_data_population_2,
                                   pop_size = pop_size,
                                   total_runtime = total_runtime,
                                   morgan = morgan,
                                   select_matrix = select_matrix,
                                   markers = markers,
                                   verbose = verbose,
                                   multiplicative_selection =
                                     multiplicative_selection,
                                   migration_rate = migration_rate,
                                   critical_fst = critical_fst,
                                   generations_between_update =
                                     generations_between_update,
                                   sampled_individuals = sampled_individuals,
                                   number_of_markers = number_of_markers,
                                   random_markers = random_markers,
                                   mutation_rate = mutation_rate,
                                   substitution_matrix = substitution_matrix,
                                   num_threads = num_threads,
                                   recombination_rate = recombination_rate))
  }

  if (!inherits(input_data_population_1, "genomeadmixr_data") ||
      !inherits(input_data_population_2, "genomeadmixr_data")) {
    reset_num_threads()
    stop("input_data should be of class genomeadmixr_data\n
          you can create such data with the functions\n
          create_input_data or vcfR_to_genomeadmixr_data")
  }

  select_matrix <- check_select_matrix(select_matrix,
                                       markers,
                                       use_data = TRUE)

  if (length(pop_size) == 1) {
    if (is.na(pop_size) ) {
      stop("need to provide population size")
    }
    pop_size <- c(pop_size, pop_size)
  }


  if (length(markers) == 1) {
    if (is.na(markers))  {
      markers <- c(-1, -1)
      track_frequency <- FALSE
    } else {
      markers <- c(markers)
      track_frequency <- TRUE
    }
  } else {
    track_frequency <- TRUE
  }

  if (track_frequency) {
    markers <- check_markers(markers, input_data_population_1$markers)
  }

  if (verbose) {
    message("markers: ", length(markers), "\n")
  }

  if (mutation_rate > 0) {
    verify_substitution_matrix(substitution_matrix)
  }

  recombination_map <- c(-1, -1)
  if (!is.na(recombination_rate)) {
    recombination_map <-
      create_recombination_map(input_data_population_1$markers,
                               recombination_rate)
  }

  selected_pop <- simulate_migration_emp_cpp(input_data_population_1$genomes,
                                             input_data_population_2$genomes,
                                             input_data_population_1$markers,
                                             select_matrix,
                                             pop_size,
                                             total_runtime,
                                             morgan,
                                             verbose,
                                             track_frequency,
                                             markers,
                                             multiplicative_selection,
                                             migration_rate,
                                             mutation_rate,
                                             substitution_matrix,
                                             num_threads,
                                             recombination_map)

  selected_popstruct_1 <- create_pop_class(selected_pop$population_1)
  selected_popstruct_2 <- create_pop_class(selected_pop$population_2)

  colnames(selected_pop$initial_frequencies) <- c("time",
                                                  "location",
                                                  "ancestor",
                                                  "frequency",
                                                  "population")

  colnames(selected_pop$final_frequencies) <- c("time",
                                                "location",
                                                "ancestor",
                                                "frequency",
                                                "population")

  initial_freq_tibble <- tibble::as_tibble(selected_pop$initial_frequencies)
  final_freq_tibble   <- tibble::as_tibble(selected_pop$final_frequencies)

  output <- generate_output_list_two_pop(selected_pop,
                                         selected_popstruct_1,
                                         selected_popstruct_2,
                                         initial_freq_tibble,
                                         final_freq_tibble,
                                         track_frequency)

  class(output) <- "genomadmixr_simulation"
  reset_num_threads()
  return(output)
}

#' @keywords internal
calc_allele_frequencies <- function(indiv, alleles) {

  for (i in seq_along(indiv$chromosome1[, 1])) {
    left <- indiv$chromosome1[i, 1]
    right <- 1
    if (i + 1 <= length(indiv$chromosome1[, 1])) {
      right <- indiv$chromosome1[i + 1, 1]
    }

    allele <- 1 + indiv$chromosome1[i, 2]
    alleles[allele] <- alleles[allele] + (right - left)
  }

  for (i in seq_along(indiv$chromosome2[, 1])) {
    left <- indiv$chromosome2[i, 1]
    right <- 1
    if (i + 1 <= length(indiv$chromosome2[, 1])) {
      right <- indiv$chromosome2[i + 1, 1]
    }

    allele <- 1 + indiv$chromosome2[i, 2]
    alleles[allele] <- alleles[allele] + (right - left)
  }

  alleles <- alleles / sum(alleles)
  return(alleles)
}

#' A subset of sequencing data from the Drosophila Genetics Reference Panel
#' @description This data set contains sequences from the 3R chromosome.
#' Included are 4603 SNPs with at least 0.05 minor allele frequency,  sequenced
#' across 410 isofemale lines. Sequences were downloaded from
#' <http://dgrp2.gnets.ncsu.edu/data.html>.
#' @references Mackay, T., Richards, S., Stone, E. et al. The Drosophila
#' melanogaster Genetic Reference Panel. Nature 482, 173–178 (2012).
#' <https://doi.org/10.1038/nature10811>
#' @format genomeadmixr_data object
#' @examples
#' data("dgrp2.3R.5k.data")
#' simulate_admixture(
#'        module = sequence_module(molecular_data = dgrp2.3R.5k.data),
#'        pop_size = 100,
#'        total_runtime = 10)
#' @usage data("dgrp2.3R.5k.data")
"dgrp2.3R.5k.data"

#' function to simulate creation of an isofemale line
#' @description create_isofemale simulates the creation of an isofemale line
#' through extreme inbreeding.
#' @param module Source population from which isofemales are generated
#' @param n Number of isofemales to be generated
#' @param inbreeding_pop_size Population size of the population used to generate
#' homozygous individuals
#' @param run_time Maximum runtime used for inbreeding
#' @param num_threads number of threads. Default is 1. Set to -1 to use all
#' available threads
#' @param verbose Displays verbose output if TRUE. Default value is FALSE
#' @details To create an isofemale, two individuals are randomly picked from
#' the source population. Using these two individuals, a new population is
#' seeded, of size \code{inbreeding_pop_size}. Then, this population is allowed
#' to inbreed until either \code{run_time} is reached, or until all individuals
#' are homozygous and genetically identical, whatever happens first.
#' @return A list of length \code{n}, where each entry is a fully homozygous
#' isofemale.
#' @export
create_iso_female <- function(module = ancestry_module(),
                              n = 1,
                              inbreeding_pop_size = 100,
                              run_time = 2000,
                              num_threads = 1,
                              verbose = FALSE) {
  if (module$type == "ancestry") {
    result <- iso_female_ancestry(source_pop = module$input_population,
                                  n = n,
                                  inbreeding_pop_size = inbreeding_pop_size,
                                  run_time = run_time,
                                  morgan = module$morgan,
                                  num_threads = num_threads,
                                  verbose = verbose)
    return(result)
  }
  if (module$type == "sequence") {
    result <- iso_female_sequence(input_data = module$input_data,
                                  n = n,
                                  inbreeding_pop_size = inbreeding_pop_size,
                                  run_time = run_time,
                                  morgan = module$morgan,
                                  recombination_rate = module$recombination_rate,
                                  num_threads = num_threads,
                                  verbose = verbose)
    return(result)
  }
  stop("wrong module used, please pick a module with
       ancestry_module() or sequence_module()")
}


#' Create isofemale
#' @description Creates isofemale individuals, given a population
#' @param source_pop Source population from which isofemales are generated
#' @param n Number of isofemales to be generated
#' @param inbreeding_pop_size Population size of the population used to generate
#' homozygous individuals
#' @param run_time Maximum runtime used for inbreeding
#' @param morgan Size of the chromosome in Morgan (e.g. the number of crossovers
#' during meiosis)
#' @param num_threads number of threads. Default is 1. Set to -1 to use all
#' available threads
#' @param verbose Displays verbose output if TRUE. Default value is FALSE
#' @details To create an isofemale, two individuals are randomly picked from
#' the source population. Using these two individuals, a new population is
#' seeded, of size \code{inbreeding_pop_size}. Then, this population is allowed
#' to inbreed until either \code{run_time} is reached, or until all individuals
#' are homozygous and genetically identical, whatever happens first.
#' @return A list of length \code{n}, where each entry is a fully homozygous
#' isofemale.
#' @export
iso_female_ancestry <- function(source_pop = NA,
                                n = 1,
                                inbreeding_pop_size = 100,
                                run_time = 2000,
                                morgan = 1,
                                num_threads = 1,
                                verbose = FALSE) {

  source_pop <- check_input_pop(source_pop)

  # first we select the individuals that will be the parents of the isofemales
  indices <- sample(seq_along(source_pop), n * 2, replace = FALSE)

  #for each isofemale
  #pick the female, then simulate until run_time
  #then pick a random individual (assuming the population is fixed now)

  iso_females <- source_pop[indices]
  output_females <- list()
  for (i in 1:n) {
    parents <- list(iso_females[[i]], iso_females[[i + n]])
    class(parents) <- "population"

    inbred_population <- simulate_admixture(module = ancestry_module(
      input_population = parents,
      morgan = morgan
    ),
    pop_size = inbreeding_pop_size,
    total_runtime = run_time,
    verbose = verbose,
    num_threads = num_threads)
    output_females[[i]] <-
      inbred_population$population[[
        sample(seq_along(inbred_population$population), 1)
      ]]

    class(output_females[[i]]) <- "individual"
  }
  return(output_females)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

calculate_allele_spectrum_cpp <- function(input_population, markers, progress_bar) {
    .Call(`_GenomeAdmixR_calculate_allele_spectrum_cpp`, input_population, markers, progress_bar)
}

simulation_data_to_genomeadmixr_data_cpp <- function(input_population, markers) {
    .Call(`_GenomeAdmixR_simulation_data_to_genomeadmixr_data_cpp`, input_population, markers)
}

simulation_data_to_plink_cpp <- function(input_population, markers) {
    .Call(`_GenomeAdmixR_simulation_data_to_plink_cpp`, input_population, markers)
}

calculate_heterozygosity_cpp <- function(input_population, markers, progress_bar) {
    .Call(`_GenomeAdmixR_calculate_heterozygosity_cpp`, input_population, markers, progress_bar)
}

vcf_to_matrix_cpp <- function(input_mat, allele_1, allele_2) {
    .Call(`_GenomeAdmixR_vcf_to_matrix_cpp`, input_mat, allele_1, allele_2)
}

simulate_cpp <- function(input_population, select, pop_size, number_of_founders, starting_proportions, total_runtime, morgan, verbose, track_frequency, track_markers, track_junctions, multiplicative_selection, num_threads) {
    .Call(`_GenomeAdmixR_simulate_cpp`, input_population, select, pop_size, number_of_founders, starting_proportions, total_runtime, morgan, verbose, track_frequency, track_markers, track_junctions, multiplicative_selection, num_threads)
}

simulate_emp_cpp <- function(input_population, marker_positions_R, select, pop_size, total_runtime, morgan, verbose, track_frequency, track_markers_R, multiplicative_selection, mutation_rate, substitution_matrix_R, num_threads, recombination_map) {
    .Call(`_GenomeAdmixR_simulate_emp_cpp`, input_population, marker_positions_R, select, pop_size, total_runtime, morgan, verbose, track_frequency, track_markers_R, multiplicative_selection, mutation_rate, substitution_matrix_R, num_threads, recombination_map)
}

simulate_migration_cpp <- function(input_population_1, input_population_2, select, pop_size, starting_frequencies, total_runtime, morgan, verbose, track_frequency, track_markers, track_junctions, multiplicative_selection, migration_rate, num_threads) {
    .Call(`_GenomeAdmixR_simulate_migration_cpp`, input_population_1, input_population_2, select, pop_size, starting_frequencies, total_runtime, morgan, verbose, track_frequency, track_markers, track_junctions, multiplicative_selection, migration_rate, num_threads)
}

simulate_migration_emp_cpp <- function(input_population_1, input_population_2, marker_positions_R, select, pop_sizes, total_runtime, morgan, verbose, track_frequency, track_markers_R, multiplicative_selection, migration_rate, mutation_rate, substitution_matrix_R, num_threads, recombination_map) {
    .Call(`_GenomeAdmixR_simulate_migration_emp_cpp`, input_population_1, input_population_2, marker_positions_R, select, pop_sizes, total_runtime, morgan, verbose, track_frequency, track_markers_R, multiplicative_selection, migration_rate, mutation_rate, substitution_matrix_R, num_threads, recombination_map)
}

#' Calculate linkage disequilibrium statistics
#' This function calculates two matrices, once containing all pairwise
#' linkage disequilibrium (ld) values, and one matrix containing all pairwise r
#' statistics
#' @param pop focal population
#' @param sampled_individuals Number of individuals randomly sampled to
#' calculate the LD matrices
#' @param markers vector of markers. If a single number is used, that number of
#' markers is randomly placed along the genome.
#' @param verbose display verbose output, default is FALSE.
#' @return An object containing two items:
#' \item{ld_matrix}{
#'   Pairwise ld statistics for all markers
#' }
#' \item{rsq_matrix}{
#'   Pairwise rsq statistics for all markers
#' }
#'@examples
#' wildpop =  simulate_admixture(
#'    module = ancestry_module(number_of_founders = 10, morgan = 1),
#'    pop_size = 1000,
#'    total_runtime = 10)
#'
#' ld_results <- calculate_ld(pop = wildpop,
#'                            markers = 10)
#'
#' plot(ld_results$ld_matrix~ld_results$dist_matrix,
#'      pch = 16,
#'      xlab="Distance between markers",
#'      ylab = "Linkage Disequilibrium")
#' @export
calculate_ld <- function(pop,
                         sampled_individuals = 10,
                         markers = NA,
                         verbose = FALSE) {

  pop <- check_input_pop(pop)

  if (length(markers) == 1) {
    if (is.na(markers)) {
      if (inherits(pop, "genomadmixr_data")) {
        markers <- pop$markers
      }
    } else {
      markers <- create_random_markers(markers[1])
    }
  }

  popx <- pop[1:sampled_individuals]
  class(popx) <- "population"
  pop_for_cpp <- population_to_vector(popx)
  if (verbose) message("starting creation of allele matrix")
  all_loci <- simulation_data_to_genomeadmixr_data_cpp(pop_for_cpp,
                                                        markers)

  all_loci[all_loci < 0] <- NA

  ld_matrix   <- matrix(nrow = length(markers), ncol = length(markers), NA)
  rsq_matrix  <- matrix(nrow = length(markers), ncol = length(markers), NA)
  dist_matrix <- matrix(nrow = length(markers), ncol = length(markers), NA)

  if (verbose) message("done creation of allele matrix")

  if (verbose) pb <- utils::txtProgressBar(min = 0,
                                           max = length(markers),
                                           style = 3)
  for (x in seq_along(markers)) {
    for (y in seq_len(x)) {
      if (x != y) {
        g1 <- all_loci[, x]
        g2 <- all_loci[, y]

        ld <- calculate_average_ld(g1, g2)
        ld_matrix[x, y] <- ld$LD
        rsq_matrix[x, y] <- ld$r_sq
        gen_dist <- abs(markers[x] - markers[y])
        dist_matrix[x, y] <- gen_dist
      }
    }
    if (verbose) utils::setTxtProgressBar(pb, x)
  }

  return(list("ld_matrix" = ld_matrix,
              "rsq_matrix" = rsq_matrix,
              "dist_matrix" = dist_matrix))
}

count_ab <- function(alleles_pos_1, alleles_pos_2, a, b) {
  v1 <- alleles_pos_1 == a
  v2 <- alleles_pos_2 == b
  total_count <- sum((v1 == v2) & v1 == TRUE)
  return(total_count)
}


#' Calculates the  ld between two alleles
#' @description calculate the average ld between two loci
#' @param alleles_pos_1 alleles at locus 1
#' @param alleles_pos_2 alleles at locus 2
#' @return a list with two entries: LD and r_squared
#' @export
calculate_average_ld <- function(alleles_pos_1,
                                 alleles_pos_2) {
  all_alleles <- c(as.vector(alleles_pos_1), as.vector(alleles_pos_2))

  if (sum(is.na(all_alleles)) > 0) {
    return(list("LD" = NA,
                "r_sq" = NA))
  }

  all_alleles <- sort(unique(all_alleles))

  ld <- 0
  r_squared <- 0
  for (i in all_alleles) {
    for (j in all_alleles) {
      p_a_i <- length(which(alleles_pos_1 == i)) / length(alleles_pos_1)
      p_b_j <- length(which(alleles_pos_2 == j)) / length(alleles_pos_2)

      countab <- count_ab(alleles_pos_1, alleles_pos_2, i, j)

      p_a_i_b_j <- countab / (length(alleles_pos_1))

      if (is.nan(p_a_i_b_j)) p_a_i_b_j <- 0

      d_i_j <- p_a_i_b_j - p_a_i * p_b_j

      d_i_j_max <- 1

      if (d_i_j < 0) {
        d_i_j_max <- min(p_a_i * p_b_j, (1 - p_a_i) * (1 - p_b_j))
      } else {
        d_i_j_max <- min(p_a_i * (1 - p_b_j), (1 - p_a_i) * p_b_j)
      }

      if (d_i_j_max > 0) {
        ld <- ld + p_a_i * p_b_j * abs(d_i_j / d_i_j_max)
        r_i_j <- (d_i_j ^ 2) / (p_a_i * (1 - p_a_i) * p_b_j * (1 - p_b_j))
        r_squared <- r_squared + p_a_i * p_b_j * r_i_j
      }
    }
  }

  return(list("LD" = ld,
              "r_sq" = r_squared)
  )
}

#' Plot the change in frequency between the start and end of a simulation
#' @description This function plots the change in frequency of one or
#' multiple ancestors after performing a simulation.
#' @param results An object which is the result of \code{simulate_admixture}
#'  being a list with four properties:
#' \code{population}, \code{frequencies}, \code{initial_frequencies} and
#' \code{final frequencies}
#' @param picked_ancestor Default is "ALL", where different colors indicate
#' different ancestors. Alternatively, for clarity, the user can specify a
#' specific ancestral allele, and only that allele is plotted
#' @param picked_population If multiple populations were simulated (in the case
#' of \code{simulate_admixture_migration}), which population should be plotted?
#' Default is population_1.
#' @return a ggplot2 object
#' @examples
#' s <- 0.1
#' select_matrix <- matrix(nrow = 1, ncol = 5)
#' select_matrix[1, ] <- c(0.25, 1.0, 1 + 0.5 * s, 1 + s, 0)
#'
#' markers <- seq(from = 0.2, to = 0.3, length.out = 100)
#'
#' selected_pop <- simulate_admixture(
#'                     module = ancestry_module(number_of_founders = 10,
#'                                              morgan = 1,
#'                                              markers = markers),
#'                     pop_size = 1000,
#'                     total_runtime = 11,
#'                     select_matrix = select_matrix)
#' require(ggplot2)
#' plot_difference_frequencies(results = selected_pop,
#'                             picked_ancestor = "ALL")
#' @export
plot_difference_frequencies <- function(results,
                                        picked_ancestor = "ALL",
                                        picked_population = 1) {

  a1 <- results$initial_frequency
  a2 <- results$final_frequency

  if ("population" %in% colnames(a1)) {
    a1 <- subset(a1,
                 a1$population == picked_population)
    a2 <- subset(a2,
                 a2$population == picked_population)
  }

  a1 <- dplyr::select(a1, c("time", "location", "ancestor", "frequency"))
  a2 <- dplyr::select(a2, c("time", "location", "ancestor", "frequency"))

  colnames(a1) <- c("time",  "location", "ancestor", "frequency_before")
  colnames(a2) <- c("time",  "location", "ancestor", "frequency_after")

  ax <- dplyr::full_join(a1, a2, by = c("location", "ancestor"))

  ax_m <- dplyr::mutate(ax,
                        "diff_frequency" =
                          ax$frequency_after - ax$frequency_before)

  if (picked_ancestor[[1]] == "ALL") {
    to_plot <- ax_m

    p1 <- ggplot2::ggplot(to_plot,
                          ggplot2::aes(x = .data[["location"]],
                                       y = .data[["diff_frequency"]],
                                    colour = as.factor(.data[["ancestor"]]))) +
      ggplot2::geom_step()
  } else {

    to_plot <- subset(ax_m,
                      ax_m$ancestor %in% picked_ancestor)

    p1 <- ggplot2::ggplot(to_plot,
                          ggplot2::aes(x = .data[["location"]],
                                       y = .data[["diff_frequency"]],
                                    colour = as.factor(.data[["ancestor"]]))) +
      ggplot2::geom_step()
  }

  if (max(to_plot$location) < 10) {
    p1 <- p1 +
      ggplot2::xlab("Location (Morgan)")
  } else {
    p1 <- p1 +
      ggplot2::xlab("Location (bp)")
  }

  p1 <- p1 +
    ggplot2::ylab("Change in Frequency") +
    ggplot2::labs(col = "Ancestor")

  return(p1)
}

#' plots a chromosome
#' @description This function plots a chromosome in the range [xmin, xmax].
#' Colors indicate different ancestry.
#' @param chrom  object of type chromosome, typically a table with two columns.
#' The first column indicates the start of an ancestry block (location in
#' Morgan), the second column indicates the ancestry type.
#' @param xmin minimum value of the range, default = 0.
#' @param xmax maximum value of the range, default = 1.
#' @return No return value
#' @examples
#' wildpop =  simulate_admixture(
#'    module = ancestry_module(number_of_founders = 10, morgan = 1),
#'    pop_size = 1000,
#'    total_runtime = 10)
#'
#' isofemale <- create_iso_female(
#'                  module = ancestry_module(input_population = wildpop,
#'                                           morgan = 1),
#'                  n = 1,
#'                  inbreeding_pop_size = 100,
#'                  run_time = 10)
#'
#' plot_chromosome(chrom = isofemale[[1]]$chromosome1)
#' # and a detail of the chromosome:
#' plot_chromosome(chrom = isofemale[[1]]$chromosome1,
#'                 xmin = 0.4,
#'                 xmax = 0.6)
#' @export
plot_chromosome <- function(chrom, xmin = 0, xmax = 1) {
  alleles <- unique(chrom[, 2])
  num_colors <- 1 + max(alleles)
  if (num_colors > 20) num_colors <- 20
  color_palette <- grDevices::rainbow(num_colors, alpha = 1)

  if (max(chrom[, 1]) > 1 && xmax == 1) {
    xmax <- max(chrom[, 1])
  }

  graphics::plot(NA,
       xlim = c(xmin, xmax),
       ylim = c(0, 1),
       xlab = "",
       ylab = "",
       xaxt = "n",
       yaxt = "n",
       bty  = "n")

  for (i in seq_along(chrom[, 1])) {
    xleft <- chrom[i, 1]
    xrght <- 1
    if (i < length(chrom[, 1])) {
      xrght <- chrom[i + 1, 1]
    }
    colour_index <- 1 + chrom[i, 2]
    colour_to_plot <- color_palette[colour_index]

    graphics::rect(xleft = xleft,
         xright = xrght,
         ybottom = 0,
         ytop = 1,
         col = colour_to_plot,
         border = NA)
  }
}

#' collect the full distribution of junctions in the population
#' @description calculates the distribution of junctions across the population
#' @param pop object of the class 'population'
#' @return vector with two entries per individual, each indicating the number of
#' junctions in the respective chromosomes
#' @export
calculate_dist_junctions <- function(pop) {
  get_num_junctions <- function(indiv) {
    v1 <- length(indiv$chromosome1[, 1]) - 1
    v2 <- length(indiv$chromosome2[, 1]) - 1 #subract one for start
    return(c(v1, v2))
  }

  vx <- unlist(lapply(pop, get_num_junctions))

  return(vx)
}

#' plot the distribution of junctions
#' @description plots the distribution of junctions in the population using
#' base R
#' @param pop of the class 'population'
#' @return No return value
#' @export
plot_dist_junctions <- function(pop) {
  junct <- calculate_dist_junctions(pop)
  vx <- table(junct)
  graphics::barplot(vx)
}

test_that("expected_number_junctions", {

  if (requireNamespace("junctions")) {

    test_expected_junction_number <- function(pop_size,
                                              run_time,
                                              morgan,
                                              replicates) {
      found <- c()
      for (r in 1:replicates) {
        vx <- simulate_admixture(module = ancestry_module(morgan = morgan),
                                 pop_size = pop_size,
                                 total_runtime = run_time)

        junct <- calculate_dist_junctions(vx$population)
        found <- c(found, mean(junct))
      }


      if (requireNamespace("junctions")) {
        expected <- junctions::number_of_junctions(N = pop_size,
                                                   H_0 = 0.5,
                                                   C = morgan,
                                                   t = run_time)

        testthat::expect_equal(mean(found), expected, tolerance = 1)
      }
    }

    used_pop_size <- 100

    test_expected_junction_number(pop_size = used_pop_size, run_time = 20,
                                  morgan = 1, replicates = 100)

    test_expected_junction_number(pop_size = used_pop_size, run_time = 20,
                                  morgan = 0.5, replicates = 100)

    test_expected_junction_number(pop_size = used_pop_size, run_time = 20,
                                  morgan = 3, replicates = 100)

    vx <- simulate_admixture(pop_size = used_pop_size,
                             total_runtime = 5)

    testthat::expect_silent(plot_dist_junctions(vx$population))
  }
})

#' @keywords internal
simulate_ancestry_until <- function(input_population_1 = NA,
                                     input_population_2 = NA,
                                     pop_size = c(100, 100),
                                     initial_frequencies = list(c(1.0, 0),
                                                                c(0, 1.0)),
                                     total_runtime = 100,
                                     morgan = 1,
                                     num_threads = 1,
                                     select_matrix = NA,
                                     markers = NA,
                                     verbose = FALSE,
                                     track_junctions = FALSE,
                                     multiplicative_selection = TRUE,
                                     migration_rate = 0.0,
                                     generations_between_update = 100,
                                     critical_fst = 0.1,
                                     sampled_individuals = 10,
                                     number_of_markers = 100,
                                     random_markers = TRUE) {

  pops <- simulate_ancestry_migration(
    input_population_1 = input_population_1,
    input_population_2 = input_population_2,
    pop_size = pop_size,
    initial_frequencies = initial_frequencies,
    total_runtime = generations_between_update,
    num_threads = num_threads,
    morgan = morgan,
    select_matrix = select_matrix,
    markers = markers,
    verbose = verbose,
    track_junctions = track_junctions,
    multiplicative_selection = multiplicative_selection,
    migration_rate = migration_rate)

  fst <- calculate_fst(pop1 = pops$population_1,
                       pop2 = pops$population_2,
                       sampled_individuals = sampled_individuals,
                       number_of_markers = number_of_markers,
                       random_markers = random_markers)

  if (verbose) message("Number of Generations\tFST")
  if (verbose) message(generations_between_update, "\t", fst)

  total_generations <- generations_between_update
  while (fst < critical_fst && total_generations < total_runtime) {

    pops <- simulate_ancestry_migration(
      input_population_1 = pops$population_1,
      input_population_2 = pops$population_2,
      pop_size = pop_size,
      initial_frequencies = initial_frequencies,
      total_runtime = generations_between_update,
      morgan = morgan,
      num_threads = num_threads,
      select_matrix = select_matrix,
      markers = markers,
      verbose = verbose,
      track_junctions = track_junctions,
      multiplicative_selection = multiplicative_selection,
      migration_rate = migration_rate)

    fst <- calculate_fst(pops$population_1, pops$population_2,
                         sampled_individuals = sampled_individuals,
                         number_of_markers = number_of_markers,
                         random_markers = random_markers)

    total_generations <- total_generations + generations_between_update
    if (verbose) message(total_generations, "\t", fst)
  }
  reset_num_threads()
  return(list("population_1" = pops$population_1,
              "population_2" = pops$population_2,
              "Number_of_generations" = total_generations,
              "FST" = fst))
}

#' Calculate heterozygosity
#' @description Calculate the average population level heterozygosity
#' @param source_pop Population for which to estimate allele frequencies, or a
#' list of individuals for which to calculate average heterozygosity
#' @param locations A vector indicating the locations (in Morgan) of markers for
#' which to calculate the heterozygosity
#' @param progress_bar Displays a progress_bar if TRUE. Default value is TRUE
#' @return A tibble containing the heterozygosities
#' @export
calculate_heterozygosity <- function(source_pop,
                                     locations,
                                     progress_bar = FALSE) {

  source_pop <- check_input_pop(source_pop)

  pop_for_cpp <- population_to_vector(source_pop)

  heterozygosity_table <- calculate_heterozygosity_cpp(pop_for_cpp,
                                                       locations,
                                                       progress_bar)

  output <- heterozygosity_table
  colnames(output) <- c("location", "heterozygosity")
  output <- tibble::as_tibble(output)

  return(output)
}

#' Save a population to file
#' @description Saves a population to file for later use
#' @param population Object of class \code{population}
#' @param file_name Name of the file to save the population
#' @param compression By default, the population is compressed to reduce file
#' size. See for more information \code{saveRDS}
#' @details This function functions as a wrapper for the base function
#' \code{saveRDS}.
#' @return No return value
#' @export
save_population <- function(population, file_name, compression = TRUE) {
  saveRDS(population, file = file_name, compress = compression)
}

#' Load a population from file
#' @description Loads a population that has previously been written to file.
#' @param file_name Name of the file to save the population
#' @return A population object
#' @seealso \code{\link{save_population}}
#' @details This function is a wrapper for \code{readRDS}.
#' @export
load_population <- function(file_name) {
  readRDS(file_name)
}

context("create_populations")

test_that("save_population", {
  pop_size <- 100
  number_of_founders <- 10
  run_time <- 10
  morgan <- 1

  vx <- simulate_admixture(module = ancestry_module(number_of_founders =
                                                      number_of_founders,
                                                    morgan = morgan),
                           pop_size = pop_size,
                           total_runtime = run_time)

  testthat::expect_true(verify_population(vx))

  save_population(vx, file_name = "test.pop")

  vy <- load_population(file_name = "test.pop")

  testthat::expect_true(verify_population(vy$population))

  testthat::expect_equal(length(vx$population), length(vy$population))

  for (i in seq_along(vx)) {
    testthat::expect_true(all.equal(vx[[i]], vy[[i]]))
  }
  testthat::expect_true(file.remove("test.pop"))
})

test_that("data", {
  data("dgrp2.3R.5k.data")
  testthat::expect_equal(length(dgrp2.3R.5k.data$markers),
                         4603)

  testthat::expect_equal(length(dgrp2.3R.5k.data$genomes[, 1]),
                         410)
})

---
title: "Walkthrough"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Walkthrough}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )
```

# GenomeAdmixR
The package GenomeAdmixR was designed to simulate the generation of iso-female
lines, and simulate genomic changes during, and after, formation of iso-female 
lines. Furthermore, it allows for simulation of effects after crossing 
individuals from iso-female lines.

```{r}
library(GenomeAdmixR)
library(ggplot2)
packageVersion("GenomeAdmixR")
```


## Simulating an iso-female line
In order to create an iso-female line, we first have to create a 'wild' 
population, from which we will draw individuals. To create such a population, we 
make use of the function 'simulate_admixture', using the ancestry module:
```{r create wildpop}
wildpop <-  simulate_admixture(
                    module = ancestry_module(number_of_founders = 4,
                                             morgan = 1),
                    pop_size = 100,
                    total_runtime = 1000,
                    num_threads = 1)
```
This creates a population of 100 diploid individuals (e.g. 2N = 200), based upon 
10 founders (e.g. 10 individuals that can be genetically distinguished from each 
other). Then, for 1000 generations, this population is inbred and genomes of the 
20 founders are allowed to mix. All individuals have 2 chromosomes (for 
simplicity and computational speed, we only model one chromosome pair), which 
are 1 Morgan long. The result can be written to file (optional), which can be 
usefull when generating for instance an extremely large wild population, which 
can be used later (e.g. read from file).

The result of the function is a structure containing 1000 individuals:
```{r}
wildpop
```
Each individual has two chromosomes, with a given number of junctions (e.g. 
delineations between two contiguous genomic stretches from different ancestors):
```{r}
wildpop[[1]]
```

In order to create an iso-female line, two random individuals are drawn from the 
wild population, and selected for inbreeding. We can simulate this process with:
```{r create isofemale}
isofemale <- create_iso_female(
                  module = ancestry_module(input_population = wildpop,
                                           morgan = 1),
                  n = 1,
                  inbreeding_pop_size = 1000,
                  run_time = 200,
                  num_threads = 1)
```

Here, n indicates the number of isofemales to be created from the same source 
population, the inbreeding population size is set to be small, in order to 
speed up computation. Maximum run time is best set hight, to assure that all 
individuals become genetically identical. 

# Visualizing individuals
Now, we can plot the isofemales, this plots the two chromosomes next to each 
other, where colors indicate different ancestors:
```{r}
plot(isofemale[[1]])
```

Because we have chosen 4 ancestors, these plots are not terribly informative, 
let's try a toy example:
```{r toyexample}
wildpop <-  simulate_admixture(
                  module = ancestry_module(number_of_founders = 4,
                                           morgan = 1),
                  pop_size = 100,
                  num_threads = 1,
                  total_runtime = 100)

isofemale <- create_iso_female(
                  module = ancestry_module(input_population = wildpop,
                                           morgan = 1),
                  n = 1,
                  inbreeding_pop_size = 100,
                  num_threads = 1,
                  run_time = 10000)
plot(wildpop$population[[1]])
plot(isofemale[[1]])
```

We can also plot a (section of) a single chromosome:
```{r plot selection of chromosome}
plot_chromosome(isofemale[[1]]$chromosome1, xmin = 0.0, xmax = 0.5)
```

# Multiple source populations

It gets more interesting if we create two populations, and draw iso-females from 
them, and cross these.
First, we have to create two populations. 
```{r create two populations}
both_populations <-
  simulate_admixture(
    module = ancestry_module(morgan = 1),
    migration = migration_settings(population_size = c(100, 100),
                        migration_rate = 0.0,
                        initial_frequencies =
                                 list(c(rep(1, 20), rep(0, 20)),
                                      c(rep(0, 20), rep(1, 20)))
                      ),
    total_runtime = 1000,
    num_threads = 1)

population_1 <- both_populations$population_1

population_2 <- both_populations$population_2
```
To make sure that the two populations dont share ancestor indices, we use the 
function 'increase_ancestor'. 

Now that we have two populations, we can generate two different iso-females 
from them:
```{r draw two isofemales}
isofemales <- create_iso_female(
                  module = ancestry_module(input_population = population_1,
                                           morgan = 1),
                  n = 2,
                  inbreeding_pop_size = 100,
                  run_time = 10000,
                  num_threads = 1)

plot_chromosome(isofemales[[1]]$chromosome1, 0, 1)
plot_chromosome(isofemales[[2]]$chromosome1, 0, 1)
```

We can now use these two isofemales to seed a new inbreeding population:
```{r seed mixed population}
mixed_population <-
  simulate_admixture(
        module = ancestry_module(input_population = list(isofemales[[1]],
                                                         isofemales[[2]]),
                                 morgan = 1),
        pop_size = 100,
        num_threads = 1,
        total_runtime = 100)
```
And plot some individuals from our new population:
```{r plot mixed_population}
plot(mixed_population$population[[1]])
```

# Statistics

## FST
We can show the effect of overlap by calculating the Fst value:
```{r calc FST}
fst <- calculate_fst(population_1,
                 population_2,
                 sampled_individuals = 10,
                 number_of_markers = 100,
                 random_markers = TRUE)
fst
```
The FST calculation function uses the library hierfstat to calculate the FST. To 
do so, it samples 10 random individuals (the parameter sampled individuals) from 
the population, and then assesses the genetic content at 100 randomly placed 
markers (number of markers, and random_markers = TRUE). To increase power, a 
higher number of individuals can be sampled, although this does increase 
computational load.

## LD
We can also calculate LD statistics. LD is only defined for markers, so again we 
have to simulate artificial markers along the chromosome. 

```{r}
  ld_results <- calculate_ld(wildpop,
                             markers = 10)

  plot(ld_results$ld_matrix~ld_results$dist_matrix,
       xlab = "Genetic Distance (Morgan)",
       ylab = "LD",
       pch = 16)
  plot(ld_results$rsq_matrix~ld_results$dist_matrix,
       xlab = "Genetic Distance (Morgan)",
       ylab = "r_sq",
       pch = 16)
  plot(ld_results$ld_matrix~ld_results$rsq_matrix,
       xlab = "r_sq",
       ylab = "LD",
       pch = 16)
```

To analyze LD patterns better, it makes more sense to create a population from 
scratch. We expect for a strongly inbred population, that there is no LD:

```{r no LD}
no_ld_pop <- simulate_admixture(
                  module = ancestry_module(number_of_founders = 4,
                                           morgan = 1),
                  pop_size = 100,
                  num_threads = 1,
                  total_runtime = 1000)

ld_results <- calculate_ld(no_ld_pop,
                           sampled_individuals = 10,
                           markers = 10)

plot(ld_results$ld_matrix~ld_results$dist_matrix,
     pch = 16,
     xlab = "Distance",
     ylab = "LD",
     xlim = c(0, 1),
     ylim = c(0, 1))
```

Alternatively, for a barely inbred population, we expect a negative relationship 
between LD and distance:

```{r strong LD}
strong_ld_pop <- simulate_admixture(
                  module = ancestry_module(number_of_founders = 4,
                                           morgan = 1),
                  pop_size = 100,
                  num_threads = 1,
                  total_runtime = 10)

ld_results <- calculate_ld(strong_ld_pop,
                           sampled_individuals = 10,
                           markers = 10)

plot(ld_results$ld_matrix~ld_results$dist_matrix,
     pch = 16,
     xlab = "Distance",
     ylab = "LD",
     xlim = c(0, 1),
     ylim = c(0, 1))
```

# Selection
Selection for specific markers (e.g. SNPs) can potentially cause local allelel 
frequency biases, and influence genetic patterns within the population as a 
whole.

The user can provide a selection 'matrix' that specifies how selection acts upon 
different genotypes. As genotypes we indicate here 'aa' (wildtype), 
'aA' (heterozygote) and 'AA' (homozygote with allele under selection), where 
wildtype is any allele not under selection. As alleles under selection we take 
the general genomic content of ancestors, and hence alleles refer to anestors. 

A perhaps interesting simulation would be one where the heterozygote has an 
advantage over the other genotypes
```{r heterozygote selection}
s <- 0.1
selection_matrix <- matrix(nrow = 1, ncol = 5)
selection_matrix[1, ] <- c(0.5,
                           1.0, 1.0 + s, 1.0,
                           0)

markers <- 0.5

selected_pop <- simulate_admixture(
                    module = ancestry_module(number_of_founders = 2,
                                             morgan = 1,
                                             markers = markers),
                    pop_size = 1000,  
                    total_runtime = 100,
                    num_threads = 1,
                    select_matrix = selection_matrix)

plot_over_time(selected_pop$frequencies, markers)
```

Indeed we observe that the frequencies of both ancestors are in equilibrium around 0.5, as expected.

Another interesting simulation would be where we give only the homozygous mutant a selective benefit. Here we expect that it takes some time before the homozygote takes over.

```{r homozygote selection}
s <- 0.1
selection_matrix[1, ] <- c(0.5,
                           1.0, 1.0, 1.0 + s,
                           0)

selected_pop <- simulate_admixture(
                    module = ancestry_module(number_of_founders = 10,
                                             morgan = 1,
                                             markers = markers),
                    pop_size = 1000,  
                    total_runtime = 300,
                    num_threads = 1,
                    select_matrix = selection_matrix)

plot_over_time(selected_pop$frequencies, markers)
```

---
title: "Demonstrate joyplot"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Demonstrate joyplot}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )
```

```{r}
library(GenomeAdmixR)
library(ggplot2)
packageVersion("GenomeAdmixR")

```

# Demonstration of how to use joyplots
To demonstrate the applicability and versatility of joyplots, we create a 
population, and select on this population on two separate locations, for two 
different ancestors.

```{r setup select_matrix}
select_matrix <- matrix(ncol = 5, nrow = 2)
s <- 0.1
select_matrix[1, ] <- c(0.23, 1.0, 1 + 0.5 * s, 1 + s, 0)
select_matrix[2, ] <- c(0.27, 1.0, 1 + 0.5 * s, 1 + s, 1)

markers <- seq(from = 0.2, to = 0.3, length.out = 100)
```
We track frequencies of all ancestors in the range [0.2, 0.3], using 100 
markers.

```{r simulate}
selected_pop <- simulate_admixture(
                    module = ancestry_module(number_of_founders = 2,
                                             morgan = 1,
                                             markers = markers),
                    pop_size = 100,
                    total_runtime = 1001,
                    num_threads = 1,
                    select_matrix = select_matrix)
```
We can now plot all the ancestors over time, we choose here to plot them only for a select subset of times.

```{r joyplot all}
time_points <- seq(from = 0, to = 1000, by = 100)
plot_joyplot_frequencies(selected_pop$frequencies,
                         time_points,
                         picked_ancestor = "ALL")
```

Alternatively, we can plot them separately.
```{r joyplot separate 0}
p1 <- plot_joyplot_frequencies(selected_pop$frequencies,
                               time_points,
                               picked_ancestor = 0)
p1 + ggplot2::geom_vline(xintercept = 0.23, lty = 2)
```
And for the other ancestor:
```{r joyplot separate 1}
p2 <- plot_joyplot_frequencies(selected_pop$frequencies,
                               time_points,
                               picked_ancestor = 1)
p2 + ggplot2::geom_vline(xintercept = 0.27, lty = 2)
```

---
title: "Empirical data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Empirical data}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
library(GenomeAdmixR)
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
print_mat <- function(mat) {
  n <- nrow(mat)
  c("\\begin{bmatrix}",
    paste0(sapply(seq_len(n - 1),
                  function(i) paste0(mat[i, ], collapse = " & ")),
           " \\\\"),
    paste0(mat[n, ], collapse = " & "),
    "\\end{bmatrix}")
}
RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )
```

## Empirical data

By default, the package uses local ancestry data, where each locus can be 
uniquely traced back to one of the founding populations of the admixed focal 
population. 
If, however, you are more interested in admixing sequence data, the GenomeAdmixR 
package provides this optionality as well, where you can import your own 
sequences and admix those in a similar fashion.

```{r creating fake data 2}
  fake_input_data <-
    create_artificial_genomeadmixr_data(number_of_individuals = 10,
                                        marker_locations = 1:10)

  fake_input_data$markers
  fake_input_data$genomes

```
Here, we create fake input data in absence of an example file. The data consists
of two components: a genome matrix and a vector with marker locations. The 
genome matrix has two rows per individual (one for each chromosome) and one 
column per molecular marker. The number of columns of the genome matrix should
match the length of the marker vector. Instead of using a/c/t/g, we use 
numerical notation, e.g. 1/2/3/4 (with missing data = 0).
Instead, you can also load sequence data from a VCF file (function 
\code{vcfR_to_genomeadmixr_data}) or from PLINK style data (function 
\code{ped_map_table_to_genomeadmixr_data}).
The loaded data can now be used as input for a simulation using the 
sequence_module:

```{r simulate admix data}
simulated_pop <- simulate_admixture(
                    module = sequence_module(molecular_data = fake_input_data),
                    pop_size = 100,
                    num_threads = 1,
                    total_runtime = 100)
```
If you want to study admixture, you might have VCF data with individuals from 
multiple populations. GenomeAdmixR provides functionality to use this data as
seeding data as well, although we do ask you to prepare two distinct VCF files,
where each VCF contains the individuals from each population.
Because \code{simulate_admixture} only accepts a single input matrix, we 
can artifically create an initial hybrid input swarm by loading data from 
multiple VCF files, and then randomly sampling individuals from these datasets
in order to form the starting dataset. We can do so using the function
combine_input_data:
```{r creating fake data}
  chosen_markers <- 1:100
   fake_input_data1 <-
    create_artificial_genomeadmixr_data(number_of_individuals = 100,
                                        marker_locations = chosen_markers,
                                        used_nucleotides = 1:2)
  fake_input_data2 <-
    create_artificial_genomeadmixr_data(number_of_individuals = 100,
                                        marker_locations = chosen_markers,
                                        used_nucleotides = 3:4)
  
  combined_data <- combine_input_data(input_data_list = list(fake_input_data1,
                                                             fake_input_data2),
                                      frequencies = c(0.5, 0.5),
                                      pop_size = 1000)
```
We can use this input data as a starting point for a simulation again:
```{r simulate}
simulated_pop <- simulate_admixture(
                      module = sequence_module(molecular_data = combined_data,
                                               markers = 1:100,
                                               morgan = 1),
                      pop_size = 1000,
                      num_threads = 1,
                      total_runtime = 100)
```
We have simulated here a population of 1000 individuals (initially randomly 
drawn from the sequences we created using combine_input_data), for 100 
generations. 
The result is identical in structure to results from 'simulate_admixture' and we
can use the same visualisation and analytical devices:

```{r visualise}
plot_over_time(simulated_pop$frequencies, focal_location = 50)
plot_joyplot_frequencies(simulated_pop$frequencies, time_points = c(0, 50, 100))
```

Furthermore, we can also perform selection on a certain allele, where the 
location under selection (first entry in the selection matrix) has to exist in
the original dataset.

```{r simulate selection}
selection_matrix <- matrix(NA, nrow = 1, ncol = 5)
selection_matrix[1, ] <- c(50, 1, 1.1, 1.2, 1)

# we expect allele 'a' to do well at 0.5 Morgan:
selected_pop <- simulate_admixture(
                      module = sequence_module(molecular_data = fake_input_data1,
                                               morgan = 1,
                                               markers = chosen_markers),
                      pop_size = 1000,
                      total_runtime = 100,
                      num_threads = 1,
                      select_matrix = selection_matrix)
plot_over_time(selected_pop$frequencies, focal_location = 50)
```

# Migration
Sequence data can also be used to seed a simulation where two populations 
exchange migrants, by specifying parameters for the migration argument, using the migration_settings function:
```{r migration}

migr_pop <-
  simulate_admixture(
        module = sequence_module(molecular_data = list(fake_input_data1,
                                                       fake_input_data2)),
        migration = migration_settings(migration_rate = 0.01,
                                       population_size = c(100, 100)),
        num_threads = 1,
        total_runtime = 100)
```

# Using simulation data as input
Lastly, not only "true" sequencing data can be used as input, but also previous
output from GenomeAdmixR simulations. In order to so, output from previous 
simulations can be converted to genomeadmixr_data, either using pre-existing
molecular markers as used in the simulation, or by superimposing markers on 
new locations.

```{r from_simulation}
simulated_pop <- simulate_admixture(
                      module = ancestry_module(), 
                      pop_size = 100,
                      num_threads = 1,
                      total_runtime = 100)
prepared_pop  <-
  simulation_data_to_genomeadmixr_data(simulation_data = simulated_pop,
                                       markers = seq(0, 1, length.out = 100))

simulated_pop2 <- simulate_admixture(
                        module = sequence_module(molecular_data = prepared_pop),
                        pop_size = 100,
                        num_threads = 1,
                        total_runtime = 100)
```

# Mutation rate
When using sequencing data, the simulations can also be extended using mutation.
One can do so by specifying the mutation rate (in probability per bp per 
generation), and by specifying the substitution matrix, where the substitution 
matrix is a 4x4 matrix, that specifies the probability of the different 
substitutions and transversions. For instance, for the JC69 model, 
the substitution matrix is given by:
```{r, results = 'asis'}
writeLines(print_mat(matrix(1 / 4, nrow = 4, ncol = 4)))
```
Please note that this is not necessarily the rate matrix! In the substitution 
matrix we use, all rows MUST sum to 1 (as these represent the probabilities 
of mutating into another base). 
For instance, for the K80 model with parameter kappa, we would get the following
substitution matrix, with kappa = 0.2:
```{r, results = 'asis'}
kappa <- 0.2
diag_entry <- 1 - (1 / 4 + 1 / 4 + 0.2 / 4)

writeLines(print_mat(matrix(c(diag_entry, kappa, 1 / 4, 1 / 4,
                              kappa, diag_entry, 1 / 4, 1 / 4,
                              1 / 4, 1 / 4, diag_entry, kappa,
                              1 / 4, 1 / 4, kappa, diag_entry),
                              nrow = 4, ncol = 4)))
```
We understand that it might be tricky to make these calculations a priori, 
therefore we automatically translate the substitution matrix to the probability 
matrix required in case the matrix is a rate matrix. Thus, summarising, the
substitution matrix provides the conditional probability of mutating to another
base, given that mutation occurs (as governed by the overall mutation rate).
Using artificial sequencing data, we can for instance show how new bases arise 
in the population due to mutation:
```{r example mutation}
  fake_input_data3 <-
  create_artificial_genomeadmixr_data(number_of_individuals = 100,
                                      marker_locations = 1:1000,
                                      used_nucleotides = 1)
  
  kappa <- 0.5
  rate_matrix <- matrix(c(0, kappa, 1, 1,
                          kappa, 0, 1, 1,
                          1, 1, 0, kappa,
                          1, 1, kappa, 0), nrow = 4, ncol = 4)
  
  mutation_rate <- 1e-5
  
  simulated_pop <- simulate_admixture(
                      module = sequence_module(molecular_data = fake_input_data3,
                                               morgan = 1,
                                               markers = chosen_markers,
                                               mutation_rate = mutation_rate,
                                               substitution_matrix = rate_matrix),
                      pop_size = 1000,
                      num_threads = 1,
                      total_runtime = 100)
  
  freqs <- calculate_allele_frequencies(simulated_pop)
  
  require(magrittr, quietly = TRUE)
  freqs %>%
    dplyr::group_by(ancestor) %>%
    dplyr::summarise(mean(frequency))
```
The final results show that indeed mutations have arisen in the population: 
the initial fake data only contained 'a' for all sequences, whereas at the end
of the simulation, we see that the other bases also exist in non-zero 
frequencies.

---
title: "Visualization"
author: "Thijs Janzen"
vignette: >
  %\VignetteIndexEntry{Visualization}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---
  
```{r setup, include=FALSE}
library(GenomeAdmixR)
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
```

# Visualization

The GenomeAdmixR package provides many different visualization options, here we 
will explore a variety of them.
First, we simulate a scenario with selection, to obtain somewhat meaningful 
results.

```{r simulate}
select_matrix <- matrix(nrow = 1, ncol = 5)
select_matrix[1, ] <- c(0.5, 1, 1 + 0.05, 1 + 0.1, 0)
population <- simulate_admixture(
                      module = ancestry_module(markers = 
                                                c(0.5, 
                                                 seq(0, 1, length.out = 100))),
                      pop_size = 1000,
                      num_threads = 1,
                      total_runtime = 200,
                      select_matrix = select_matrix)
```
Now, we can first view whether selection on our marker has yielded an increase 
in frequency:

## Plot over time
```{r plot over time}
plot_over_time(population$frequencies, focal_location = 0.500)
```
Indeed, we observe that over time the frequency of the allele under selection 
(0) increases to fixation due to selection. 

## Plot Frequencies
How are the alleles scattered across the genome? we can answer that with the 
function plot_frequencies:
```{r plot frequencies}
plot_frequencies(population, locations = seq(0, 1, length.out = 1000))
```
As expected, we observe a huge increase around the location of the marker under 
selection (at 0.5 Morgan).

## plot difference frequencies
If instead, we are interested in the change in frequency of a marker, we can do 
so using plot_difference_frequencies.

```{r plot difference frequencies}
plot_difference_frequencies(population)
```

## plot_start_end
Visualized in another way, plot_start_end plots the frequency distributions at 
the start and at the end of the simulation:
```{r plot start end}
plot_start_end(population)
```

## joyplots
If, indeed, we are more interested in the progression over time, we can also 
consult a so called 'joyplot', or 'ridgeplot':
```{r joyplot}
plot_joyplot_frequencies(population$frequencies,
                         time_points = c(0, 10, 25, 50, 100, 199))
```

## Plotting individual chromosomes
Lastly, individual chromosomes can be visualized using the standard plotting 
functions, where both the entire chromosome, and a fraction of the chromosome, 
can be visualized:

```{r plot chromosome}
plot(population$population[[1]])

plot_chromosome(population$population[[1]]$chromosome1,
                xmin = 0.45, xmax = 0.55)
```

---
title: "Demonstrate isofemales"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Demonstrate isofemales}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---
  
```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width = 6)
RcppParallel::setThreadOptions( getOption("Ncpus",  RcppParallel::defaultNumThreads()) )
```

# Creation of isofemale lines
The package GenomeAdmixR was designed to simulate the generation of iso-female 
lines, and simulate genomic changes during, and after, formation of iso-female 
lines. Furthermore, it allows for simulation of effects after crossing 
individuals from iso-female lines.

```{r load libraries}
library(GenomeAdmixR)
library(ggplot2)
packageVersion("GenomeAdmixR")
```
We will analyze whether we can genetically tell apart two populations, where one 
population was founded by crossing two isofemale lines that were created from 
the same source population, and the other population was founded by crossing 
two isofemale lines created from different source populations. 

First, we generate two source populations using \code{simulate_admixture} and 
then increase the founder labels in the second population to ensure that both 
populations do not share founder labels.

```{r create populations}
pops <-
  simulate_admixture(
        module = ancestry_module(),
        migration = migration_settings(migration_rate = 0,
                                         population_size = c(100, 100),
                                         initial_frequencies =
                                 list(c(1, 1, 1, 1, 0, 0, 0, 0),
                                      c(0, 0, 0, 0, 1, 1, 1, 1))),
     total_runtime = 1000,
     num_threads = 1)

pop_1 <- pops$population_1
pop_2 <- pops$population_2
```

To ensure that our two wild populations are not inbred, we estimate the average 
Linkage Disequilibrium (LD) for both populations using the function 
calculate_LD. In the absence of inbreeding, we expect the average LD to be low 
or zero.

```{r calculate LD}
mean(calculate_ld(pop = pop_1,
                  sampled_individuals = 10,
                  markers = 30)$ld_matrix,
     na.rm = TRUE)
mean(calculate_ld(pop = pop_2,
                  sampled_individuals = 10,
                  markers = 30)$ld_matrix,
     na.rm = TRUE)
```
Now we can start to generate isofemale lines from both populations, using \code{create_iso_female}.
```{r create isofemale lines}
iso_females_pop_1 <- create_iso_female(
                          module = ancestry_module(input_population = pop_1),
                          n = 2,
                          num_threads = 1,
                          inbreeding_pop_size = 100)
iso_females_pop_2 <- create_iso_female(
                          module = ancestry_module(input_population = pop_2),
                          n = 2,
                          num_threads = 1,
                          inbreeding_pop_size = 100)
```
Using the function \code{create_population_from_individuals} we then create
three populations, two where the isofemales are drawn from the same source 
population, and one where we mix the two, e.g. 1x1, 1x2 and 2x2, where 
isofemales 1 and 2 indicate isofemales from source populations 1 and 2 
respectively. 
```{r create from individuals}
pop_1_1 <- simulate_admixture(
               module = ancestry_module(input_population = iso_females_pop_1),
               pop_size = 1000,
               num_threads = 1,
               total_runtime = 1000)

pop_1_2 <- simulate_admixture(
                    module = ancestry_module(input_population =
                                               list(iso_females_pop_1[[1]],
                                                    iso_females_pop_2[[1]])),
                    pop_size = 1000,
                    num_threads = 1,
                    total_runtime = 1000)

pop_2_2 <- simulate_admixture(
                module = ancestry_module(input_population = iso_females_pop_2),
                num_threads = 1,
                pop_size = 1000,
                total_runtime = 1000)
```
Then, using the function \code{calculate_fst} we calculate all pairwise FST 
values. Here, we expect to find the highest FST for the 1x2 comparison. 
```{r FST calculation}
f1 <- calculate_fst(pop_1_1, pop_1_2,
                    sampled_individuals = 10, number_of_markers = 100)
# this one should be highest
f2 <- calculate_fst(pop_1_1, pop_2_2,
                    sampled_individuals = 10, number_of_markers = 100)
f3 <- calculate_fst(pop_1_2, pop_2_2,
                    sampled_individuals = 10, number_of_markers = 100)
f1
f2
f3
```
Which confirms our hypothesis and demonstrates how combining isofemale lines 
from different source populations generates a population with maximal genetic 
diversity. 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_load.R
\name{load_population}
\alias{load_population}
\title{Load a population from file}
\usage{
load_population(file_name)
}
\arguments{
\item{file_name}{Name of the file to save the population}
}
\value{
A population object
}
\description{
Loads a population that has previously been written to file.
}
\details{
This function is a wrapper for \code{readRDS}.
}
\seealso{
\code{\link{save_population}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{dgrp2.3R.5k.data}
\alias{dgrp2.3R.5k.data}
\title{A subset of sequencing data from the Drosophila Genetics Reference Panel}
\format{
genomeadmixr_data object
}
\usage{
data("dgrp2.3R.5k.data")
}
\description{
This data set contains sequences from the 3R chromosome.
Included are 4603 SNPs with at least 0.05 minor allele frequency,  sequenced
across 410 isofemale lines. Sequences were downloaded from
<http://dgrp2.gnets.ncsu.edu/data.html>.
}
\examples{
data("dgrp2.3R.5k.data")
simulate_admixture(
       module = sequence_module(molecular_data = dgrp2.3R.5k.data),
       pop_size = 100,
       total_runtime = 10)
}
\references{
Mackay, T., Richards, S., Stone, E. et al. The Drosophila
melanogaster Genetic Reference Panel. Nature 482, 173–178 (2012).
<https://doi.org/10.1038/nature10811>
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_input_data.R
\name{plink_to_genomeadmixr_data}
\alias{plink_to_genomeadmixr_data}
\title{function to convert plink style (ped/map) data to genome_admixr_data}
\usage{
plink_to_genomeadmixr_data(
  ped_data,
  map_data,
  chosen_chromosome,
  verbose = FALSE
)
}
\arguments{
\item{ped_data}{result of read.table(ped_file, header = F)}

\item{map_data}{result of read.table(map_file, header = F)}

\item{chosen_chromosome}{chromosome of choice}

\item{verbose}{verbose output}
}
\value{
genomeadmixr_data object ready for simulate_admixture_data
}
\description{
function to convert plink style (ped/map) data to genome_admixr_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_ld.R
\name{calculate_average_ld}
\alias{calculate_average_ld}
\title{Calculates the  ld between two alleles}
\usage{
calculate_average_ld(alleles_pos_1, alleles_pos_2)
}
\arguments{
\item{alleles_pos_1}{alleles at locus 1}

\item{alleles_pos_2}{alleles at locus 2}
}
\value{
a list with two entries: LD and r_squared
}
\description{
calculate the average ld between two loci
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/migration_settings.R
\name{migration_settings}
\alias{migration_settings}
\title{Function to manage settings associated with migration}
\usage{
migration_settings(
  migration_rate = NA,
  stop_at_critical_fst = FALSE,
  critical_fst = NA,
  population_size = c(100, 100),
  initial_frequencies = list(c(1, 0), c(0, 1)),
  generations_between_update = 10,
  sampled_individuals = 10,
  number_of_markers = 100,
  random_markers = TRUE
)
}
\arguments{
\item{migration_rate}{Rate of migration between the two populations.
Migration is implemented such that with probability m (migration rate) one
of the two parents of a new offspring is from the other population, with
probability 1-m both parents are of the focal population.}

\item{stop_at_critical_fst}{option to stop at a critical FST value
, default is FALSE}

\item{critical_fst}{the critical fst value to stop, if
\code{stop_simulation_at_critical_fst} is TRUE}

\item{population_size}{vector of population sizes, one size for each
population}

\item{initial_frequencies}{A list describing the initial frequency of each
ancestor in each population. Each entry in the list contains a vector with
the frequencies for all ancestor. The length of the vector indicates the
number of unique ancestors. If a vector not summing to 1 is provided, the
vector is normalized.}

\item{generations_between_update}{The number of generations after which the
simulation has to check again whether the critical Fst value is exceeded}

\item{sampled_individuals}{Number of individuals to be sampled at random from
the population to estimate Fst}

\item{number_of_markers}{Number of markers to be used to estimate Fst}

\item{random_markers}{Are the markers to estimate Fst randomly distributed,
or regularly distributed? Default is TRUE.}
}
\value{
list with migration associated settings. To be used to pass on
migration settings to simulate_admixture.
}
\description{
creates a list with settings associated with migration.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/default_params_doc.R
\name{default_params_doc}
\alias{default_params_doc}
\title{Default parameter documentation}
\usage{
default_params_doc(
  module,
  total_runtime,
  migration,
  select_matrix,
  pop_size,
  multiplicative_selection,
  verbose,
  num_threads,
  morgan,
  markers,
  track_junctions
)
}
\arguments{
\item{module}{Chosen module to simulate, either created with
\code{module_ancestry} or
\code{module_sequence}.}

\item{total_runtime}{Number of generations}

\item{migration}{settings associated with migration, should be created with
\code{\link{migration_settings}}}

\item{select_matrix}{Selection matrix indicating the markers which are under
selection. If not provided by the user, the simulation proceeds neutrally. If
provided, each row in the matrix should contain five entries:
\itemize{
\item{location of the marker under selection (in Morgan) }
\item{fitness of wildtype (aa)}
\item{fitness of heterozygote (aA)}
\item{fitness of homozygote mutant (AA)}
\item{Ancestral type that represents the mutant allele A}
}}

\item{pop_size}{The number of individuals in the population. If the number is
larger than the number of individuals in the input population (if provided),
additional individuals are sampled randomly from the input population to
reach the intended size.}

\item{multiplicative_selection}{Default: TRUE. If TRUE, fitness is calculated
for multiple markers by multiplying fitness values for each marker. If FALSE,
fitness is calculated by adding fitness values for each marker.}

\item{verbose}{Verbose output if TRUE. Default value is FALSE}

\item{num_threads}{number of threads. Default is 1. Set to -1 to use all
available threads}

\item{morgan}{Length of the chromosome in Morgan (e.g. the number of
crossovers during meiosis)}

\item{markers}{A vector of locations of markers (relative locations in
[0, 1]). If a vector is provided, ancestry at these marker positions is
tracked for every generation.}

\item{track_junctions}{Track the average number of junctions over time if
TRUE}
}
\value{
Nothing
}
\description{
This function's purpose is to list all parameter documentation to be
inherited by the relevant functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_input_data.R
\name{create_artificial_genomeadmixr_data}
\alias{create_artificial_genomeadmixr_data}
\title{function to generate artificial genomeadmixr_data}
\usage{
create_artificial_genomeadmixr_data(
  number_of_individuals,
  marker_locations,
  used_nucleotides = 1:4,
  nucleotide_frequencies = NA
)
}
\arguments{
\item{number_of_individuals}{number of individuals}

\item{marker_locations}{location of markers, either in bp or Morgan}

\item{used_nucleotides}{subset or full set of [1/2/3/4] (reflecting a/c/t/g)}

\item{nucleotide_frequencies}{frequencies of the used nucleotides, if not
provided, equal frequencies are assumed.}
}
\value{
genomeadmixr_data object ready for simulate_admixture_data
}
\description{
function to generate artificial genomeadmixr_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/junctions.R
\name{calculate_dist_junctions}
\alias{calculate_dist_junctions}
\title{collect the full distribution of junctions in the population}
\usage{
calculate_dist_junctions(pop)
}
\arguments{
\item{pop}{object of the class 'population'}
}
\value{
vector with two entries per individual, each indicating the number of
junctions in the respective chromosomes
}
\description{
calculates the distribution of junctions across the population
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_allele_frequencies.R
\name{calculate_allele_frequencies}
\alias{calculate_allele_frequencies}
\title{Calculate allele frequencies}
\usage{
calculate_allele_frequencies(
  source_pop,
  locations = seq(0, 1, length.out = 100),
  progress_bar = FALSE
)
}
\arguments{
\item{source_pop}{Population for which to estimate allele frequencies}

\item{locations}{A vector indicating the locations (in Morgan) where
to calculate the allele frequencies.}

\item{progress_bar}{Displays a progress_bar if TRUE. Default value is TRUE}
}
\value{
A tibble containing the allele frequencies
}
\description{
Calculate for a number of regularly spaced markers the relative
frequency of each ancestor in the population.
}
\details{
Markers are equidistantly spaced, with a distance of
\code{step_size} in between them.
}
\examples{
number_founders = 20
wildpop =  simulate_admixture(
   module = ancestry_module(number_of_founders = 20, morgan = 1),
   pop_size = 1000,
   total_runtime = 10,
   num_threads = 1)

freq_output <- calculate_allele_frequencies(wildpop,
                                            progress_bar = TRUE)

require(ggplot2)
ggplot(freq_output, aes(x=location, y = frequency,
                        col = as.factor(ancestor))) +
  geom_line()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{plot.individual}
\alias{plot.individual}
\title{plot the genome of an individual}
\usage{
\method{plot}{individual}(x, cols = NA, ...)
}
\arguments{
\item{x}{object of type individual}

\item{cols}{colors for the different ancestors}

\item{...}{other arguments}
}
\value{
No return value
}
\description{
visualise ancestry blocks on both chromosomes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_heterozygosity.R
\name{calculate_heterozygosity}
\alias{calculate_heterozygosity}
\title{Calculate heterozygosity}
\usage{
calculate_heterozygosity(source_pop, locations, progress_bar = FALSE)
}
\arguments{
\item{source_pop}{Population for which to estimate allele frequencies, or a
list of individuals for which to calculate average heterozygosity}

\item{locations}{A vector indicating the locations (in Morgan) of markers for
which to calculate the heterozygosity}

\item{progress_bar}{Displays a progress_bar if TRUE. Default value is TRUE}
}
\value{
A tibble containing the heterozygosities
}
\description{
Calculate the average population level heterozygosity
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_start_end.R
\name{plot_start_end}
\alias{plot_start_end}
\title{Plot both the starting frequencies and the final frequencies in one plot}
\usage{
plot_start_end(results, picked_ancestor = "ALL", picked_population = 1)
}
\arguments{
\item{results}{An object which is the result of \code{simulate_admixture},
being a list with four properties:
\code{population}, \code{frequencies}, \code{initial_frequencies} and
\code{final frequencies}}

\item{picked_ancestor}{Default is "ALL", where different colors indicate
different ancestors. Alternatively, for clarity, the user can specify a
specific ancestral allele, and only that allele is plotted}

\item{picked_population}{If multiple populations were simulated (in the case
of \code{simulate_admixture_migration}), which population should be plotted?
Default is population_1.}
}
\value{
a ggplot object
}
\description{
This function plots the distribution of both the starting and
the final frequencies in one plot
}
\examples{
markers <- seq(from = 0.2, to = 0.3, length.out = 100)

pop <- simulate_admixture(
            module = ancestry_module(number_of_founders = 3,
                                     morgan = 1,
                                     markers = markers),
           pop_size = 1000,
           total_runtime = 11)
require(ggplot2)
plot_start_end(pop,
               picked_ancestor = "ALL")
plot_start_end(pop,
               picked_ancestor = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_fst.R
\name{calculate_fst}
\alias{calculate_fst}
\title{Calculate FST}
\usage{
calculate_fst(
  pop1,
  pop2,
  sampled_individuals = 10,
  number_of_markers = 100,
  random_markers = FALSE
)
}
\arguments{
\item{pop1}{Population object}

\item{pop2}{Population object}

\item{sampled_individuals}{Number of individuals to base the FST upon.
Individuals are randomly drawn from each population, a lower number speeds
up calculations.}

\item{number_of_markers}{Number of markers along the chromosome used to
calculate FST metrics.}

\item{random_markers}{If TRUE, markers are randomly spaced along the
chromosome, if FALSE, markers are equidistantly spaced along the chromosome.}
}
\value{
FST value
}
\description{
The FST value between two populations is calculated, given a
number of markers. Markers are superimposed upon the (known) ancestry along
the chromosome for all sampled individuals. Markers can be chosen to be
regularly spaced, or randomly distributed.
}
\details{
Uses the function \code{wc} from the package \code{hierfstat} to
calculate the FST. The function \code{wc} computes the Weir and Cockerham
F statistic.
}
\examples{
two_populations <- simulate_admixture(
                        module = ancestry_module(),
                       migration = migration_settings(migration_rate = 0.01,
                                           population_size = c(100, 100)))

FST <- calculate_fst(pop1 = two_populations$population_1,
                     pop2 = two_populations$population_2,
                     sampled_individuals = 10,
                     number_of_markers = 100,
                     random_markers = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{print.individual}
\alias{print.individual}
\title{print an individual to the console}
\usage{
\method{print}{individual}(x, ...)
}
\arguments{
\item{x}{individual}

\item{...}{other arguments}
}
\value{
No return value
}
\description{
prints an object of class individual to the console
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_input_data.R
\name{combine_input_data}
\alias{combine_input_data}
\title{combine sequence data that was previously read from file into a population}
\usage{
combine_input_data(input_data_list, frequencies = NA, pop_size)
}
\arguments{
\item{input_data_list}{list where each entry is the result of
\code{create_input_data}}

\item{frequencies}{frequency of each entry in the list in the starting
population}

\item{pop_size}{intended population size}
}
\value{
the input data entries are combined to one single population that can
be used to seed \code{simulate_admixture_data}. Output is identical to
\code{create_input_data}
}
\description{
Create data in a format that can be used by GenomeAdmixR,
entries are sampled randomly from each input data set, with replacement.
Probability of sampling from each input data set is driven by the input
frequencies, and total number of individuals sampled is driven by pop_size.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_input_data.R
\name{vcfR_to_genomeadmixr_data}
\alias{vcfR_to_genomeadmixr_data}
\title{function to convert a vcfR object to genome_admixr_data}
\usage{
vcfR_to_genomeadmixr_data(
  vcfr_object,
  chosen_chromosome,
  number_of_snps = NA,
  random_snps = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{vcfr_object}{result of vcfR::read.vcfR}

\item{chosen_chromosome}{chromosome of choice}

\item{number_of_snps}{number of snps to be loaded from the vcf file, default
is to load all snps}

\item{random_snps}{if a subset of all snps has to be taken, should these
be sampled sequentially (e.g. the first 100 snps) or randomly (100 randomly
sampled snps) (examples are for 'number_of_snps' = 100).}

\item{verbose}{if true, print progress bar}
}
\value{
genomeadmixr_data object ready for simulate_admixture_data
}
\description{
function to convert a vcfR object to genome_admixr_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_ld.R
\name{calculate_ld}
\alias{calculate_ld}
\title{Calculate linkage disequilibrium statistics
This function calculates two matrices, once containing all pairwise
linkage disequilibrium (ld) values, and one matrix containing all pairwise r
statistics}
\usage{
calculate_ld(pop, sampled_individuals = 10, markers = NA, verbose = FALSE)
}
\arguments{
\item{pop}{focal population}

\item{sampled_individuals}{Number of individuals randomly sampled to
calculate the LD matrices}

\item{markers}{vector of markers. If a single number is used, that number of
markers is randomly placed along the genome.}

\item{verbose}{display verbose output, default is FALSE.}
}
\value{
An object containing two items:
\item{ld_matrix}{
  Pairwise ld statistics for all markers
}
\item{rsq_matrix}{
  Pairwise rsq statistics for all markers
}
}
\description{
Calculate linkage disequilibrium statistics
This function calculates two matrices, once containing all pairwise
linkage disequilibrium (ld) values, and one matrix containing all pairwise r
statistics
}
\examples{
wildpop =  simulate_admixture(
   module = ancestry_module(number_of_founders = 10, morgan = 1),
   pop_size = 1000,
   total_runtime = 10)

ld_results <- calculate_ld(pop = wildpop,
                           markers = 10)

plot(ld_results$ld_matrix~ld_results$dist_matrix,
     pch = 16,
     xlab="Distance between markers",
     ylab = "Linkage Disequilibrium")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_ancestry_migration.R
\name{simulate_ancestry_migration}
\alias{simulate_ancestry_migration}
\title{Individual based simulation of the breakdown of contiguous ancestry blocks in
two populations linked by migration}
\usage{
simulate_ancestry_migration(
  input_population_1 = NA,
  input_population_2 = NA,
  pop_size = c(100, 100),
  initial_frequencies = list(c(1, 0), c(0, 1)),
  total_runtime = 100,
  morgan = 1,
  num_threads = 1,
  select_matrix = NA,
  markers = NA,
  verbose = FALSE,
  track_junctions = FALSE,
  multiplicative_selection = TRUE,
  migration_rate = 0,
  stop_at_critical_fst = FALSE,
  critical_fst = 0.1,
  generations_between_update = 100,
  sampled_individuals = 10,
  number_of_markers = 100,
  random_markers = TRUE
)
}
\arguments{
\item{input_population_1}{Potential earlier simulated population used as
starting point for the simulation. If not provided by the user, the
simulation starts from scratch.}

\item{input_population_2}{Potential earlier simulated population used as
starting point for the simulation. If not provided by the user,
the simulation starts from scratch.}

\item{pop_size}{Vector containing the number of individuals in both
populations.}

\item{initial_frequencies}{A list describing the initial frequency of each
ancestor in each population. Each entry in the list contains a vector with
the frequencies for all ancestor. The length of the vector indicates the
number of unique ancestors. If a vector not summing to 1 is provided, the
vector is normalized.}

\item{total_runtime}{Number of generations}

\item{morgan}{Length of the chromosome in Morgan (e.g. the number of
crossovers during meiosis)}

\item{num_threads}{number of threads. Default is 1. Set to -1 to use all
available threads}

\item{select_matrix}{Selection matrix indicating the markers which are under
selection. If not provided by the user, the simulation proceeds neutrally. If
provided, each row in the matrix should contain five entries:
\itemize{
\item{location of the marker under selection (in Morgan) }
\item{fitness of wildtype (aa)}
\item{fitness of heterozygote (aA)}
\item{fitness of homozygote mutant (AA)}
\item{Ancestral type that represents the mutant allele A}
}}

\item{markers}{A vector of locations of markers (relative locations in
[0, 1]). If a vector is provided, ancestry at these marker positions is
tracked for every generation.}

\item{verbose}{Verbose output if TRUE. Default value is FALSE}

\item{track_junctions}{Track the average number of junctions over time if
TRUE}

\item{multiplicative_selection}{Default: TRUE. If TRUE, fitness is calculated
for multiple markers by multiplying fitness values for each marker. If FALSE,
fitness is calculated by adding fitness values for each marker.}

\item{migration_rate}{Rate of migration between the two populations.
Migration is implemented such that with probability m (migration rate) one
of the two parents of a new offspring is from the other population, with
probability 1-m both parents are of the focal population.}

\item{stop_at_critical_fst}{option to stop at a critical FST value
, default is FALSE}

\item{critical_fst}{the critical fst value to stop, if
\code{stop_simulation_at_critical_fst} is TRUE}

\item{generations_between_update}{The number of generations after which the
simulation has to check again whether the critical Fst value is exceeded}

\item{sampled_individuals}{Number of individuals to be sampled at random from
the population to estimate Fst}

\item{number_of_markers}{Number of markers to be used to estimate Fst}

\item{random_markers}{Are the markers to estimate Fst randomly distributed,
or regularly distributed? Default is TRUE.}
}
\value{
A list with: \code{population_1}, \code{population_2} two population
objects, and three tibbles with allele frequencies (only contain values of a
vector was provided to the argument \code{markers}: \code{frequencies},
\code{initial_frequencies} and \code{final_frequencies}. Each tibble contains
five columns, \code{time}, \code{location}, \code{ancestor}, \code{frequency}
and \code{population}, which indicates the number of generations, the
location along the chromosome of the marker, the ancestral allele at that
location in that generation, the frequency of that allele and the population
in which it was recorded (1 or 2). If a critical fst value was used to
terminate the simulation, and object \code{FST} with the final FST estimate
is returned as well.
}
\description{
Individual based simulation of the breakdown of contiguous
ancestry blocks, with or without selection. Simulations can be started from
scratch, or from a predefined input population. Two populations are
simulated, connected by migration
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_input_data.R
\name{simulation_data_to_genomeadmixr_data}
\alias{simulation_data_to_genomeadmixr_data}
\title{function to convert ped/map data to genome_admixr_data}
\usage{
simulation_data_to_genomeadmixr_data(
  simulation_data,
  markers = NA,
  verbose = FALSE
)
}
\arguments{
\item{simulation_data}{result of simulate_admixture}

\item{markers}{vector of locations of markers (in Morgan). If no vector is
provided, the function searches for marker locations in the simulation_data.}

\item{verbose}{provide verbose output (default is FALSE)}
}
\value{
genomeadmixr_data object ready for simulate_admixture_data
}
\description{
function to convert ped/map data to genome_admixr_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sequence_module.R
\name{sequence_module}
\alias{sequence_module}
\title{create sequence module}
\usage{
sequence_module(
  molecular_data = NA,
  initial_frequencies = NA,
  morgan = 1,
  recombination_rate = NA,
  markers = NA,
  mutation_rate = 0,
  substitution_matrix = matrix(1/4, 4, 4)
)
}
\arguments{
\item{molecular_data}{Genomic data used as input, should be of type
genomeadmixr_data. Either a single dataset is provided, or a list of
multiple genomeadmixr_data objects.}

\item{initial_frequencies}{A vector describing the initial contribution of
each provided input data set to the starting hybrid swarm. By default, equal
frequencies are assumed. If a vector not summing to 1 is provided, the vector
is normalized.}

\item{morgan}{Length of the molecular sequence in Morgan (e.g. the number of
crossovers during meiosis), alternatively, the recombination rate can be
used, see below.}

\item{recombination_rate}{rate in cM / Mbp, used to map recombination to the
markers. If the recombination_rate is not set, the value for Morgan is used,
assuming that the markers included span an entire chromosome.}

\item{markers}{A vector of locations of markers, these markers are
tracked for every generation.}

\item{mutation_rate}{the per base probability of mutation. Default is 0.}

\item{substitution_matrix}{a 4x4 matrix representing the probability of
mutating to another base (where [1/2/3/4] = [a/c/t/g]), conditional on the
event of a mutation happening. Default is the JC69 matrix, with equal
probabilities for all transitions / transversions.}
}
\value{
sequence module object, used as starting point for
simulate_admixture.
}
\description{
creates a sequence module, which contains all relevant
information in order to perform a simulation based on sequence data.
}

\name{GenomeAdmixR-package}
\alias{GenomeAdmixR-package}
\alias{GenomeAdmixR}
\docType{package}
\title{
 Simulate Admixture of Genomes
}
\description{
   Individual-based simulations forward in time, simulating how
patterns in ancestry along the genome change after admixture. The simulation
assumes Wright-Fisher dynamics, e.g. random mating and non-overlapping generations. In the simulation, instead of specific alleles, local ancestry is tracked, thus assuming that local molecular data can always be uniquely traced back to one of the founding individuals (populations). The package provides functionality to perform such simulations, but also to perform post-hoc statistical analyses and to visualize the obtained results.
   Version 2.1.13 - Added support for machines without TBB \cr
   Version 2.1.12 -Update to C++17 \cr
   Version 2.1.11 - Replaced Rf_error with Rcpp::stop() \cr
   Version 2.1.10 - Fixed memory bug, improved documentation \cr
   Version 2.1.9  - updated tbb::task_scheduler_init to tbb::global_control \cr
   Version 2.1.7  - Improve documentation \cr
   Version 2.1.6  - check classes with inherits \cr
   Version 2.1.5  - Removed debugging output \cr
   Version 2.1.4  - Only output when verbose = TRUE \cr
   Version 2.1.3  - Changed DOI link in description \cr
   Version 2.1.2  - Improved testing \cr
   Version 2.1.1  - Removed GNU make dependency \cr
   Version 2.1    - Removed error in calculate_allele_frequency \cr
   Version 2.0.1  - Moved migration outside the modules \cr
   Version 2.0    - Added ancestry_module and sequence_module to distinguish between
                  implementations of the model \cr
   Version 1.2    - Added example sequencing data \cr
   Version 1.2    - Added the option to load sequence data for admixing \cr
   Version 1.1    - Fixed a minor bug with plot_joyplot_frequencies \cr
   Version 1.1    - Improved tests \cr
   Version 1.1    - Improved recombination code (again) \cr
   Version 1.0    - Release associated with bioRxiv submission, to be found here: https://doi.org/10.1101/2020.10.19.343491 \cr
   Version 0.66   - Improved recombination code, about twice as fast \cr
   Version 0.65   - Added testing and added logo \cr
   Version 0.64   - Reduced cyclomatic complexity \cr
   Version 0.63   - Updated random number generation \cr
   Version 0.62   - Updated to Roxygen \cr
   Version 0.61   - Added plot_over_time \cr
   Version 0.60   - Added admixture with migration \cr
   Version 0.59   - Updated frequency code under the hood \cr
   Version 0.58   - Renamed to GenomeAdmixR \cr
   Version 0.58   - Collapsed and improved many functions \cr
   Version 0.57   - Added function to generate admixed individuals \cr
   Version 0.56   - Added starting frequencies to 'simulate_admixture' \cr
   Version 0.55   - Extended 'calculate_marker_frequency' to handle a vector of locations \cr
   Version 0.55   - Increased accuracy of choosing a random position for recombination, this should prevent the rare bug fixed in version 0.54 \cr
   Version 0.54   - Fixed a MAJOR bug regarding recombination: in rare cases, a crossover position could be picked on an existing junction, due to the limited number of digits in uniform() \cr
   Version 0.54   - Improved plot_difference_frequencies to handle modified input \cr
   Version 0.53   - Added multiplicative_selection \cr
   Version 0.52   - Added plot_difference_frequencies \cr
   Version 0.51   - Added tajima's d calculation \cr
   Version 0.50   - Added simulated_admixture until \cr
   Version 0.49   - Added 'simulate' to cpp \cr
   Version 0.48   - Added a general 'simulate' function \cr
   Version 0.47   - Changed the effect of migration \cr
   Version 0.46   - Added joyplot & increase_ancestor \cr
   Version 0.45   - Removed create_two_populations \cr
   Version 0.44   - Added tracking regions \cr
   Version 0.43   - Fixed bugs in select_population \cr
   Version 0.42   - Added initial and final frequency tables \cr
   Version 0.41   - Added multiple marker support \cr
   Version 0.40   - Collapsed selection functions \cr
   Version 0.39   - Added support for non-additive selection \cr
   Version 0.38   - Added track frequencies \cr
   Version 0.37   - Removed selection on regions \cr
   Version 0.36   - Added progress_bar option \cr
   Version 0.35   - Added calculate_marker_frequency \cr
   Version 0.34   - Added selection_markers \cr
   Version 0.33   - Fixed bugs in selection \cr
   Version 0.32   - Moved Fish.h code to Fish.cpp \cr
   Version 0.31   - Changed random number generator to R based \cr
   Version 0.30   - Added Recombination = 1 code \cr
   Version 0.29   - Changed internal junction representation: removed .left \cr
   Version 0.28   - Reverted to Agner Fog Random number generation \cr
   Version 0.27   - Speed up return types \cr
   Version 0.26   - Added class verification code \cr
   Version 0.25   - Squashed plotting bug \cr
   Version 0.24   - Removed Output.cpp \cr
   Version 0.23   - Removed number_of_founders from calc_allele_spectrum \cr
   Version 0.22   - Added save and load functions \cr
   Version 0.21   - Changed random-seed management \cr
   Version 0.20   - Removed superfluous code \cr
   Version 0.19   - Removed number_of_founders from Fst and LD code \cr
   Version 0.18   - Start of tracking changes \cr
}
\author{
Thijs Janzen
Maintainer: (thijsjanzen@gmail.com)
}
\references{
Janzen T, Diaz F. Individual-based simulations of genome evolution with ancestry: The GenomeAdmixR R package. Methods Ecol Evol. 2021; 12: 1346–1357. https://doi.org/10.1111/2041-210X.13612
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_chromosome.R
\name{plot_chromosome}
\alias{plot_chromosome}
\title{plots a chromosome}
\usage{
plot_chromosome(chrom, xmin = 0, xmax = 1)
}
\arguments{
\item{chrom}{object of type chromosome, typically a table with two columns.
The first column indicates the start of an ancestry block (location in
Morgan), the second column indicates the ancestry type.}

\item{xmin}{minimum value of the range, default = 0.}

\item{xmax}{maximum value of the range, default = 1.}
}
\value{
No return value
}
\description{
This function plots a chromosome in the range [xmin, xmax].
Colors indicate different ancestry.
}
\examples{
wildpop =  simulate_admixture(
   module = ancestry_module(number_of_founders = 10, morgan = 1),
   pop_size = 1000,
   total_runtime = 10)

isofemale <- create_iso_female(
                 module = ancestry_module(input_population = wildpop,
                                          morgan = 1),
                 n = 1,
                 inbreeding_pop_size = 100,
                 run_time = 10)

plot_chromosome(chrom = isofemale[[1]]$chromosome1)
# and a detail of the chromosome:
plot_chromosome(chrom = isofemale[[1]]$chromosome1,
                xmin = 0.4,
                xmax = 0.6)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_iso_female.R
\name{create_iso_female}
\alias{create_iso_female}
\title{function to simulate creation of an isofemale line}
\usage{
create_iso_female(
  module = ancestry_module(),
  n = 1,
  inbreeding_pop_size = 100,
  run_time = 2000,
  num_threads = 1,
  verbose = FALSE
)
}
\arguments{
\item{module}{Source population from which isofemales are generated}

\item{n}{Number of isofemales to be generated}

\item{inbreeding_pop_size}{Population size of the population used to generate
homozygous individuals}

\item{run_time}{Maximum runtime used for inbreeding}

\item{num_threads}{number of threads. Default is 1. Set to -1 to use all
available threads}

\item{verbose}{Displays verbose output if TRUE. Default value is FALSE}
}
\value{
A list of length \code{n}, where each entry is a fully homozygous
isofemale.
}
\description{
create_isofemale simulates the creation of an isofemale line
through extreme inbreeding.
}
\details{
To create an isofemale, two individuals are randomly picked from
the source population. Using these two individuals, a new population is
seeded, of size \code{inbreeding_pop_size}. Then, this population is allowed
to inbreed until either \code{run_time} is reached, or until all individuals
are homozygous and genetically identical, whatever happens first.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_marker_frequency.R
\name{calculate_marker_frequency}
\alias{calculate_marker_frequency}
\title{Calculate allele frequencies at a specific marker location}
\usage{
calculate_marker_frequency(pop, location)
}
\arguments{
\item{pop}{Population for which to estimate allele frequencies at the
given marker}

\item{location}{A vector or scalar of location(s) along the chromosome for
which allele frequencies are to be calculated. Locations are in Morgan.}
}
\value{
A tibble containing the frequency of each present ancestor at the
provided location. Ancestors with frequency = 0 are dropped out of the table.
The tibble contains three columns: location, ancestor and frequency.
}
\description{
Calculate the relative frequency of each ancestor in the
population at a specific marker location
}
\examples{
wildpop =  simulate_admixture(
   module = ancestry_module(number_of_founders = 20, morgan = 1),
   pop_size = 1000,
   total_runtime = 10)

avg_frequencies <- calculate_marker_frequency(pop = wildpop,
                                              location = 0.5)

frequencies <-
   calculate_marker_frequency(pop = wildpop,
                              location = seq(0.4, 0.5, by = 0.01))
require(ggplot2)
ggplot(frequencies, aes(x = location, y = frequency, col = ancestor)) +
  geom_step()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_frequencies.R
\name{plot_frequencies}
\alias{plot_frequencies}
\title{Plot the frequencies of all ancestors along the genome.}
\usage{
plot_frequencies(
  result,
  locations = seq(0, 1, length.out = 100),
  progress_bar = FALSE
)
}
\arguments{
\item{result}{An object which is the result of \code{select_population} or
\code{create_population_selection}, being a list with four properties:
\code{population}, \code{frequencies}, \code{initial_frequencies} and
\code{final frequencies}}

\item{locations}{A vector indicating the locations (in Morgan) where to
calculate the allele frequencies.}

\item{progress_bar}{Displays a progress_bar if TRUE. Default value is FALSE}
}
\value{
a ggplot2 object
}
\description{
This function plots the frequency of all ancestors after
performing a simulation.
}
\examples{
pop <- simulate_admixture(
             module = ancestry_module(number_of_founders = 4),
             pop_size = 1000,
             total_runtime = 11)
require(ggplot2)
plot_frequencies(result = pop)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_input_data.R
\name{read_input_data}
\alias{read_input_data}
\title{read sequence data from file to be used in simulation}
\usage{
read_input_data(
  file_names,
  type,
  chosen_chromosome,
  number_of_snps = NA,
  random_snps = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{file_names}{names of input files}

\item{type}{type of data, options are 'ped' and 'vcf'}

\item{chosen_chromosome}{GenomeAdmixR simulates only a single chromosome.}

\item{number_of_snps}{number of snps to be loaded from file, default
is to load all snps}

\item{random_snps}{if a subset of all snps has to be taken, should these
be sampled sequentially (e.g. the first 100 snps) or randomly (100 randomly
sampled snps) (examples are for 'number_of_snps' = 100).}

\item{verbose}{give verbose output}
}
\value{
list with two properties: \code{genomes} a matrix with the
sequence translated to numerics, such that [actg] corresponds to [1234], and
missing data is represented with "-". Rows in the matrix correspond to
chromosomes, and columns represent bases. Two consecutive rows represent an
individual, such that rows 1-2 are individual, rows 3-4 are one individual
etc. \code{markers} corresponds to the locations of the markers (in bp) on
the chosen chromosome.
}
\description{
Create data in a format that can be used by GenomeAdmixR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/joyplot_frequencies.R
\name{plot_joyplot_frequencies}
\alias{plot_joyplot_frequencies}
\title{make a joy plot of the distribution of allele frequencies within a region}
\usage{
plot_joyplot_frequencies(
  frequencies,
  time_points,
  picked_ancestor = "ALL",
  picked_population = 1
)
}
\arguments{
\item{frequencies}{A tibble containing four columns: \code{time},
\code{location}, \code{ancestor}, \code{frequency}. Typically one of the
items returned by \code{create_population_selection} or
\code{select_population} when the user specifies \code{track_frequency}.}

\item{time_points}{A sequence of time points for which the user wants to
create the joyplot}

\item{picked_ancestor}{Default is "ALL", where different colors indicate
different ancestors. Alternatively, for clarity, the user can specify a
specific ancestral allele, and only that allele is plotted}

\item{picked_population}{If multiple populations were simulated (in the case
of \code{simulate_admixture_migration}), which population should be plotted?
Default is population_1.}
}
\value{
a ggplot object
}
\description{
This function plots the distribution of allele frequencies
within a region over time, making use of a 'joyplot'
}
\examples{
\donttest{
s <- 0.01
select_matrix <- matrix(nrow = 1, ncol = 5)
select_matrix[1, ] <- c(0.25, 1.0, 1 + 0.5 * s, 1 + s, 0)

markers <- seq(from = 0.2, to = 0.3, length.out = 100)

selected_pop <- simulate_admixture(
                    module = ancestry_module(number_of_founders = 10,
                                             morgan = 1,
                                             markers = markers),
                    pop_size = 1000,
                    total_runtime = 11,
                    select_matrix = select_matrix)
require(ggplot2)
plot_joyplot_frequencies(frequencies = selected_pop$frequencies,
                         time_points = 0:11,
                         picked_ancestor = "ALL")

# joyplot frequencies returns a ggplot object, so we can
# add extra elements:
plot_joyplot_frequencies(frequencies = selected_pop$frequencies,
                         time_points = 0:11,
                         picked_ancestor = "ALL") +
  ggplot2::xlab("Location") +
  ggplot2::ylab("Generations")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_difference_frequencies.R
\name{plot_difference_frequencies}
\alias{plot_difference_frequencies}
\title{Plot the change in frequency between the start and end of a simulation}
\usage{
plot_difference_frequencies(
  results,
  picked_ancestor = "ALL",
  picked_population = 1
)
}
\arguments{
\item{results}{An object which is the result of \code{simulate_admixture}
 being a list with four properties:
\code{population}, \code{frequencies}, \code{initial_frequencies} and
\code{final frequencies}}

\item{picked_ancestor}{Default is "ALL", where different colors indicate
different ancestors. Alternatively, for clarity, the user can specify a
specific ancestral allele, and only that allele is plotted}

\item{picked_population}{If multiple populations were simulated (in the case
of \code{simulate_admixture_migration}), which population should be plotted?
Default is population_1.}
}
\value{
a ggplot2 object
}
\description{
This function plots the change in frequency of one or
multiple ancestors after performing a simulation.
}
\examples{
s <- 0.1
select_matrix <- matrix(nrow = 1, ncol = 5)
select_matrix[1, ] <- c(0.25, 1.0, 1 + 0.5 * s, 1 + s, 0)

markers <- seq(from = 0.2, to = 0.3, length.out = 100)

selected_pop <- simulate_admixture(
                    module = ancestry_module(number_of_founders = 10,
                                             morgan = 1,
                                             markers = markers),
                    pop_size = 1000,
                    total_runtime = 11,
                    select_matrix = select_matrix)
require(ggplot2)
plot_difference_frequencies(results = selected_pop,
                            picked_ancestor = "ALL")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/junctions.R
\name{plot_dist_junctions}
\alias{plot_dist_junctions}
\title{plot the distribution of junctions}
\usage{
plot_dist_junctions(pop)
}
\arguments{
\item{pop}{of the class 'population'}
}
\value{
No return value
}
\description{
plots the distribution of junctions in the population using
base R
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_sequence.R
\name{simulate_sequence}
\alias{simulate_sequence}
\title{Individual based simulation of the breakdown of contiguous ancestry blocks.}
\usage{
simulate_sequence(
  input_data = NA,
  pop_size = NA,
  initial_frequencies = NA,
  total_runtime = 100,
  morgan = 1,
  recombination_rate = NA,
  num_threads = 1,
  select_matrix = NA,
  markers = NA,
  verbose = FALSE,
  multiplicative_selection = TRUE,
  mutation_rate = 0,
  substitution_matrix = matrix(1/4, 4, 4)
)
}
\arguments{
\item{input_data}{Genomic data used as input, should be of type
genomeadmixr_data. Either a single dataset is provided, or a list of
multiple genomeadmixr_data objects.}

\item{pop_size}{Vector containing the number of individuals in both
populations.}

\item{initial_frequencies}{A vector describing the initial contribution of
each provided input data set to the starting hybrid swarm. By default, equal
frequencies are assumed. If a vector not summing to 1 is provided, the vector
is normalized.}

\item{total_runtime}{Number of generations}

\item{morgan}{Length of the chromosome in Morgan (e.g. the number of
crossovers during meiosis)}

\item{recombination_rate}{rate in cM / Mbp, used to map recombination to the
markers. If the recombination_rate is not set, the value for Morgan is used,
assuming that the markers included span an entire chromosome.}

\item{num_threads}{number of threads. Default is 1. Set to -1 to use all
available threads}

\item{select_matrix}{Selection matrix indicating the markers which are under
selection. If not provided by the user, the simulation proceeds neutrally. If
provided, each row in the matrix should contain five entries:
\itemize{
\item{location of the marker under selection (in Morgan) }
\item{fitness of wildtype (aa)}
\item{fitness of heterozygote (aA)}
\item{fitness of homozygote mutant (AA)}
\item{Ancestral type that represents the mutant allele A}
}}

\item{markers}{A vector of locations of markers (relative locations in
[0, 1]). If a vector is provided, ancestry at these marker positions is
tracked for every generation.}

\item{verbose}{Verbose output if TRUE. Default value is FALSE}

\item{multiplicative_selection}{Default: TRUE. If TRUE, fitness is calculated
for multiple markers by multiplying fitness values for each marker. If FALSE,
fitness is calculated by adding fitness values for each marker.}

\item{mutation_rate}{the per base probability of mutation. Default is 0.}

\item{substitution_matrix}{a 4x4 matrix representing the probability of
mutating to another base (where [1/2/3/4] = [a/c/t/g]), conditional on the
event of a mutation happening. Default is the JC69 matrix, with equal
probabilities for all transitions / transversions.}
}
\value{
A list with: \code{population} a population object, and three tibbles
with allele frequencies (only contain values of a vector was provided to the
argument \code{markers}: \code{frequencies} , \code{initial_frequencies} and
\code{final_frequencies}. Each tibble contains four columns, \code{time},
\code{location}, \code{ancestor} and \code{frequency}, which indicates the
number of generations, the location along the chromosome of the marker, the
ancestral allele at that location in that generation, and finally, the
frequency of that allele.
}
\description{
Individual based simulation of the breakdown of contiguous
ancestry blocks, with or without selection. Simulations can be started from
scratch, or from a predefined input population.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_over_time.R
\name{plot_over_time}
\alias{plot_over_time}
\title{Plot the frequencies of all ancestors over time}
\usage{
plot_over_time(frequencies, focal_location)
}
\arguments{
\item{frequencies}{A tibble containing four columns: \code{time},
\code{location}, \code{ancestor}, \code{frequency}. A fifth colum
\code{population} can be included if the tibble is the result of
\code{simulate_admixture_migration}.}

\item{focal_location}{Location (in Morgan) where to plot the allele
frequencies.}
}
\value{
a ggplot2 object
}
\description{
This function plots the frequency of all ancestors over time at
a specific location on the chromosome, after performing a simulation.
}
\examples{
pop <- simulate_admixture(
           module = ancestry_module(number_of_founders = 10,
                                    markers = 0.5),
           pop_size = 1000,
           total_runtime = 11)
require(ggplot2)
plot_over_time(frequencies = pop$frequencies,
               focal_location = 0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_ancestry.R
\name{simulate_ancestry}
\alias{simulate_ancestry}
\title{Individual based simulation of the breakdown of contiguous ancestry blocks.}
\usage{
simulate_ancestry(
  input_population = NA,
  pop_size = NA,
  number_of_founders = 2,
  initial_frequencies = NA,
  total_runtime = 100,
  morgan = 1,
  num_threads = 1,
  select_matrix = NA,
  markers = NA,
  verbose = FALSE,
  track_junctions = FALSE,
  multiplicative_selection = TRUE
)
}
\arguments{
\item{input_population}{Potential earlier simulated population used as
starting point for the simulation. If not provided by the user, the
simulation starts from scratch.}

\item{pop_size}{The number of individuals in the population. If the number is
larger than the number of individuals in the input population (if provided),
additional individuals are sampled randomly from the input population to
reach the intended size.}

\item{number_of_founders}{Number of unique ancestors}

\item{initial_frequencies}{A vector describing the initial frequency of each
ancestor. By default, equal frequencies are assumed. If a vector not summing
to 1 is provided, the vector is normalized.}

\item{total_runtime}{Number of generations}

\item{morgan}{Length of the chromosome in Morgan (e.g. the number of
crossovers during meiosis)}

\item{num_threads}{number of threads. Default is 1. Set to -1 to use all
available threads}

\item{select_matrix}{Selection matrix indicating the markers which are under
selection. If not provided by the user, the simulation proceeds neutrally. If
provided, each row in the matrix should contain five entries:
\itemize{
\item{location of the marker under selection (in Morgan) }
\item{fitness of wildtype (aa)}
\item{fitness of heterozygote (aA)}
\item{fitness of homozygote mutant (AA)}
\item{Ancestral type that represents the mutant allele A}
}}

\item{markers}{A vector of locations of markers (relative locations in
[0, 1]). If a vector is provided, ancestry at these marker positions is
tracked for every generation.}

\item{verbose}{Verbose output if TRUE. Default value is FALSE}

\item{track_junctions}{Track the average number of junctions over time if
TRUE}

\item{multiplicative_selection}{Default: TRUE. If TRUE, fitness is calculated
for multiple markers by multiplying fitness values for each marker. If FALSE,
fitness is calculated by adding fitness values for each marker.}
}
\value{
A list with: \code{population} a population object, and three tibbles
with allele frequencies (only contain values of a vector was provided to the
argument \code{markers}: \code{frequencies} , \code{initial_frequencies} and
\code{final_frequencies}. Each tibble contains four columns, \code{time},
\code{location}, \code{ancestor} and \code{frequency}, which indicates the
number of generations, the location along the chromosome of the marker, the
ancestral allele at that location in that generation, and finally, the
frequency of that allele.
}
\description{
Individual based simulation of the breakdown of contiguous
ancestry blocks, with or without selection. Simulations can be started from
scratch, or from a predefined input population.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_load.R
\name{save_population}
\alias{save_population}
\title{Save a population to file}
\usage{
save_population(population, file_name, compression = TRUE)
}
\arguments{
\item{population}{Object of class \code{population}}

\item{file_name}{Name of the file to save the population}

\item{compression}{By default, the population is compressed to reduce file
size. See for more information \code{saveRDS}}
}
\value{
No return value
}
\description{
Saves a population to file for later use
}
\details{
This function functions as a wrapper for the base function
\code{saveRDS}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{print.population}
\alias{print.population}
\title{print a population object}
\usage{
\method{print}{population}(x, ...)
}
\arguments{
\item{x}{input population}

\item{...}{other arguments}
}
\value{
No return value
}
\description{
prints the contents of a population nicely
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ancestry_module.R
\name{ancestry_module}
\alias{ancestry_module}
\title{Creates a module to start simulations tracking local ancestry}
\usage{
ancestry_module(
  input_population = NA,
  number_of_founders = 2,
  initial_frequencies = NA,
  morgan = 1,
  markers = NA,
  track_junctions = FALSE
)
}
\arguments{
\item{input_population}{Potential earlier simulated population used as
starting point for the simulation. If not provided by the user, the
simulation starts from scratch.}

\item{number_of_founders}{Number of unique ancestors / ancestries to be
tracked in the simulation}

\item{initial_frequencies}{A vector describing the initial frequency of each
ancestor / ancestry. By default, equal frequencies are assumed. If a vector
not summing to 1 is provided, the vector is normalized.}

\item{morgan}{Length of the genomic stretch simulated, expressed in Morgan
(e.g. the number of crossovers during meiosis)}

\item{markers}{A vector of locations of markers, with the location in Morgan.
Ancestry at these marker positions is tracked for every generation.}

\item{track_junctions}{Tracks the average number of junctions over time if
TRUE}
}
\value{
list with type = "Ancestry". Can be used in simulate_admixture.
}
\description{
Module to perform simulations based on local ancestry
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_admixture.R
\name{simulate_admixture}
\alias{simulate_admixture}
\title{Individual based simulation of the breakdown of contiguous ancestry blocks.}
\usage{
simulate_admixture(
  module = ancestry_module(),
  pop_size = 100,
  total_runtime = 100,
  migration = migration_settings(),
  select_matrix = NA,
  multiplicative_selection = TRUE,
  verbose = FALSE,
  num_threads = 1
)
}
\arguments{
\item{module}{Chosen module to simulate, either created with
\code{module_ancestry} or
\code{module_sequence}.}

\item{pop_size}{The number of individuals in the population. If the number is
larger than the number of individuals in the input population (if provided),
additional individuals are sampled randomly from the input population to
reach the intended size.}

\item{total_runtime}{Number of generations}

\item{migration}{settings associated with migration, should be created with
\code{\link{migration_settings}}}

\item{select_matrix}{Selection matrix indicating the markers which are under
selection. If not provided by the user, the simulation proceeds neutrally. If
provided, each row in the matrix should contain five entries:
\itemize{
\item{location of the marker under selection (in Morgan) }
\item{fitness of wildtype (aa)}
\item{fitness of heterozygote (aA)}
\item{fitness of homozygote mutant (AA)}
\item{Ancestral type that represents the mutant allele A}
}}

\item{multiplicative_selection}{Default: TRUE. If TRUE, fitness is calculated
for multiple markers by multiplying fitness values for each marker. If FALSE,
fitness is calculated by adding fitness values for each marker.}

\item{verbose}{Verbose output if TRUE. Default value is FALSE}

\item{num_threads}{number of threads. Default is 1. Set to -1 to use all
available threads}
}
\value{
A list with: \code{population} a population object, and three tibbles
with allele frequencies (only contain values of a vector was provided to the
argument \code{markers}: \code{frequencies} , \code{initial_frequencies} and
\code{final_frequencies}. Each tibble contains four columns, \code{time},
\code{location}, \code{ancestor} and \code{frequency}, which indicates the
number of generations, the location along the chromosome of the marker, the
ancestral allele at that location in that generation, and finally, the
frequency of that allele.
}
\description{
Individual based simulation of the breakdown of contiguous
ancestry blocks, with or without selection. Simulations can be started from
scratch, or from a predefined input population.
}
\examples{
# local ancestry simulation
two_populations <- simulate_admixture(
                         module = ancestry_module(number_of_founders = 3,
                                                  morgan = 0.8),
                         migration = migration_settings(
                                         migration_rate = 0.01,
                                         population_size = c(100, 100)),
                         total_runtime = 10)
 # sequence simulation
 data(dgrp2.3R.5k.data)

sequence_population <-
      simulate_admixture(
                  module = sequence_module(molecular_data = dgrp2.3R.5k.data,
                           recombination_rate = 0.2,
                           mutation_rate = 1e-5),
                  pop_size = 1000,
                  total_runtime = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_input_data.R
\name{write_as_plink}
\alias{write_as_plink}
\title{function to write simulation output as PLINK style data}
\usage{
write_as_plink(
  input_pop,
  marker_locations,
  file_name_prefix,
  chromosome = 1,
  recombination_rate = 1
)
}
\arguments{
\item{input_pop}{input population, either of class "population" or of class
"genomeadmixr_data"}

\item{marker_locations}{location of markers, in bp}

\item{file_name_prefix}{prefix of the ped/map files.}

\item{chromosome}{chromosome indication for map file}

\item{recombination_rate}{recombination rate in cM / kb}
}
\value{
No return value
}
\description{
function to write simulation output as PLINK style data
}
